@@ -5,11 +5,9 @@ BIN=`dirname $0`
 
 LIB=$BIN/../lib
 
-LIBS=$LIB/log4jjar:$LIB/commons-logging.jar
+LIBS=$LIB/commons-logging.jar
 
-echo $CLASSPATH
-
-CLASSPATH=$BIN/../classes:$BIN/../conf:$CLASSPATH:$LIBS
+CLASSPATH=$BIN/../classes:$CLASSPATH:$LIBS
 
 # OS specific support (must be 'true' or 'false').
 cygwin=false;
@@ -25,12 +23,15 @@ else
    CP=$CLASSPATH
 fi
 
+#java -classpath $CP -Dbind.address=192.168.2.5 org.jgroups.demos.Draw -props /home/bela/udp.xml &
+#sleep 5
 
 count=0
 while [ $count -lt 20 ]
 do
   echo "Starting Draw instance #$count"
-  java -classpath $CP -Dbind.address=192.168.2.5 org.jgroups.demos.Draw -props $HOME/udp.xml &
-  # sleep 1
+  # change the IP address to your system
+  java -classpath $CP -Dbind.address=192.168.1.5 org.jgroups.demos.Draw -props /home/bela/udp.xml &
+  #  sleep 1
   count=$(($count+1))
 done
\ No newline at end of file
