@@ -1,4 +1,4 @@
-// $Id: Event.java,v 1.54.2.2 2008/07/30 11:51:45 belaban Exp $
+// $Id: Event.java,v 1.63 2008/07/30 11:45:52 belaban Exp $
 
 package org.jgroups;
 
@@ -10,7 +10,7 @@ package org.jgroups;
  */
 public class Event {
     public static final int MSG                                =  1;  // arg = Message
-    public static final int CONNECT                            =  2;  // arg = clster name (string)
+    public static final int CONNECT                            =  2;  // arg = cluster name (string)
     public static final int DISCONNECT                         =  4;  // arg = member address (Address)
     public static final int VIEW_CHANGE                        =  6;  // arg = View (or MergeView in case of merge)
     public static final int SET_LOCAL_ADDRESS                  =  8;  // arg = Address
@@ -28,8 +28,6 @@ public class Event {
     public static final int GET_DIGEST                         = 39;  //
     public static final int SET_DIGEST                         = 41;  // arg = Digest
     public static final int EXIT                               = 46;  // received when member was forced out of the group
-    public static final int PERF                               = 47;  // for performance measurements
-    public static final int HEARD_FROM                         = 50;  // arg = Vector (list of Addresses)
     public static final int UNSUSPECT                          = 51;  // arg = Address (of unsuspected member)
     public static final int MERGE_DIGEST                       = 53;  // arg = Digest
     public static final int CONFIG                             = 56;  // arg = Map<String,Object> (config properties)
@@ -50,8 +48,7 @@ public class Event {
     public static final int CONNECT_WITH_STATE_TRANSFER        = 80;  // arg = cluster name (string)
     public static final int DISABLE_UNICASTS_TO                = 81;  // arg = Address (member)
     public static final int START_PARTITION                    = 82;  // arg = null;
-    public static final int STOP_PARTITION                     = 83;  // arg = null;
-    public static final int INFO                               = 84;  // arg = Map<String,Object>
+    public static final int STOP_PARTITION                     = 83;  // arg = null; 
     public static final int PREPARE_VIEW                       = 86;  // arg = View
 
     public static final int USER_DEFINED                       = 1000; // arg = <user def., e.g. evt type + data>
@@ -115,8 +112,6 @@ public class Event {
             case SET_DIGEST:             return "SET_DIGEST";
             case MERGE:                  return "MERGE"; // Added by gianlucac@tin.it to support partitions merging in GMS
             case EXIT:                   return "EXIT";
-            case PERF:                   return "PERF";
-            case HEARD_FROM:             return "HEARD_FROM";
             case UNSUSPECT:              return "UNSUSPECT";
             case MERGE_DIGEST:           return "MERGE_DIGEST";
             case CONFIG:                 return "CONFIG";
@@ -137,8 +132,7 @@ public class Event {
             case CONNECT_WITH_STATE_TRANSFER:    return "CONNECT_WITH_STATE_TRANSFER";
             case DISABLE_UNICASTS_TO:    return "DISABLE_UNICASTS_TO";
             case START_PARTITION:        return "START_PARTITION";
-            case STOP_PARTITION:         return "STOP_PARTITION";
-            case INFO:                   return "INFO";
+            case STOP_PARTITION:         return "STOP_PARTITION";            
             case PREPARE_VIEW:           return "PREPARE_VIEW";
             case USER_DEFINED:           return "USER_DEFINED";
             default:                     return "UNDEFINED(" + t + ")";
