@@ -1,7 +1,11 @@
+
 package org.jgroups;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+import org.jgroups.annotations.MBean;
+import org.jgroups.annotations.ManagedAttribute;
+import org.jgroups.annotations.ManagedOperation;
 import org.jgroups.conf.ConfiguratorFactory;
 import org.jgroups.conf.ProtocolStackConfigurator;
 import org.jgroups.stack.IpAddress;
@@ -71,19 +75,15 @@ import java.util.concurrent.Exchanger;
  * the construction of the stack will be aborted.
  *
  * @author Bela Ban
- * @version $Id: JChannel.java,v 1.158.2.23 2008/11/12 08:08:23 vlada Exp $
+ * @version $Id: JChannel.java,v 1.209 2008/11/28 14:45:16 belaban Exp $
  */
+@MBean(description="JGroups channel")
 public class JChannel extends Channel {
 
-    /**
-     * The default protocol stack used by the default constructor.
-     */
+    /** The default protocol stack used by the default constructor  */
     public static final String DEFAULT_PROTOCOL_STACK="udp.xml";
 
-    static final String FORCE_PROPS="force.properties";
-
-    /* the protocol stack configuration string */
-    private String props=null;
+    protected String properties=null;
 
     /*the address of this JChannel instance*/
     private Address local_addr=null;
@@ -114,21 +114,30 @@ public class JChannel extends Channel {
     private static final long GET_STATE_DEFAULT_TIMEOUT=5000;
     /*if FLUSH is used channel waits for UNBLOCK event, this is the default timeout, 5 secs*/
     private static final long FLUSH_UNBLOCK_TIMEOUT=5000;
+    
     /*flag to indicate whether to receive blocks, if this is set to true, receive_views is set to true*/
+    @ManagedAttribute(description="Flag indicating whether to receive blocks",writable=true)
     private boolean receive_blocks=false;
+    
     /*flag to indicate whether to receive local messages
      *if this is set to false, the JChannel will not receive messages sent by itself*/
+    @ManagedAttribute(description="Flag indicating whether to receive this channel's own messages",writable=true)
     private boolean receive_local_msgs=true;
+    
     /*flag to indicate whether the channel will reconnect (reopen) when the exit message is received*/
+    @ManagedAttribute(description="Toggles whether the channel will reconnect after shun",writable=true)
     private boolean auto_reconnect=true;
+    
     /*flag t indicate whether the state is supposed to be retrieved after the channel is reconnected
      *setting this to true, automatically forces auto_reconnect to true*/
+    @ManagedAttribute(description="Toggles whether state should to be retrieved after reconnect",writable=true)
     private boolean auto_getstate=false;
+    
     /*channel connected flag*/
-    protected volatile boolean connected=false;
+    protected boolean connected=false;
 
     /*channel closed flag*/
-    protected volatile boolean closed=false;      // close() has been called, channel is unusable
+    protected boolean closed=false;      // close() has been called, channel is unusable
 
     /** True if a state transfer protocol is available, false otherwise */
     private boolean state_transfer_supported=false; // set by CONFIG event from STATE_TRANSFER protocol
@@ -142,11 +151,12 @@ public class JChannel extends Channel {
      */
     protected final Map<String,Object> additional_data=new HashMap<String,Object>();
     
-    protected final ConcurrentMap<String,Object> info=new ConcurrentHashMap<String,Object>();
+    protected final ConcurrentMap<String,Object> config=new ConcurrentHashMap<String,Object>();
 
     protected final Log log=LogFactory.getLog(getClass());
 
     /** Collect statistics */
+    @ManagedAttribute(description="Collect channel statistics",writable=true)
     protected boolean stats=true;
 
     protected long sent_msgs=0, received_msgs=0, sent_bytes=0, received_bytes=0;
@@ -279,6 +289,22 @@ public class JChannel extends Channel {
     }
 
 
+    /**
+     * Creates a channel with the same configuration as the channel passed to this constructor. This is used by
+     * testing code, and should not be used by any other code !
+     * @param ch
+     * @throws ChannelException
+     */
+    public JChannel(JChannel ch) throws ChannelException {
+        init(ch);
+        auto_reconnect=ch.auto_reconnect;
+        auto_getstate=ch.auto_getstate;
+        receive_blocks=ch.receive_blocks;
+        receive_local_msgs=ch.receive_local_msgs;
+        receive_blocks=ch.receive_blocks;
+    }
+
+ 
     /**
      * Returns the protocol stack.
      * Currently used by Debugger.
@@ -294,12 +320,14 @@ public class JChannel extends Channel {
     }
 
     /**
-     * returns the protocol stack configuration in string format.
-     * an example of this property is<BR>
+     * Returns the protocol stack configuration in string format. An example of this property is<BR>
      * "UDP:PING:FD:STABLE:NAKACK:UNICAST:FRAG:FLUSH:GMS:VIEW_ENFORCER:STATE_TRANSFER:QUEUE"
      */
     public String getProperties() {
-        return props;
+        String retval=prot_stack != null? prot_stack.printProtocolSpec(true) : null;
+        if(retval != null)
+            properties=retval;
+        return properties;
     }
 
     public boolean statsEnabled() {
@@ -310,19 +338,26 @@ public class JChannel extends Channel {
         this.stats=stats;
     }
 
+    @ManagedOperation
     public void resetStats() {
         sent_msgs=received_msgs=sent_bytes=received_bytes=0;
     }
 
+    @ManagedAttribute
     public long getSentMessages() {return sent_msgs;}
+    @ManagedAttribute
     public long getSentBytes() {return sent_bytes;}
+    @ManagedAttribute
     public long getReceivedMessages() {return received_msgs;}
+    @ManagedAttribute
     public long getReceivedBytes() {return received_bytes;}
+    @ManagedAttribute
     public int getNumberOfTasksInTimer() {
         TimeScheduler timer=getTimer();
         return timer != null? timer.size() : -1;
     }
 
+    @ManagedAttribute
     public int getTimerThreads() {
         TimeScheduler timer=getTimer();
         return timer != null? timer.getCorePoolSize() : -1;
@@ -337,6 +372,7 @@ public class JChannel extends Channel {
      * Returns a pretty-printed form of all the protocols. If include_properties
      * is set, the properties for each protocol will also be printed.
      */
+    @ManagedOperation
     public String printProtocolSpec(boolean include_properties) {
         ProtocolStack ps=getProtocolStack();
         return ps != null? ps.printProtocolSpec(include_properties) : null;
@@ -356,6 +392,7 @@ public class JChannel extends Channel {
      * @exception ChannelClosedException The channel is closed and therefore cannot be used any longer.
      *                                   A new channel has to be created first.
      */
+    @ManagedOperation(description="Connects the channel to a group")
     public synchronized void connect(String cluster_name) throws ChannelException {
         if(connected) {
             if(log.isTraceEnabled()) log.trace("already connected to " + cluster_name);
@@ -365,7 +402,7 @@ public class JChannel extends Channel {
         startStack(cluster_name);
 
         if(cluster_name != null) {    // only connect if we are not a unicast channel
-
+            
             Event connect_event=new Event(Event.CONNECT, cluster_name);
             Object res=downcall(connect_event);  // waits forever until connected (or channel is closed)
             if(res != null && res instanceof Exception) { // the JOIN was rejected by the coordinator
@@ -436,9 +473,7 @@ public class JChannel extends Channel {
 
             try {
                 Event connect_event=new Event(Event.CONNECT_WITH_STATE_TRANSFER, cluster_name);
-                Object res=downcall(connect_event); // waits forever until
-                // connected (or channel is
-                // closed)
+                Object res=downcall(connect_event); // waits forever until connected (or channel is closed)
                 joinSuccessful=!(res != null && res instanceof Exception);
                 if(!joinSuccessful) {
                     stopStack(true, false);
@@ -491,6 +526,7 @@ public class JChannel extends Channel {
      * <li> Notifies the listener, if the listener is available<BR>
      * </ol>
      */
+    @ManagedOperation(description="Disconnects the channel if it is connected")
     public synchronized void disconnect() {
         if(closed) return;
 
@@ -514,12 +550,13 @@ public class JChannel extends Channel {
      * After this method has been called, the channel us unusable.<BR>
      * This operation will disconnect the channel and close the channel receive queue immediately<BR>
      */
+    @ManagedOperation(description="Disconnects and destroys the channel")
     public synchronized void close() {
         _close(true, true); // by default disconnect before closing channel and close mq
     }
 
-
-    /** Shuts down the channel without disconnecting */
+    
+    @ManagedOperation(description="Shuts down the channel without disconnecting")
     public synchronized void shutdown() {
         down(new Event(Event.SHUTDOWN));
         _close(false, true); // by default disconnect before closing channel and close mq
@@ -541,6 +578,8 @@ public class JChannel extends Channel {
         try {
             mq.reset();
 
+            String props=getProperties();
+
             // new stack is created on open() - bela June 12 2003
             prot_stack=new ProtocolStack(this, props);
             prot_stack.setup();
@@ -554,6 +593,7 @@ public class JChannel extends Channel {
     /**
      * returns true if the Open operation has been called successfully
      */
+    @ManagedAttribute
     public boolean isOpen() {
         return !closed;
     }
@@ -562,15 +602,17 @@ public class JChannel extends Channel {
     /**
      * returns true if the Connect operation has been called successfully
      */
+    @ManagedAttribute
     public boolean isConnected() {
         return connected;
     }
 
+    @ManagedAttribute
     public int getNumMessages() {
-        return mq != null? mq.size() : -1;
+        return mq.size();
     }
 
-
+    @ManagedOperation
     public String dumpQueue() {
         return Util.dumpQueue(mq);
     }
@@ -580,6 +622,7 @@ public class JChannel extends Channel {
      * @return Map<String,Map>. A map where the keys are the protocols ("channel" pseudo key is
      * used for the channel itself") and the values are property maps.
      */
+    @ManagedOperation
     public Map<String,Object> dumpStats() {
         Map<String,Object> retval=prot_stack.dumpStats();
         if(retval != null) {
@@ -590,6 +633,7 @@ public class JChannel extends Channel {
         return retval;
     }
 
+    @ManagedOperation
     public Map<String,Object> dumpStats(String protocol_name) {
         return prot_stack.dumpStats(protocol_name);
     }
@@ -613,6 +657,7 @@ public class JChannel extends Channel {
      * @exception ChannelNotConnectedException
      * @exception ChannelClosedException
      */
+    @ManagedOperation
     public void send(Message msg) throws ChannelNotConnectedException, ChannelClosedException {
         checkClosedOrNotConnected();
         if(msg == null)
@@ -636,6 +681,7 @@ public class JChannel extends Channel {
      * @exception ChannelClosedException
      * @see JChannel#send
      */
+    @ManagedOperation
     public void send(Address dst, Address src, Serializable obj) throws ChannelNotConnectedException, ChannelClosedException {
         send(new Message(dst, src, obj));
     }
@@ -721,7 +767,17 @@ public class JChannel extends Channel {
     public View getView() {
         return closed || !connected ? null : my_view;
     }
-
+    
+    @ManagedAttribute(name="View")
+    public String getViewAsString() {
+        View v=getView();
+        return v != null ? v.toString() : "n/a";
+    }
+    
+    @ManagedAttribute
+    public static String getVersion() {
+        return Version.printDescription();
+    }  
 
     /**
      * returns the local address of the channel
@@ -731,11 +787,11 @@ public class JChannel extends Channel {
         return closed ? null : local_addr;
     }
 
+    @ManagedAttribute(name="LocalAddress")
     public String getLocalAddressAsString() {
         return local_addr != null? local_addr.toString() : "n/a";
     }
 
-
     /**
      * returns the name of the channel
      * if the channel is not connected or if it is closed it will return null
@@ -745,6 +801,7 @@ public class JChannel extends Channel {
         return closed ? null : !connected ? null : cluster_name;
     }
 
+    @ManagedAttribute(description="Returns cluster name this channel is connected to")
     public String getClusterName() {
         return closed ? null : !connected ? null : cluster_name;
     }
@@ -899,9 +956,6 @@ public class JChannel extends Channel {
      * 
      * @param target
      *                State provider. If null, coordinator is used
-     * @param state_id
-     *                The ID of the substate. If null, the entire state will be
-     *                transferred
      * @param timeout
      *                the number of milliseconds to wait for the operation to
      *                complete successfully. 0 waits until the state has been
@@ -1020,7 +1074,7 @@ public class JChannel extends Channel {
      * @throws IllegalStateException
      *                 if flush is used in this channel and cluster could not be
      *                 flushed
-     */
+     */    
     public boolean getState(Address target, String state_id, long timeout,boolean useFlushIfPresent) throws ChannelNotConnectedException, ChannelClosedException {
         checkClosedOrNotConnected();
         if(!state_transfer_supported) {
@@ -1035,7 +1089,7 @@ public class JChannel extends Channel {
                 log.trace("cannot get state from myself (" + target + "): probably the first member");
             return false;
         }
-        
+              
         boolean initiateFlush = flushSupported() && useFlushIfPresent;
         
         if(initiateFlush){
@@ -1127,145 +1181,143 @@ public class JChannel extends Channel {
 
         switch(type) {
 
-        case Event.MSG:
-            msg=(Message)evt.getArg();
-            if(stats) {
-                received_msgs++;
-                received_bytes+=msg.getLength();
-            }
+            case Event.MSG:
+                msg=(Message)evt.getArg();
+                if(stats) {
+                    received_msgs++;
+                    received_bytes+=msg.getLength();
+                }
 
-            if(!receive_local_msgs) {  // discard local messages (sent by myself to me)
-                if(local_addr != null && msg.getSrc() != null)
-                    if(local_addr.equals(msg.getSrc()))
-                        return null;
-            }
-            break;
+                if(!receive_local_msgs) {  // discard local messages (sent by myself to me)
+                    if(local_addr != null && msg.getSrc() != null)
+                        if(local_addr.equals(msg.getSrc()))
+                            return null;
+                }
+                break;
 
-        case Event.VIEW_CHANGE:
-            View tmp=(View)evt.getArg();
-            if(tmp instanceof MergeView)
-                my_view=new View(tmp.getVid(), tmp.getMembers());
-            else
-                my_view=tmp;
+            case Event.VIEW_CHANGE:
+                View tmp=(View)evt.getArg();
+                if(tmp instanceof MergeView)
+                    my_view=new View(tmp.getVid(), tmp.getMembers());
+                else
+                    my_view=tmp;
 
-            /*
+                /*
              * Bela&Vladimir Oct 27th,2006 (JGroups 2.4)- we need to switch to
              * connected=true because client can invoke channel.getView() in
              * viewAccepted() callback invoked on this thread
              * (see Event.VIEW_CHANGE handling below)
              */
 
-            // not good: we are only connected when we returned from connect() - bela June 22 2007
-            //            if(connected == false) {
-            //                connected=true;
-            //            }
-            break;
+                // not good: we are only connected when we returned from connect() - bela June 22 2007
+                //            if(connected == false) {
+                //                connected=true;
+                //            }
+                break;
 
-        case Event.CONFIG:
-            Map<String,Object> config=(Map<String,Object>)evt.getArg();
-            if(config != null) {
-                if(config.containsKey("state_transfer")) {
-                    state_transfer_supported=((Boolean)config.get("state_transfer")).booleanValue();
+            case Event.CONFIG:
+                Map<String,Object> config=(Map<String,Object>)evt.getArg();
+                if(config != null) {
+                    if(config.containsKey("state_transfer")) {
+                        state_transfer_supported=((Boolean)config.get("state_transfer")).booleanValue();
+                    }
+                    if(config.containsKey("flush_supported")) {
+                        flush_supported=((Boolean)config.get("flush_supported")).booleanValue();
+                    }
+                    config.putAll(config);
                 }
-                if(config.containsKey("flush_supported")) {
-                    flush_supported=((Boolean)config.get("flush_supported")).booleanValue();
+                break;
+                
+            case Event.GET_STATE_OK:
+                StateTransferInfo state_info=(StateTransferInfo)evt.getArg();
+                byte[] state=state_info.state;
+
+                try {
+                    if(up_handler != null) {
+                        return up_handler.up(evt);
+                    }
+
+                    if(state != null) {
+                        String state_id=state_info.state_id;
+                        if(receiver != null) {
+                            try {
+                                if(receiver instanceof ExtendedReceiver && state_id != null)
+                                    ((ExtendedReceiver)receiver).setState(state_id, state);
+                                else
+                                    receiver.setState(state);
+                            }
+                            catch(Throwable t) {
+                                if(log.isWarnEnabled())
+                                    log.warn("failed calling setState() in receiver", t);
+                            }
+                        }
+                        else {
+                            try {
+                                mq.add(new Event(Event.STATE_RECEIVED, state_info));
+                            }
+                            catch(Exception e) {
+                            }
+                        }
+                    }
                 }
-            }
-            break;
-            
-        case Event.INFO:
-           Map<String, Object> m = (Map<String, Object>) evt.getArg();
-           info.putAll(m);            
-           break;    
+                finally {
+                    state_promise.setResult(state != null? Boolean.TRUE : Boolean.FALSE);
+                }
+                break;
+            case Event.STATE_TRANSFER_INPUTSTREAM_CLOSED:
+                state_promise.setResult(Boolean.TRUE);
+                break;
 
-        case Event.GET_STATE_OK:
-            StateTransferInfo state_info = (StateTransferInfo) evt.getArg();
-            byte[] state = state_info.state;
+            case Event.STATE_TRANSFER_INPUTSTREAM:
+                StateTransferInfo sti=(StateTransferInfo)evt.getArg();
+                InputStream is=sti.inputStream;
+                //Oct 13,2006 moved to down() when Event.STATE_TRANSFER_INPUTSTREAM_CLOSED is received
+                //state_promise.setResult(is != null? Boolean.TRUE : Boolean.FALSE);
 
-            try{
-                if(up_handler != null){
+                if(up_handler != null) {
                     return up_handler.up(evt);
                 }
 
-                if(state != null){
-                    String state_id = state_info.state_id;
-                    if(receiver != null){
-                        try{
-                            if(receiver instanceof ExtendedReceiver && state_id != null)
-                                ((ExtendedReceiver) receiver).setState(state_id, state);
+                if(is != null) {
+                    if(receiver instanceof ExtendedReceiver) {
+                        try {
+                            if(sti.state_id == null)
+                                ((ExtendedReceiver)receiver).setState(is);
                             else
-                                receiver.setState(state);
-                        }catch(Throwable t){
+                                ((ExtendedReceiver)receiver).setState(sti.state_id, is);
+                        }
+                        catch(Throwable t) {
                             if(log.isWarnEnabled())
                                 log.warn("failed calling setState() in receiver", t);
                         }
-                    }else{
-                        try{
-                            mq.add(new Event(Event.STATE_RECEIVED, state_info));
+                    }
+                    else if(receiver instanceof Receiver){
+                        if(log.isWarnEnabled()){
+                            log.warn("Channel has STREAMING_STATE_TRANSFER, however," +
+                                    " application does not implement ExtendedMessageListener. State is not transfered");
+                            Util.close(is);
+                        }
+                    }
+                    else {
+                        try {
+                            mq.add(new Event(Event.STATE_TRANSFER_INPUTSTREAM, sti));
                         }
                         catch(Exception e) {
+                        }
                     }
                 }
-                }
-            }
-            finally {
-                state_promise.setResult(state != null ? Boolean.TRUE : Boolean.FALSE);
-            }
-            break;
-        case Event.STATE_TRANSFER_INPUTSTREAM_CLOSED:
-            state_promise.setResult(Boolean.TRUE);
-            break;     
+                break;
 
-        case Event.STATE_TRANSFER_INPUTSTREAM:
-            StateTransferInfo sti=(StateTransferInfo)evt.getArg();
-            InputStream is=sti.inputStream;
-            //Oct 13,2006 moved to down() when Event.STATE_TRANSFER_INPUTSTREAM_CLOSED is received
-            //state_promise.setResult(is != null? Boolean.TRUE : Boolean.FALSE);
+            case Event.SET_LOCAL_ADDRESS:
+                local_addr_promise.setResult((Address)evt.getArg());
+                break;
 
-            if(up_handler != null) {
-                return up_handler.up(evt);
-            }
+            case Event.EXIT:
+                handleExit(evt);
+                return null;  // no need to pass event up; already done in handleExit()
 
-            if(is != null) {
-                if(receiver instanceof ExtendedReceiver) {
-                    try {
-                        if(sti.state_id == null)
-                            ((ExtendedReceiver)receiver).setState(is);
-                        else
-                            ((ExtendedReceiver)receiver).setState(sti.state_id, is);
-                    }
-                    catch(Throwable t) {
-                        if(log.isWarnEnabled())
-                            log.warn("failed calling setState() in receiver", t);
-                    }
-                }
-                else if(receiver instanceof Receiver){
-                    if(log.isWarnEnabled()){
-                        log.warn("Channel has STREAMING_STATE_TRANSFER, however," +
-                                " application does not implement ExtendedMessageListener. State is not transfered");
-                        Util.close(is);
-                    }
-                }
-                else {
-                    try {
-                        mq.add(new Event(Event.STATE_TRANSFER_INPUTSTREAM, sti));
-                    }
-                    catch(Exception e) {
-                    }
-                }
-            }
-            break;
-
-        case Event.SET_LOCAL_ADDRESS:
-            local_addr_promise.setResult((Address)evt.getArg());
-            break;
-
-        case Event.EXIT:
-            handleExit(evt);
-            return null;  // no need to pass event up; already done in handleExit()
-
-        default:
-            break;
+            default:
+                break;
         }
 
 
@@ -1274,7 +1326,7 @@ public class JChannel extends Channel {
             Object ret=up_handler.up(evt);
 
             if(type == Event.UNBLOCK){
-               flush_unblock_promise.setResult(Boolean.TRUE);
+                flush_unblock_promise.setResult(Boolean.TRUE);
             }
             return ret;
         }
@@ -1337,7 +1389,7 @@ public class JChannel extends Channel {
                 }
                 break;
             case Event.STATE_TRANSFER_OUTPUTSTREAM:
-        	StateTransferInfo sti=(StateTransferInfo)evt.getArg();
+                StateTransferInfo sti=(StateTransferInfo)evt.getArg();
                 OutputStream os=sti.outputStream;
                 if(receiver instanceof ExtendedReceiver) {                    
                     if(os != null) {
@@ -1480,6 +1532,7 @@ public class JChannel extends Channel {
 
 
 
+    @ManagedOperation
     public String toString(boolean details) {
         StringBuilder sb=new StringBuilder();
         sb.append("local_addr=").append(local_addr).append('\n');
@@ -1487,15 +1540,14 @@ public class JChannel extends Channel {
         sb.append("my_view=").append(my_view).append('\n');
         sb.append("connected=").append(connected).append('\n');
         sb.append("closed=").append(closed).append('\n');
-        if(mq != null)
-            sb.append("incoming queue size=").append(mq.size()).append('\n');
+        sb.append("incoming queue size=").append(mq.size()).append('\n');
         if(details) {
             sb.append("receive_blocks=").append(receive_blocks).append('\n');
             sb.append("receive_local_msgs=").append(receive_local_msgs).append('\n');
             sb.append("auto_reconnect=").append(auto_reconnect).append('\n');
             sb.append("auto_getstate=").append(auto_getstate).append('\n');
             sb.append("state_transfer_supported=").append(state_transfer_supported).append('\n');
-            sb.append("props=").append(props).append('\n');
+            sb.append("props=").append(getProperties()).append('\n');
         }
 
         return sb.toString();
@@ -1508,12 +1560,30 @@ public class JChannel extends Channel {
     protected final void init(ProtocolStackConfigurator configurator) throws ChannelException {
         if(log.isInfoEnabled())
             log.info("JGroups version: " + Version.description);
+        
         // ConfiguratorFactory.substituteVariables(configurator); // replace vars with system props
-        props=configurator.getProtocolStackString();
-        props=Util.substituteVariable(props);
-        prot_stack=new ProtocolStack(this, props);
+        String tmp=configurator.getProtocolStackString();
+        tmp=Util.substituteVariable(tmp); // replace vars with system props
+
+        prot_stack=new ProtocolStack(this, tmp);
         try {
             prot_stack.setup(); // Setup protocol stack (creates protocol, calls init() on them)
+            properties=tmp;
+        }
+        catch(Throwable e) {
+            throw new ChannelException("unable to setup the protocol stack: " + e.getMessage(), e);
+        }
+    }
+
+    protected final void init(JChannel ch) throws ChannelException {
+        if(ch == null)
+            throw new IllegalArgumentException("channel is null");
+        if(log.isInfoEnabled())
+            log.info("JGroups version: " + Version.description);
+        prot_stack=new ProtocolStack(this, null);
+        try {
+            prot_stack.setup(ch.getProtocolStack()); // Setup protocol stack (creates protocol, calls init() on them)
+            getProperties();
         }
         catch(Throwable e) {
             throw new ChannelException("unable to setup the protocol stack: " + e.getMessage(), e);
@@ -1677,10 +1747,10 @@ public class JChannel extends Channel {
         init(); // sets local_addr=null; changed March 18 2003 (bela) -- prevented successful rejoining
     }
 
-    protected void stopStack(boolean disconnect, boolean destroy) {
+    protected void stopStack(boolean stop, boolean destroy) {
         if(prot_stack != null) {
             try {
-                if(disconnect)
+                if(stop)
                     prot_stack.stopStack(cluster_name);
 
                 if(destroy)
@@ -1699,8 +1769,7 @@ public class JChannel extends Channel {
 
 
     public final void closeMessageQueue(boolean flush_entries) {
-        if(mq != null)
-            mq.close(flush_entries);
+        mq.close(flush_entries);
     }
 
 
@@ -1835,12 +1904,12 @@ public class JChannel extends Channel {
     
     @Override
     public Map<String, Object> getInfo(){
-       return new HashMap<String, Object>(info);
+       return new HashMap<String, Object>(config);
     }
 
     public void setInfo(String key, Object value) {
         if(key != null)
-            info.put(key, value);
+            config.put(key, value);
     }
 
     Address determineCoordinator() {
@@ -1848,7 +1917,7 @@ public class JChannel extends Channel {
         if(mbrs == null)
             return null;
         if(!mbrs.isEmpty())
-            return (Address)mbrs.firstElement();
+            return mbrs.firstElement();
         return null;
     }
 
@@ -1903,13 +1972,11 @@ public class JChannel extends Channel {
     }
 
     class CloserThread extends Thread {
-        final Event evt;
-        final Thread t=null;
-
+        final Event exitEvent;              
 
         CloserThread(Event evt) {
             super(Util.getGlobalThreadGroup(), "CloserThread");
-            this.evt=evt;
+            this.exitEvent=evt;            
             setDaemon(true);
         }
 
@@ -1918,66 +1985,78 @@ public class JChannel extends Channel {
             try {
                 String old_cluster_name=cluster_name; // remember because close() will null it
                 if(log.isDebugEnabled())
-                    log.debug("closing the channel");
-                _close(false, false); // do not disconnect before closing channel, do not close mq (yet !)
-
-                if(up_handler != null)
-                    up_handler.up(this.evt);
-                else {
-                    try {
-                        if(receiver == null)
-                            mq.add(this.evt);
+                    log.debug("CloserThread - closing the channel " + local_addr);
+                
+                //consider closing and connecting as one operation to prevent 
+                //other threads from intervleaving 
+                synchronized(JChannel.this){
+                    _close(false, false); // do not disconnect before closing channel, do not close mq (yet !)
+    
+                    if(up_handler != null)
+                        up_handler.up(exitEvent);
+                    else {
+                        try {
+                            if(receiver == null)
+                                mq.add(exitEvent);
+                        }
+                        catch(Exception ex) {
+                            if(log.isErrorEnabled()) log.error(ex);
+                        }
                     }
-                    catch(Exception ex) {
-                        if(log.isErrorEnabled()) log.error("exception: " + ex);
-                    }
-                }
-
-                if(mq != null) {
+    
+                    
                     Util.sleep(500); // give the mq thread a bit of time to deliver EXIT to the application
                     try {
                         mq.close(false);
                     }
                     catch(Exception ex) {
-                    }
-                }
-
-                if(auto_reconnect) {
-                    try {
-                        if(log.isDebugEnabled()) log.debug("reconnecting to group " + old_cluster_name);
-                        open();
-                        if(additional_data != null) {
-                            // send previously set additional_data down the stack - other protocols (e.g. TP) use it
-                            Map<String,Object> m=new HashMap<String,Object>(additional_data);
-                            down(new Event(Event.CONFIG, m));
-                        }
-                    }
-                    catch(Exception ex) {
-                        if(log.isErrorEnabled()) log.error("failure reopening channel: " + ex);
-                        return;
-                    }
-
-                    while(!connected) {
+                    }                
+    
+                    if(auto_reconnect) {
                         try {
-                            connect(old_cluster_name);
-                            notifyChannelReconnected(local_addr);
+                            if(log.isDebugEnabled()) log.debug("CloserThread - reconnecting to group " + old_cluster_name);
+                            open();
+                            if(additional_data != null) {
+                                // send previously set additional_data down the stack - other protocols (e.g. TP) use it
+                                Map<String,Object> m=new HashMap<String,Object>(additional_data);
+                                down(new Event(Event.CONFIG, m));
+                            }
                         }
                         catch(Exception ex) {
-                            if(log.isErrorEnabled()) log.error("failure reconnecting to channel, retrying", ex);
-                            Util.sleep(1000); // sleep 1 sec between reconnect attempts
+                            if(log.isErrorEnabled()) log.error(ex);
+                            return;
+                        }
+    
+                        while(!connected) {
+                            try {
+                                connect(old_cluster_name);
+                                notifyChannelReconnected(local_addr);
+                            }
+                            catch(Exception ex) {
+                                if(log.isErrorEnabled()) log.error("CloserThread - failure reconnecting to channel, retrying", ex);                               
+                                //release the lock on channel so other threads can potentially close or shutdown
+                                JChannel.this.wait(1000);
+                                
+                                if(!isOpen()){
+                                    //other thread closed the channel in the meantime, give up from reconnecting
+                                    break;
+                                }
+                            }
                         }
                     }
                 }
 
-                if(auto_getstate && state_transfer_supported) {
+                boolean canFetchState = auto_getstate && state_transfer_supported && connected;
+                if(canFetchState) {
                     if(log.isDebugEnabled())
                         log.debug("fetching the state (auto_getstate=true)");
+                    
                     boolean rc=JChannel.this.getState(null, GET_STATE_DEFAULT_TIMEOUT);
                     if(log.isDebugEnabled()) {
                         if(rc)
-                            log.debug("state was retrieved successfully");
+                            log.debug("CloserThread - state was retrieved successfully");
                         else
-                            log.debug("state transfer failed");
+                            log.debug("CloserThread - state transfer failed");
                     }
                 }
 
@@ -1987,8 +2066,7 @@ public class JChannel extends Channel {
             }
             finally {
                 closer=null;
-            }
+            }              
         }
     }
-
 }
