@@ -6,13 +6,15 @@ import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.jgroups.conf.ClassConfigurator;
 import org.jgroups.stack.IpAddress;
-import org.jgroups.util.*;
+import org.jgroups.util.Buffer;
+import org.jgroups.util.Headers;
+import org.jgroups.util.Streamable;
+import org.jgroups.util.Util;
 
 import java.io.*;
 import java.util.HashSet;
 import java.util.Map;
 import java.util.Set;
-import java.util.concurrent.ConcurrentHashMap;
 
 
 /**
@@ -24,9 +26,9 @@ import java.util.concurrent.ConcurrentHashMap;
  * The byte buffer can point to a reference, and we can subset it using index and length. However,
  * when the message is serialized, we only write the bytes between index and length.
  * @author Bela Ban
- * @version $Id: Message.java,v 1.76.2.9 2008/09/16 13:09:12 belaban Exp $
+ * @version $Id: Message.java,v 1.94 2008/11/11 09:30:08 belaban Exp $
  */
-public class Message implements Externalizable, Streamable {
+public class Message implements Streamable {
     protected Address dest_addr=null;
     protected Address src_addr=null;
 
@@ -34,51 +36,38 @@ public class Message implements Externalizable, Streamable {
     private byte[]    buf=null;
 
     /** The index into the payload (usually 0) */
-    protected transient int     offset=0;
+    protected int     offset=0;
 
     /** The number of bytes in the buffer (usually buf.length is buf not equal to null). */
-    protected transient int     length=0;
+    protected int     length=0;
 
     /** All headers are placed here */
     protected Headers headers;
 
+    private byte      flags;
+
     protected static final Log log=LogFactory.getLog(Message.class);
 
-    private static final long serialVersionUID=7966206671974139740L;
+    static final Set<Class>    nonStreamableHeaders=new HashSet<Class>();
 
-    static final byte DEST_SET      = 1;
-    static final byte SRC_SET       = 2;
-    static final byte BUF_SET       = 4;
+
+    static final byte DEST_SET         =  1;
+    static final byte SRC_SET          =  2;
+    static final byte BUF_SET          =  4;
     // static final byte HDRS_SET=8; // bela July 15 2005: not needed, we always create headers
-    static final byte IPADDR_DEST   =16;
-    static final byte IPADDR_SRC    =32;
-    static final byte SRC_HOST_NULL =64;
+    static final byte IPADDR_DEST      = 16;
+    static final byte IPADDR_SRC       = 32;
+    static final byte SRC_HOST_NULL    = 64;
 
 
     // =========================== Flags ==============================
-    public static final byte OOB       = 1;
-    public static final byte LOW_PRIO  = 2; // not yet sure if we want this flag...
-    public static final byte HIGH_PRIO = 4; // not yet sure if we want this flag...
+    public static final byte OOB               = 1;
+    public static final byte LOW_PRIO          = 2; // not yet sure if we want this flag...
+    public static final byte HIGH_PRIO         = 4; // not yet sure if we want this flag...
 
-    private byte flags=0;
 
-    static final Set<Class> nonStreamableHeaders=new HashSet<Class>();
 
-    /** Map<Address,Address>. Maintains mappings to canonical addresses */
-    private static final ConcurrentHashMap<Address,Address> canonicalAddresses=new ConcurrentHashMap<Address,Address>();
-    private static final boolean DISABLE_CANONICALIZATION;
 
-    static {
-        boolean b;
-        try {
-            b=Boolean.getBoolean("disable_canonicalization");
-        }
-        catch (java.security.AccessControlException e) {
-            // this will happen in an applet context
-            b=true;
-        }
-        DISABLE_CANONICALIZATION=b;
-    }
 
 
     /** Public constructor
@@ -160,10 +149,7 @@ public class Message implements Externalizable, Streamable {
     }
 
     public void setDest(Address new_dest) {
-        if(DISABLE_CANONICALIZATION)
-            dest_addr=new_dest;
-        else
-            dest_addr=canonicalAddress(new_dest);
+        dest_addr=new_dest;
     }
 
     public Address getSrc() {
@@ -171,10 +157,7 @@ public class Message implements Externalizable, Streamable {
     }
 
     public void setSrc(Address new_src) {
-        if(DISABLE_CANONICALIZATION)
-            src_addr=new_src;
-        else
-            src_addr=canonicalAddress(new_src);
+        src_addr=new_src;
     }
 
     /**
@@ -250,7 +233,6 @@ public class Message implements Externalizable, Streamable {
         }
     }
 
-
     /** Returns the offset into the buffer at which the data starts */
     public int getOffset() {
         return offset;
@@ -490,89 +472,6 @@ public class Message implements Externalizable, Streamable {
 
 
 
-    /* ----------------------------------- Interface Externalizable ------------------------------- */
-
-    public void writeExternal(ObjectOutput out) throws IOException {
-        int             len;
-        Externalizable  hdr;
-        Map.Entry       entry;
-
-        if(dest_addr != null) {
-            out.writeBoolean(true);
-            Marshaller.write(dest_addr, out);
-        }
-        else {
-            out.writeBoolean(false);
-        }
-
-        if(src_addr != null) {
-            out.writeBoolean(true);
-            Marshaller.write(src_addr, out);
-        }
-        else {
-            out.writeBoolean(false);
-        }
-
-        out.write(flags);
-
-        if(buf == null)
-            out.writeInt(0);
-        else {
-            out.writeInt(length);
-            out.write(buf, offset, length);
-        }
-
-        len=headers.size();
-        out.writeInt(len);
-        final Object[] data=headers.getRawData();
-
-        for(int i=0; i < data.length; i+=2) {
-            if(data[i] != null) {
-                out.writeUTF((String)data[i]);
-                hdr=(Externalizable)data[i+1];
-                Marshaller.write(hdr, out);
-            }
-        }
-    }
-
-
-    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
-        boolean  destAddressExist=in.readBoolean();
-
-        if(destAddressExist) {
-            dest_addr=(Address)Marshaller.read(in);
-            if(!DISABLE_CANONICALIZATION)
-                dest_addr=canonicalAddress(dest_addr);
-        }
-
-        boolean srcAddressExist=in.readBoolean();
-        if(srcAddressExist) {
-            src_addr=(Address)Marshaller.read(in);
-            if(!DISABLE_CANONICALIZATION)
-                src_addr=canonicalAddress(src_addr);
-        }
-
-        flags=in.readByte();
-
-        int i=in.readInt();
-        if(i != 0) {
-            buf=new byte[i];
-            in.readFully(buf);
-            offset=0;
-            length=buf.length;
-        }
-
-        int len=in.readInt();
-        while(len-- > 0) {
-            String key=in.readUTF();
-            Header value=(Header)Marshaller.read(in);
-            headers.putHeader(key, value);
-        }
-    }
-
-    /* --------------------------------- End of Interface Externalizable ----------------------------- */
-
-
     /* ----------------------------------- Interface Streamable  ------------------------------- */
 
     /**
@@ -650,8 +549,6 @@ public class Message implements Externalizable, Streamable {
             else {
                 src_addr=Util.readAddress(in);
             }
-            if(!DISABLE_CANONICALIZATION)
-                src_addr=canonicalAddress(src_addr);
         }
 
         // 3. buf
@@ -688,7 +585,7 @@ public class Message implements Externalizable, Streamable {
         StringBuilder sb=new StringBuilder();
         boolean first=true;
         if(isFlagSet(OOB)) {
-                first=false;
+            first=false;
             sb.append("OOB");
         }
         if(isFlagSet(LOW_PRIO)) {
@@ -714,7 +611,7 @@ public class Message implements Externalizable, Streamable {
         ObjectOutputStream oos=null;
         int size=value.size();
         try {
-            magic_number=ClassConfigurator.getInstance(false).getMagicNumber(value.getClass());
+            magic_number=ClassConfigurator.getMagicNumber(value.getClass());
             // write the magic number or the class name
             out.writeShort(magic_number);
             if(magic_number == -1) {
@@ -741,11 +638,6 @@ public class Message implements Externalizable, Streamable {
                 }
             }
         }
-        catch(ChannelException e) {
-            IOException io_ex=new IOException("failed writing header");
-            io_ex.initCause(e);
-            throw io_ex;
-        }
         finally {
             if(oos != null)
                 oos.close(); // this is a no-op on ByteArrayOutputStream
@@ -763,13 +655,13 @@ public class Message implements Externalizable, Streamable {
         try {
             magic_number=in.readShort();
             if(magic_number != -1) {
-                clazz=ClassConfigurator.getInstance(false).get(magic_number);
+                clazz=ClassConfigurator.get(magic_number);
                 if(clazz == null)
                     throw new IllegalArgumentException("magic number " + magic_number + " is not available in magic map");
             }
             else {
                 classname=in.readUTF();
-                clazz=ClassConfigurator.getInstance(false).get(classname);
+                clazz=ClassConfigurator.get(classname);
             }
 
             in.readShort(); // we discard the size since we don't use it
@@ -800,27 +692,6 @@ public class Message implements Externalizable, Streamable {
         return new Headers(m);
     }
 
-    /** canonicalize addresses to some extent.  There are race conditions
-     * allowed in this method, so it may not fully canonicalize an address
-     * @param nonCanonicalAddress
-     * @return canonical representation of the address
-     */
-    private static Address canonicalAddress(Address nonCanonicalAddress) {
-        Address result=null;
-        if(nonCanonicalAddress == null) {
-            return null;
-        }
-        // do not synchronize between get/put on the canonical map to avoid cost of contention
-        // this can allow multiple equivalent addresses to leak out, but it's worth the cost savings
-        try {
-            result=canonicalAddresses.putIfAbsent(nonCanonicalAddress, nonCanonicalAddress);
-            return result != null? result : nonCanonicalAddress;
-        }
-        catch(NullPointerException npe) {
-            // no action needed
-        }
-        return result;
-    }
 
     /* ------------------------------- End of Private methods ---------------------------- */
 
