@@ -15,19 +15,19 @@ import org.jgroups.annotations.Immutable;
  * Z = 0-63 for micro versions
  *
  * @author Bela Ban
- * @version $Id: Version.java,v 1.59.2.13 2008/11/12 11:15:26 belaban Exp $
+ * @version $Id: Version.java,v 1.66 2008/12/05 11:37:45 belaban Exp $
  * Holds version information for JGroups.
  */
 @Immutable
 public class Version {
     public static final short major = 2;
-    public static final short minor = 6;
-    public static final short micro = 7;
-    public static final String description="2.6.7.GA";
+    public static final short minor = 7;
+    public static final short micro = 0;
+    public static final String description="2.7.0.GA";
 
     public static final short version=encode(major, minor, micro);
     public static final String string_version=print(version);
-    public static final String cvs="$Id: Version.java,v 1.59.2.13 2008/11/12 11:15:26 belaban Exp $";
+    public static final String cvs="$Id: Version.java,v 1.66 2008/12/05 11:37:45 belaban Exp $";
 
     private static final int MAJOR_SHIFT = 11;
     private static final int MINOR_SHIFT = 6;
@@ -101,10 +101,10 @@ public class Version {
      * @return
      */
     public static boolean isBinaryCompatible(short ver) {
-        if(ver == version)
+        if(version == ver)
             return true;
-        short[] tmp=decode(ver);
-        short tmp_major=tmp[0], tmp_minor=tmp[1];
+        short tmp_major=(short)((ver & MAJOR_MASK) >> MAJOR_SHIFT);
+        short tmp_minor=(short)((ver & MINOR_MASK) >> MINOR_SHIFT);
         return major == tmp_major && minor == tmp_minor;
     }
 
