@@ -27,7 +27,6 @@ import java.util.concurrent.locks.ReentrantLock;
 /**
  * Shared class for TCP connection tables.
  * @author Scott Marlow
- * @author Bela Ban
  */
 public abstract class BasicConnectionTable {
     private ThreadFactory factory;
@@ -70,7 +69,7 @@ public abstract class BasicConnectionTable {
 
     final static long   MAX_JOIN_TIMEOUT=Global.THREAD_SHUTDOWN_WAIT_TIME;
 
-    protected BasicConnectionTable() {
+    protected BasicConnectionTable() {        
         factory = new DefaultThreadFactory(new ThreadGroup(Util.getGlobalThreadGroup(),"ConnectionTable"),"Connection Table", false);
     }
 
@@ -185,7 +184,7 @@ public abstract class BasicConnectionTable {
             try {
                 if(pm != null) {
                     int tmp_port=srv_sock.getLocalPort();
-                    pm.removePort(tmp_port);
+                    pm.updatePort(tmp_port);
                 }
                 ServerSocket tmp=srv_sock;
                 srv_sock=null;
