@@ -38,7 +38,7 @@ import java.util.Vector;
  * the application instead of protocol level.
  *
  * @author Bela Ban
- * @version $Id: MessageDispatcher.java,v 1.74.2.4 2008/11/14 15:16:41 belaban Exp $
+ * @version $Id: MessageDispatcher.java,v 1.81 2008/12/05 14:59:40 belaban Exp $
  */
 public class MessageDispatcher implements RequestHandler {
     protected Channel channel=null;
@@ -50,7 +50,6 @@ public class MessageDispatcher implements RequestHandler {
     protected TransportAdapter transport_adapter=null;
     protected final Collection members=new TreeSet();
     protected Address local_addr=null;
-    protected boolean deadlock_detection=false;
     protected PullPushAdapter adapter=null;
     protected PullPushHandler handler=null;
     protected Serializable id=null;
@@ -86,7 +85,6 @@ public class MessageDispatcher implements RequestHandler {
 
     public MessageDispatcher(Channel channel, MessageListener l, MembershipListener l2, boolean deadlock_detection) {
         this.channel=channel;
-        this.deadlock_detection=deadlock_detection;
         prot_adapter=new ProtocolAdapter();
         if(channel != null) {
             local_addr=channel.getLocalAddress();
@@ -102,7 +100,6 @@ public class MessageDispatcher implements RequestHandler {
     public MessageDispatcher(Channel channel, MessageListener l, MembershipListener l2,
                              boolean deadlock_detection, boolean concurrent_processing) {
         this.channel=channel;
-        this.deadlock_detection=deadlock_detection;
         this.concurrent_processing=concurrent_processing;
         prot_adapter=new ProtocolAdapter();
         if(channel != null) {
@@ -259,17 +256,16 @@ public class MessageDispatcher implements RequestHandler {
         }
     }
 
-    public boolean getDeadlockDetection() {return deadlock_detection;}
+    @Deprecated
+    public boolean getDeadlockDetection() {return false;}
 
+    @Deprecated
     public void setDeadlockDetection(boolean flag) {
-        deadlock_detection=flag;
-        if(corr != null)
-            corr.setDeadlockDetection(flag);
     }
 
 
     public boolean getConcurrentProcessing() {return concurrent_processing;}
-
+    
     public void setConcurrentProcessing(boolean flag) {
         this.concurrent_processing=flag;
     }
@@ -278,12 +274,12 @@ public class MessageDispatcher implements RequestHandler {
     public final void start() {
         if(corr == null) {
             if(transport_adapter != null) {
-                corr=new RequestCorrelator("MessageDispatcher", transport_adapter,
-                                           this, deadlock_detection, local_addr, concurrent_processing);
+                corr=new RequestCorrelator("MsgDisp", transport_adapter,
+                                           this, local_addr, concurrent_processing);
             }
             else {
-                corr=new RequestCorrelator("MessageDispatcher", prot_adapter,
-                                           this, deadlock_detection, local_addr, concurrent_processing);
+                corr=new RequestCorrelator("MsgDisp", prot_adapter,
+                                           this, local_addr, concurrent_processing);
             }
         }
         correlatorStarted();
@@ -293,11 +289,11 @@ public class MessageDispatcher implements RequestHandler {
             Vector tmp_mbrs=channel.getView() != null ? channel.getView().getMembers() : null;
             setMembers(tmp_mbrs);
             if(channel instanceof JChannel) {
-                TP transport=((JChannel)channel).getProtocolStack().getTransport();
+                TP transport=channel.getProtocolStack().getTransport();
                 corr.registerProbeHandler(transport);
-            }
         }
     }
+    }
 
     protected void correlatorStarted() {
         ;
@@ -310,7 +306,7 @@ public class MessageDispatcher implements RequestHandler {
         }
 
         if(channel instanceof JChannel) {
-            TP transport=((JChannel)channel).getProtocolStack().getTransport();
+            TP transport=channel.getProtocolStack().getTransport();
             corr.unregisterProbeHandler(transport);
         }
 
