@@ -1,4 +1,4 @@
-// $Id: RpcDispatcher.java,v 1.30.2.3 2008/05/30 11:36:53 belaban Exp $
+// $Id: RpcDispatcher.java,v 1.36 2008/10/10 14:53:30 belaban Exp $
 
 package org.jgroups.blocks;
 
@@ -6,6 +6,7 @@ package org.jgroups.blocks;
 import org.jgroups.*;
 import org.jgroups.util.RspList;
 import org.jgroups.util.Util;
+import org.jgroups.util.Buffer;
 
 import java.io.Serializable;
 import java.lang.reflect.Method;
@@ -32,10 +33,10 @@ import java.util.Vector;
 public class RpcDispatcher extends MessageDispatcher implements ChannelListener {
     protected Object        server_obj=null;
     /** Marshaller to marshall requests at the caller and unmarshal requests at the receiver(s) */
-    protected Marshaller    req_marshaller=null;
+    protected Marshaller2   req_marshaller=null;
 
     /** Marshaller to marshal responses at the receiver(s) and unmarshal responses at the caller */
-    protected Marshaller    rsp_marshaller=null;
+    protected Marshaller2   rsp_marshaller=null;
     protected final List    additionalChannelListeners=new ArrayList();
     protected MethodLookup  method_lookup=null;
 
@@ -53,14 +54,14 @@ public class RpcDispatcher extends MessageDispatcher implements ChannelListener
 
     public RpcDispatcher(Channel channel, MessageListener l, MembershipListener l2, Object server_obj,
                          boolean deadlock_detection) {
-        super(channel, l, l2, deadlock_detection);
+        super(channel, l, l2);
         channel.addChannelListener(this);
         this.server_obj=server_obj;
     }
 
     public RpcDispatcher(Channel channel, MessageListener l, MembershipListener l2, Object server_obj,
                          boolean deadlock_detection, boolean concurrent_processing) {
-        super(channel, l, l2, deadlock_detection, concurrent_processing);
+        super(channel, l, l2, false, concurrent_processing);
         channel.addChannelListener(this);
         this.server_obj=server_obj;
     }
@@ -90,25 +91,89 @@ public class RpcDispatcher extends MessageDispatcher implements ChannelListener
     }
 
 
+    public interface Marshaller2 extends Marshaller {
+        /**
+         * Marshals the object into a byte[] buffer and returns a Buffer with a ref to the underlying byte[] buffer,
+         * offset and length.<br/>
+         * <em>
+         * Note that the underlying byte[] buffer must not be changed as this would change the buffer of a message which
+         * potentially can get retransmitted, and such a retransmission would then carry a ref to a changed byte[] buffer !
+         * </em>
+         * @param obj
+         * @return
+         * @throws Exception
+         */
+        Buffer objectToBuffer(Object obj) throws Exception;
+
+        Object objectFromByteBuffer(byte[] buf, int offset, int length) throws Exception;
+    }
+
+
+    /** Used to provide a Marshaller2 interface to a Marshaller. This class is for internal use only, and will be
+     * removed in 3.0 when Marshaller and Marshaller2 get merged. Do not use, but provide an implementation of
+     * Marshaller directly, e.g. in setRequestMarshaller().
+     */
+    public static class MarshallerAdapter implements Marshaller2 {
+        private final Marshaller marshaller;
+
+        public MarshallerAdapter(Marshaller marshaller) {
+            this.marshaller=marshaller;
+        }
+
+        public byte[] objectToByteBuffer(Object obj) throws Exception {
+            return marshaller.objectToByteBuffer(obj);
+        }
+
+        public Object objectFromByteBuffer(byte[] buf) throws Exception {
+            return buf == null? null : marshaller.objectFromByteBuffer(buf);
+        }
+
+        public Buffer objectToBuffer(Object obj) throws Exception {
+            byte[] buf=marshaller.objectToByteBuffer(obj);
+            return new Buffer(buf, 0, buf.length);
+        }
+
+        public Object objectFromByteBuffer(byte[] buf, int offset, int length) throws Exception {
+            if(buf == null || (offset == 0 && length == buf.length))
+                return marshaller.objectFromByteBuffer(buf);
+            byte[] tmp=new byte[length];
+            System.arraycopy(buf, offset, tmp, 0, length);
+            return marshaller.objectFromByteBuffer(tmp);
+        }
+
+    }
+
+
     public String getName() {return "RpcDispatcher";}
 
     public Marshaller getRequestMarshaller()             {return req_marshaller;}
 
     public void setRequestMarshaller(Marshaller m) {
-        this.req_marshaller=m;
+        if(m == null)
+            this.req_marshaller=null;
+        else if(m instanceof Marshaller2)
+            this.req_marshaller=(Marshaller2)m;
+        else
+            this.req_marshaller=new MarshallerAdapter(m);
     }
 
     public Marshaller getResponseMarshaller()             {return rsp_marshaller;}
 
     public void setResponseMarshaller(Marshaller m) {
-        this.rsp_marshaller=m;
+        if(m == null)
+            this.rsp_marshaller=null;
+        else if(m instanceof Marshaller2)
+            this.rsp_marshaller=(Marshaller2)m;
+        else
+            this.rsp_marshaller=new MarshallerAdapter(m);
+
         if(corr != null)
-            corr.setMarshaller(m);
+            corr.setMarshaller(this.rsp_marshaller);
     }
 
     public Marshaller getMarshaller() {return req_marshaller;}
     
-    public void setMarshaller(Marshaller m) {req_marshaller=m;}
+    public void setMarshaller(Marshaller m) {setRequestMarshaller(m);}
 
     public Object getServerObject() {return server_obj;}
 
@@ -198,9 +263,9 @@ public class RpcDispatcher extends MessageDispatcher implements ChannelListener
             log.trace(new StringBuilder("dests=").append(dests).append(", method_call=").append(method_call).
                     append(", mode=").append(mode).append(", timeout=").append(timeout));
 
-        byte[] buf;
+        Object buf;
         try {
-            buf=req_marshaller != null? req_marshaller.objectToByteBuffer(method_call) : Util.objectToByteBuffer(method_call);
+            buf=req_marshaller != null? req_marshaller.objectToBuffer(method_call) : Util.objectToByteBuffer(method_call);
         }
         catch(Exception e) {
             // if(log.isErrorEnabled()) log.error("exception", e);
@@ -210,7 +275,11 @@ public class RpcDispatcher extends MessageDispatcher implements ChannelListener
             throw new RuntimeException("failure to marshal argument(s)", e);
         }
 
-        Message msg=new Message(null, null, buf);
+        Message msg=new Message();
+        if(buf instanceof Buffer)
+            msg.setBuffer((Buffer)buf);
+        else
+            msg.setBuffer((byte[])buf);
         if(oob)
             msg.setFlag(Message.OOB);
         RspList  retval=super.castMessage(dests, msg, mode, timeout, use_anycasting, filter);
@@ -236,15 +305,19 @@ public class RpcDispatcher extends MessageDispatcher implements ChannelListener
     }
 
     public Object callRemoteMethod(Address dest, MethodCall method_call, int mode, long timeout, boolean oob) throws Throwable {
-        byte[]   buf=null;
+        Object   buf=null;
         Message  msg=null;
         Object   retval=null;
 
         if(log.isTraceEnabled())
             log.trace("dest=" + dest + ", method_call=" + method_call + ", mode=" + mode + ", timeout=" + timeout);
 
-        buf=req_marshaller != null? req_marshaller.objectToByteBuffer(method_call) : Util.objectToByteBuffer(method_call);
-        msg=new Message(dest, null, buf);
+        buf=req_marshaller != null? req_marshaller.objectToBuffer(method_call) : Util.objectToByteBuffer(method_call);
+        msg=new Message(dest, null, null);
+        if(buf instanceof Buffer)
+            msg.setBuffer((Buffer)buf);
+        else
+            msg.setBuffer((byte[])buf);
         if(oob)
             msg.setFlag(Message.OOB);
         retval=super.sendMessage(msg, mode, timeout);
@@ -280,7 +353,9 @@ public class RpcDispatcher extends MessageDispatcher implements ChannelListener
         }
 
         try {
-            body=req_marshaller != null? req_marshaller.objectFromByteBuffer(req.getBuffer()) : req.getObject();
+            body=req_marshaller != null?
+                    req_marshaller.objectFromByteBuffer(req.getBuffer(), req.getOffset(), req.getLength())
+                    : req.getObject();
         }
         catch(Throwable e) {
             if(log.isErrorEnabled()) log.error("exception marshalling object", e);
@@ -289,10 +364,9 @@ public class RpcDispatcher extends MessageDispatcher implements ChannelListener
 
         if(!(body instanceof MethodCall)) {
             if(log.isErrorEnabled()) log.error("message does not contain a MethodCall object");
-
+            
             // create an exception to represent this and return it
-            IllegalArgumentException e = new IllegalArgumentException("message does not contain a MethodCall object") ;
-            return e ;
+            return  new IllegalArgumentException("message does not contain a MethodCall object") ;
         }
 
         method_call=(MethodCall)body;
