@@ -1,6 +1,10 @@
+
 package org.jgroups.protocols;
 
 import org.jgroups.*;
+import org.jgroups.annotations.MBean;
+import org.jgroups.annotations.ManagedAttribute;
+import org.jgroups.annotations.Property;
 import org.jgroups.protocols.pbcast.JoinRsp;
 import org.jgroups.stack.Protocol;
 import org.jgroups.util.Promise;
@@ -13,43 +17,76 @@ import java.util.concurrent.TimeUnit;
 
 
 /**
- * The Discovery protocol layer retrieves the initial membership (used by the GMS when started
- * by sending event FIND_INITIAL_MBRS down the stack). We do this by specific subclasses, e.g. by mcasting PING
- * requests to an IP MCAST address or, if gossiping is enabled, by contacting the GossipRouter.
- * The responses should allow us to determine the coordinator whom we have to
- * contact, e.g. in case we want to join the group.  When we are a server (after having
- * received the BECOME_SERVER event), we'll respond to PING requests with a PING
- * response.<p> The FIND_INITIAL_MBRS event will eventually be answered with a
- * FIND_INITIAL_MBRS_OK event up the stack.
- * The following properties are available
+ * The Discovery protocol layer retrieves the initial membership (used by the
+ * GMS when started by sending event FIND_INITIAL_MBRS down the stack). We do
+ * this by specific subclasses, e.g. by mcasting PING requests to an IP MCAST
+ * address or, if gossiping is enabled, by contacting the GossipRouter. The
+ * responses should allow us to determine the coordinator whom we have to
+ * contact, e.g. in case we want to join the group. When we are a server (after
+ * having received the BECOME_SERVER event), we'll respond to PING requests with
+ * a PING response.
+ * <p>
+ * The FIND_INITIAL_MBRS event will eventually be answered with a
+ * FIND_INITIAL_MBRS_OK event up the stack. The following properties are
+ * available
  * <ul>
- * <li>timeout - the timeout (ms) to wait for the initial members, default is 3000=3 secs
- * <li>num_initial_members - the minimum number of initial members for a FIND_INITAL_MBRS, default is 2
- * <li>num_ping_requests - the number of GET_MBRS_REQ messages to be sent (min=1), distributed over timeout ms
+ * <li>timeout - the timeout (ms) to wait for the initial members, default is
+ * 3000=3 secs
+ * <li>num_initial_members - the minimum number of initial members for a
+ * FIND_INITAL_MBRS, default is 2
+ * <li>num_ping_requests - the number of GET_MBRS_REQ messages to be sent
+ * (min=1), distributed over timeout ms
  * </ul>
+ * 
  * @author Bela Ban
- * @version $Id: Discovery.java,v 1.32.2.6 2008/09/19 10:56:43 belaban Exp $
+ * @version $Id: Discovery.java,v 1.52 2008/10/21 12:37:10 vlada Exp $
  */
-public abstract class Discovery extends Protocol {
-    final Vector<Address>	members=new Vector<Address>(11);
-    Address			local_addr=null;
-    String			group_addr=null;
-    long			timeout=3000;
-    int				num_initial_members=2;
-    boolean			is_server=false;
-    TimeScheduler		timer=null;
+@MBean
+public abstract class Discovery extends Protocol {   
+    
+    
+    /* -----------------------------------------    Properties     -------------------------------------------------- */
 
-    /** Number of GET_MBRS_REQ messages to be sent (min=1), distributed over timeout ms */
-    int                     num_ping_requests=2;
-    int                     num_discovery_requests=0;
+    @Property(description="Timeout to wait for the initial members. Default is 3000 msec")
+    @ManagedAttribute(description="Timeout (ms) to wait for the initial members", writable=true)
+    long timeout=3000;
 
-    /** Minimum number of server responses (PingRsp.isServer()=true). If this value is greater than 0, we'll ignore num_initial_members */
+    @Property(description="Minimum number of initial members to get a response from. Default is 2")
+    @ManagedAttribute(description="Minimum number of initial members to get a response from", writable=true)
+    int num_initial_members=2;
+
+    @Property(description="Minimum number of server responses (PingRsp.isServer()=true). If this value is " +
+            "greater than 0, we'll ignore num_initial_members")
+    @ManagedAttribute(writable=true, description="Minimum number of server responses (PingRsp.isServer()=true). " +
+            "If this value is greater than 0, we'll ignore num_initial_members")
     int num_initial_srv_members=0;
 
-    /** Return from the discovery phase as soon as we have 1 coordinator response */
+    @Property(description="Return from the discovery phase as soon as we have 1 coordinator response")
+    @ManagedAttribute(writable=true, description="Return from the discovery phase as soon as we have 1 coordinator response")
     boolean break_on_coord_rsp=true;
 
+    @Property(description="Number of discovery requests to be sent distributed over timeout. Default is 2")
+    @ManagedAttribute(description="Number of discovery requests to be sent (min=1), " + "distributed over timeout ms", writable=true)
+    int num_ping_requests=2;
 
+    
+    /* ---------------------------------------------   JMX      ------------------------------------------------------ */
+
+    
+    @ManagedAttribute(description="Total number of discovery requests sent ")
+    int num_discovery_requests=0;
+
+
+    
+    /* --------------------------------------------- Fields ------------------------------------------------------ */
+    
+    
+    private volatile boolean is_server=false;
+    protected TimeScheduler timer=null;
+
+    private final Vector<Address> members=new Vector<Address>(11);
+    protected Address local_addr=null;
+    protected String group_addr=null;
     private final Set<Responses> ping_responses=new HashSet<Responses>();
     private final PingSenderTask sender=new PingSenderTask();
     
@@ -66,7 +103,7 @@ public abstract class Discovery extends Protocol {
     public void localAddressSet(Address addr) {
     }
 
-    public abstract void sendGetMembersRequest();
+    public abstract void sendGetMembersRequest(String cluster_name);
 
 
     public void handleDisconnect() {
@@ -109,70 +146,7 @@ public abstract class Discovery extends Protocol {
         ret.addElement(new Integer(Event.FIND_INITIAL_MBRS));
         return ret;
     }
-
-    /**
-     * sets the properties of the PING protocol.
-     * The following properties are available
-     * property: timeout - the timeout (ms) to wait for the initial members, default is 3000=3 secs
-     * property: num_initial_members - the minimum number of initial members for a FIND_INITAL_MBRS, default is 2
-     * @param props - a property set
-     * @return returns true if all properties were parsed properly
-     *         returns false if there are unrecnogized properties in the property set
-     */
-    public boolean setProperties(Properties props) {
-        String str;
-
-        super.setProperties(props);
-        str=props.getProperty("timeout");              // max time to wait for initial members
-        if(str != null) {
-            timeout=Long.parseLong(str);
-            if(timeout <= 0) {
-                if(log.isErrorEnabled()) log.error("timeout must be > 0");
-                return false;
-            }
-            props.remove("timeout");
-        }
-
-        str=props.getProperty("num_initial_members");  // wait for at most n members
-        if(str != null) {
-            num_initial_members=Integer.parseInt(str);
-            props.remove("num_initial_members");
-        }
-
-        str=props.getProperty("num_initial_srv_members");  // wait for at most n server members
-        if(str != null) {
-            num_initial_srv_members=Integer.parseInt(str);
-            props.remove("num_initial_srv_members");
-        }
-
-        str=props.getProperty("break_on_coord_rsp");  // terminate discovery phase when we receive 1 response from a coord
-        if(str != null) {
-            break_on_coord_rsp=Boolean.parseBoolean(str);
-            props.remove("break_on_coord_rsp");
-        }
-
-        str=props.getProperty("num_ping_requests");  // number of GET_MBRS_REQ messages
-        if(str != null) {
-            num_ping_requests=Integer.parseInt(str);
-            props.remove("num_ping_requests");
-            if(num_ping_requests < 1)
-                num_ping_requests=1;
-        }
-
-        if(!props.isEmpty()) {
-            StringBuilder sb=new StringBuilder();
-            for(Enumeration<?> e=props.propertyNames(); e.hasMoreElements();) {
-                sb.append(e.nextElement().toString());
-                if(e.hasMoreElements()) {
-                    sb.append(", ");
-                }
-            }
-            if(log.isErrorEnabled()) log.error("The following properties are not recognized: " + sb);
-            return false;
-        }
-        return true;
-    }
-
+    
     public void resetStats() {
         super.resetStats();
         num_discovery_requests=0;
@@ -199,12 +173,12 @@ public abstract class Discovery extends Protocol {
             ping_responses.add(rsps);
         }
 
-        sender.start();
+        sender.start(group_addr);
         try {
             return rsps.get(timeout);
         }
         catch(Exception e) {
-            return new LinkedList<PingRsp>();           
+            return new LinkedList<PingRsp>();
         }
         finally {
         	sender.stop();
@@ -269,6 +243,23 @@ public abstract class Discovery extends Protocol {
                 if(local_addr != null && msg.getSrc() != null && local_addr.equals(msg.getSrc())) {
                     return null;
                 }
+
+                if(group_addr == null || hdr.cluster_name == null) {
+                    if(log.isWarnEnabled())
+                        log.warn("group_addr (" + group_addr + ") or cluster_name of header (" + hdr.cluster_name
+                        + ") is null; passing up discovery request from " + msg.getSrc() + ", but this should not" +
+                                " be the case");
+                }
+                else {
+                    if(!group_addr.equals(hdr.cluster_name)) {
+                        if(log.isWarnEnabled())
+                            log.warn("discarding discovery request for cluster '" + hdr.cluster_name + "' from " +
+                                    msg.getSrc() + "; our cluster name is '" + group_addr + "'. " +
+                                    "Please separate your clusters cleanly.");
+                        return null;
+                    }
+                }
+
                 synchronized(members) {
                     coord=!members.isEmpty()? members.firstElement() : local_addr;
                 }
@@ -288,8 +279,7 @@ public abstract class Discovery extends Protocol {
 
                 if(log.isTraceEnabled())
                     log.trace("received GET_MBRS_RSP, rsp=" + rsp);
-                // myfuture.addResponse(rsp);
-
+                
                 synchronized(ping_responses) {
                     for(Responses rsps: ping_responses)
                         rsps.addResponse(rsp);
@@ -393,17 +383,17 @@ public abstract class Discovery extends Protocol {
     
 
     class PingSenderTask {        
-        private Future<?>	senderFuture;
+	private Future<?>      senderFuture;
 
         public PingSenderTask() {}
 
-        public synchronized void start() {
+        public synchronized void start(final String cluster_name) {
             long delay = (long)(timeout / (double)num_ping_requests);
             if(senderFuture == null || senderFuture.isDone()) {
                 senderFuture=timer.scheduleWithFixedDelay(new Runnable() {
                     public void run() {
                         try {
-                            sendGetMembersRequest();
+                            sendGetMembersRequest(cluster_name);
                         }
                         catch(Exception ex) {
                             if(log.isErrorEnabled())
@@ -419,14 +409,14 @@ public abstract class Discovery extends Protocol {
                 senderFuture.cancel(true);
                 senderFuture=null;
             }
-        }
+        }      
     }
 
 
     private static class Responses {
-        final Promise<JoinRsp>       promise;
-        final List<PingRsp> ping_rsps=new LinkedList<PingRsp>();
-        final int           num_expected_rsps;
+        final Promise<JoinRsp>  promise;
+        final List<PingRsp>     ping_rsps=new LinkedList<PingRsp>();
+        final int               num_expected_rsps;
         final int               num_expected_srv_rsps;
         final boolean           break_on_coord_rsp;
 
@@ -444,7 +434,7 @@ public abstract class Discovery extends Protocol {
             try {
                 if(!ping_rsps.contains(rsp)) {
                     ping_rsps.add(rsp);
-                        promise.getCond().signalAll();
+                    promise.getCond().signalAll();
                 }
             }
             finally {
@@ -452,7 +442,7 @@ public abstract class Discovery extends Protocol {
             }
         }
 
-        public List<PingRsp> get(long timeout) throws InterruptedException {
+        public List<PingRsp> get(long timeout) throws InterruptedException{
             long start_time=System.currentTimeMillis(), time_to_wait=timeout;
 
             promise.getLock().lock();
@@ -486,7 +476,7 @@ public abstract class Discovery extends Protocol {
             for(PingRsp rsp: rsps) {
                 if(rsp.isServer())
                     cnt++;
-    }
+            }
             return cnt;
         }
 
@@ -500,5 +490,5 @@ public abstract class Discovery extends Protocol {
             return false;
         }
 
-}
+    }
 }
