@@ -2,6 +2,10 @@ package org.jgroups.protocols;
 
 import org.jgroups.*;
 import org.jgroups.annotations.GuardedBy;
+import org.jgroups.annotations.MBean;
+import org.jgroups.annotations.ManagedAttribute;
+import org.jgroups.annotations.ManagedOperation;
+import org.jgroups.annotations.Property;
 import org.jgroups.stack.Protocol;
 import org.jgroups.util.BoundedList;
 import org.jgroups.util.Streamable;
@@ -34,18 +38,101 @@ import java.util.concurrent.locks.ReentrantLock;
  * <li>Receivers don't send the full credits (max_credits), but rather tha actual number of bytes received
  * <ol/>
  * @author Bela Ban
- * @version $Id: FC.java,v 1.90.2.4 2008/08/19 08:38:44 belaban Exp $
+ * @version $Id: FC.java,v 1.101 2008/10/10 13:46:12 vlada Exp $
  */
+@MBean(description="Simple flow control protocol based on a credit system")
 public class FC extends Protocol {
 
+    private final static FcHeader REPLENISH_HDR=new FcHeader(FcHeader.REPLENISH);
+    private final static FcHeader CREDIT_REQUEST_HDR=new FcHeader(FcHeader.CREDIT_REQUEST);  
+    private final static String name="FC";
+    
+    
+    /* -----------------------------------------    Properties     -------------------------------------------------- */
+    
+    /**
+     * Max number of bytes to send per receiver until an ack must
+     * be received before continuing sending
+     */
+    @ManagedAttribute(description="Max number of bytes to send per receiver until an ack must " + 
+                                   "be received before continuing sending",writable=true)
+    @Property(description="Max number of bytes to send per receiver until an ack must be received to proceed. Default is 500000 bytes")                                   
+    private long max_credits=500000;
+
+    /**
+     * Max time (in milliseconds) to block. If credit hasn't been received after max_block_time, we send
+     * a REPLENISHMENT request to the members from which we expect credits. A value <= 0 means to
+     * wait forever.
+     */
+    @ManagedAttribute(description="Max time (in milliseconds) to block",writable=true)
+    @Property(description="Max time (in milliseconds) to block. Default is 5000 msec")
+    private long max_block_time=5000;
+
+    /**
+     * Defines the max number of milliseconds for a message to block before being sent, based on the length of
+     * the message. The property is defined as a comma-separated list of values (separated by ':'), where the key
+     * is the size in bytes and the value is the number of milliseconds to block.
+     * Example: max_block_times="50:1,500:3,1500:5,10000:10,100000:100". This means that messages up to 50 bytes wait
+     * 1 ms max until they get sent, messages up to 500 bytes 3 ms, and so on.
+     * If a message's length (size of the payload in bytes) is for example 15'000 bytes,
+     * FC blocks it for a max of 100 ms.
+     */
+    private Map<Long,Long> max_block_times=null;
+
+    /** Keeps track of the end time after which a message should not get blocked anymore */
+    private static final ThreadLocal<Long> end_time=new ThreadLocal<Long>();
+
+
+    /**
+     * If credits fall below this limit, we send more credits to the sender. (We also send when
+     * credits are exhausted (0 credits left))
+     */
+    @ManagedAttribute(description="If credits fall below this limit, we send more credits to the sender",writable=true)
+    @Property(description="If credits fall below this limit, we send more credits to the sender. Default is 0.25")
+    private double min_threshold=0.25;
+
+    /**
+     * Computed as <tt>max_credits</tt> times <tt>min_theshold</tt>. If explicitly set, this will
+     * override the above computation
+     */
+    @ManagedAttribute(description="Computed as max_credits x min_theshold",writable=true)
+    @Property(description="Computed as max_credits x min_theshold unless explicitely set")
+    private long min_credits=0;
+    
+    /**
+     * Whether an up thread that comes back down should be allowed to
+     * bypass blocking if all credits are exhausted. Avoids JGRP-465.
+     * Set to false by default in 2.5 because we have OOB messages for credit replenishments - this flag should not be set
+     * to true if the concurrent stack is used
+     */
+    @Property
+    private boolean ignore_synchronous_response=false;
+    
+    
+    
+    
+    /* ---------------------------------------------   JMX      ------------------------------------------------------ */
+    
+    
+    private int num_blockings=0;
+    private int num_credit_requests_received=0, num_credit_requests_sent=0;
+    private int num_credit_responses_sent=0, num_credit_responses_received=0;
+    private long total_time_blocking=0;
+
+    private final BoundedList<Long> last_blockings=new BoundedList<Long>(50);
+    
+    
+    
+    /* --------------------------------------------- Fields ------------------------------------------------------ */
+    
+    
     /**
      * Map<Address,Long>: keys are members, values are credits left. For each send, the
      * number of credits is decremented by the message size. A HashMap rather than a ConcurrentHashMap is
      * currently used as there might be null values
      */
     @GuardedBy("sent_lock")
-    private final Map<Address, Long> sent=new HashMap<Address, Long>(11);
-    // final Map sent=new ConcurrentHashMap(11);
+    private final Map<Address, Long> sent=new HashMap<Address, Long>(11);   
 
     /**
      * Map<Address,Long>: keys are members, values are credits left (in bytes).
@@ -63,59 +150,19 @@ public class FC extends Protocol {
     @GuardedBy("sent_lock")
     private final Set<Address> creditors=new HashSet<Address>(11);
 
+    
     /** Peers who have asked for credit that we didn't have */
     private final Set<Address> pending_requesters=new HashSet<Address>(11);
 
-    /**
-     * Max number of bytes to send per receiver until an ack must
-     * be received before continuing sending
-     */
-    private long max_credits=500000;
-
-    /**
-     * Max time (in milliseconds) to block. If credit hasn't been received after max_block_time, we send
-     * a REPLENISHMENT request to the members from which we expect credits. A value <= 0 means to
-     * wait forever. Note that when max_block_time elapses, we do not send the message, instead we send only
-     * a credit request. If we want a message to get sent after N milliseconds, use max_block_times instead.
-     */
-    private long max_block_time=5000;
-
-    /**
-     * Defines the max number of milliseconds for a message to block before being sent, based on the length of
-     * the message. The property is defined as a comma-separated list of values (separated by ':'), where the key
-     * is the size in bytes and the value is the number of milliseconds to block.
-     * Example: max_block_times="50:1,500:3,1500:5,10000:10,100000:100". This means that messages up to 50 bytes wait
-     * 1 ms max until they get sent, messages up to 500 bytes 3 ms, and so on. 
-     * If a message's length (size of the payload in bytes) is for example 15'000 bytes,
-     * FC blocks it for a max of 100 ms.
-     */
-    private Map<Long,Long> max_block_times=null;
-
-    /** Keeps track of the end time after which a message should not get blocked anymore */
-    private static final ThreadLocal<Long> end_time=new ThreadLocal<Long>();
-    
-
-    /**
-     * If credits fall below this limit, we send more credits to the sender. (We also send when
-     * credits are exhausted (0 credits left))
-     */
-    private double min_threshold=0.25;
-
-    /**
-     * Computed as <tt>max_credits</tt> times <tt>min_theshold</tt>. If explicitly set, this will
-     * override the above computation
-     */
-    private long min_credits=0;
-
     /**
      * Whether FC is still running, this is set to false when the protocol terminates (on stop())
      */
-    private boolean running=true;
+    private volatile boolean running=true;
 
 
     private boolean frag_size_received=false;
 
-
+   
     /**
      * the lowest credits of any destination (sent_msgs)
      */
@@ -131,36 +178,17 @@ public class FC extends Protocol {
 
     /** Mutex to block on down() */
     private final Condition credits_available=sent_lock.newCondition();
-
-    /**
-     * Whether an up thread that comes back down should be allowed to
-     * bypass blocking if all credits are exhausted. Avoids JGRP-465.
-     * Set to false by default in 2.5 because we have OOB messages for credit replenishments - this flag should not be set
-     * to true if the concurrent stack is used
-     */
-    private boolean ignore_synchronous_response=false;
+   
 
     /**
      * Thread that carries messages through up() and shouldn't be blocked
      * in down() if ignore_synchronous_response==true. JGRP-465.
      */
-    private Thread ignore_thread;
-
-    private static final String name="FC";
+    private Thread ignore_thread;   
 
     /** Last time a credit request was sent. Used to prevent credit request storms */
     @GuardedBy("sent_lock")
-    private long last_credit_request=0;
-
-    private int num_blockings=0;
-    private int num_credit_requests_received=0, num_credit_requests_sent=0;
-    private int num_credit_responses_sent=0, num_credit_responses_received=0;
-    private long total_time_blocking=0;
-
-    private final BoundedList<Long> last_blockings=new BoundedList<Long>(50);
-
-    private final static FcHeader REPLENISH_HDR=new FcHeader(FcHeader.REPLENISH);
-    private final static FcHeader CREDIT_REQUEST_HDR=new FcHeader(FcHeader.CREDIT_REQUEST);
+    private long last_credit_request=0;   
 
 
     public final String getName() {
@@ -199,6 +227,7 @@ public class FC extends Protocol {
         this.min_credits=min_credits;
     }
 
+    @ManagedAttribute(description="Number of times flow control blocks sender")
     public int getNumberOfBlockings() {
         return num_blockings;
     }
@@ -211,38 +240,100 @@ public class FC extends Protocol {
         max_block_time=t;
     }
 
+    @ManagedAttribute(writable=true)
+    @Property(description="Max times to block for the listed messages sizes (Message.getLength())")
+    public void setMaxBlockTimes(String str) {
+        if(str == null) return;
+        Long prev_key=null, prev_val=null;
+        List<String> vals=Util.parseCommaDelimitedStrings(str);
+        if(max_block_times == null)
+            max_block_times=new TreeMap<Long,Long>();
+        for(String tmp: vals) {
+            int index=tmp.indexOf(':');
+            if(index == -1)
+                throw new IllegalArgumentException("element '" + tmp + "'  is missing a ':' separator");
+            Long key=Long.parseLong(tmp.substring(0, index).trim());
+            Long val=Long.parseLong(tmp.substring(index +1).trim());
+
+            // sanity checks:
+            if(key < 0 || val < 0)
+                throw new IllegalArgumentException("keys and values must be >= 0");
+
+            if(prev_key != null) {
+                if(key <= prev_key)
+                    throw new IllegalArgumentException("keys are not sorted: " + vals);
+            }
+            prev_key=key;
+
+            if(prev_val != null) {
+                if(val <= prev_val)
+                    throw new IllegalArgumentException("values are not sorted: " + vals);
+            }
+            prev_val=val;
+            max_block_times.put(key, val);
+        }
+        if(log.isDebugEnabled())
+            log.debug("max_block_times: " + max_block_times);
+    }
+
+    @ManagedAttribute
+    public String getMaxBlockTimes() {
+        if(max_block_times == null) return "n/a";
+        StringBuilder sb=new StringBuilder();
+        boolean first=true;
+        for(Map.Entry<Long,Long> entry: max_block_times.entrySet()) {
+            if(!first) {
+                sb.append(", ");
+            }
+            else {
+                first=false;
+            }
+            sb.append(entry.getKey()).append(":").append(entry.getValue());
+        }
+        return sb.toString();
+    }
+    
+    @ManagedAttribute(description="Total time (ms) spent in flow control block")
     public long getTotalTimeBlocked() {
         return total_time_blocking;
     }
 
+    @ManagedAttribute(description="Average time spent in a flow control block")
     public double getAverageTimeBlocked() {
         return num_blockings == 0? 0.0 : total_time_blocking / (double)num_blockings;
     }
 
+    @ManagedAttribute(description="Number of credit requests received")
     public int getNumberOfCreditRequestsReceived() {
         return num_credit_requests_received;
     }
-
+    
+    @ManagedAttribute(description="Number of credit requests sent")
     public int getNumberOfCreditRequestsSent() {
         return num_credit_requests_sent;
     }
 
+    @ManagedAttribute(description="Number of credit responses received")
     public int getNumberOfCreditResponsesReceived() {
         return num_credit_responses_received;
     }
 
+    @ManagedAttribute(description="Number of credit responses sent")
     public int getNumberOfCreditResponsesSent() {
         return num_credit_responses_sent;
     }
 
+    @ManagedOperation(description="Print sender credits")
     public String printSenderCredits() {
         return printMap(sent);
     }
 
+    @ManagedOperation(description="Print receiver credits")
     public String printReceiverCredits() {
         return printMap(received);
     }
 
+    @ManagedOperation(description="Print credits")
     public String printCredits() {
         StringBuilder sb=new StringBuilder();
         sb.append("senders:\n").append(printMap(sent)).append("\n\nreceivers:\n").append(printMap(received));
@@ -250,19 +341,13 @@ public class FC extends Protocol {
     }
 
     public Map<String, Object> dumpStats() {
-        Map<String, Object> retval=super.dumpStats();
-        if(retval == null)
-            retval=new HashMap<String, Object>();
+        Map<String, Object> retval=super.dumpStats();      
         retval.put("senders", printMap(sent));
-        retval.put("receivers", printMap(received));
-        retval.put("num_blockings", this.num_blockings);
-        retval.put("avg_time_blocked", getAverageTimeBlocked());
-        retval.put("num_replenishments", this.num_credit_responses_received);
-        retval.put("total_time_blocked", total_time_blocking);
-        retval.put("num_credit_requests", (long)num_credit_requests_sent);
+        retval.put("receivers", printMap(received));       
         return retval;
     }
 
+    @ManagedOperation(description="Print last blocking times")
     public String showLastBlockingTimes() {
         return last_blockings.toString();
     }
@@ -284,6 +369,7 @@ public class FC extends Protocol {
     /**
      * Allows to unblock a blocked sender from an external program, e.g. JMX
      */
+    @ManagedOperation(description="Unblock a sender")
     public void unblock() {
         sent_lock.lock();
         try {
@@ -303,86 +389,12 @@ public class FC extends Protocol {
         }
     }
 
-
-    public boolean setProperties(Properties props) {
-        String str;
-        boolean min_credits_set=false;
-
-        super.setProperties(props);
-        str=props.getProperty("max_credits");
-        if(str != null) {
-            max_credits=Long.parseLong(str);
-            props.remove("max_credits");
-        }
-
-        str=props.getProperty("min_threshold");
-        if(str != null) {
-            min_threshold=Double.parseDouble(str);
-            props.remove("min_threshold");
-        }
-
-        str=props.getProperty("min_credits");
-        if(str != null) {
-            min_credits=Long.parseLong(str);
-            props.remove("min_credits");
-            min_credits_set=true;
-        }
-
+    public void init() throws Exception {
+        boolean min_credits_set = min_credits != 0;
         if(!min_credits_set)
-            min_credits=(long)((double)max_credits * min_threshold);
-
-        str=props.getProperty("max_block_time");
-        if(str != null) {
-            max_block_time=Long.parseLong(str);
-            props.remove("max_block_time");
-        }
-
-        str=props.getProperty("max_block_times");
-        if(str != null) {
-            Long prev_key=null, prev_val=null;
-            List<String> vals=Util.parseCommaDelimitedStrings(str);
-            max_block_times=new TreeMap<Long,Long>();
-            for(String tmp: vals) {
-                int index=tmp.indexOf(':');
-                if(index == -1)
-                    throw new IllegalArgumentException("element '" + tmp + "'  is missing a ':' separator");
-                Long key=Long.parseLong(tmp.substring(0, index).trim());
-                Long val=Long.parseLong(tmp.substring(index +1).trim());
-
-                // sanity checks:
-                if(key < 0 || val < 0)
-                    throw new IllegalArgumentException("keys and values must be >= 0");
-
-                if(prev_key != null) {
-                    if(key <= prev_key)
-                        throw new IllegalArgumentException("keys are not sorted: " + vals);
-                }
-                prev_key=key;
-
-                 if(prev_val != null) {
-                    if(val <= prev_val)
-                        throw new IllegalArgumentException("values are not sorted: " + vals);
-                }
-                prev_val=val;
-                max_block_times.put(key, val);
-            }
-            if(log.isDebugEnabled())
-                log.debug("max_block_times: " + max_block_times);
-            props.remove("max_block_times");
-        }
-
-        Util.checkBufferSize("FC.max_credits", max_credits);
-        str=props.getProperty("ignore_synchronous_response");
-        if(str != null) {
-            ignore_synchronous_response=Boolean.valueOf(str);
-            props.remove("ignore_synchronous_response");
-        }
-
-        if(!props.isEmpty()) {
-            log.error("the following properties are not recognized: " + props);
-            return false;
-        }
-        return true;
+            min_credits=(long)(max_credits * min_threshold);
+        
+        Util.checkBufferSize("FC.max_credits", max_credits);       
     }
 
     public void start() throws Exception {
@@ -420,9 +432,9 @@ public class FC extends Protocol {
         switch(evt.getType()) {
             case Event.MSG:
                 return handleDownMessage(evt);
-            case Event.INFO:
-                handleInfo((Map<String,Object>)evt.getArg());
-                return null;
+            case Event.CONFIG:
+                handleConfigEvent((Map<String,Object>)evt.getArg()); 
+                break;
             case Event.VIEW_CHANGE:
                 handleViewChange(((View)evt.getArg()).getMembers());
                 break;
@@ -479,16 +491,16 @@ public class FC extends Protocol {
                 handleViewChange(((View)evt.getArg()).getMembers());
                 break;
 
-            case Event.INFO:
+            case Event.CONFIG:
                 Map<String,Object> map=(Map<String,Object>)evt.getArg();
-                handleInfo(map);
+                handleConfigEvent(map);
                 break;
         }
         return up_prot.up(evt);
     }
 
 
-    private void handleInfo(Map<String,Object> info) {
+    private void handleConfigEvent(Map<String,Object> info) {
         if(info != null) {
             Integer frag_size=(Integer)info.get("frag_size");
             if(frag_size != null) {
