@@ -4,6 +4,10 @@ package org.jgroups.protocols;
 
 import org.jgroups.*;
 import org.jgroups.annotations.GuardedBy;
+import org.jgroups.annotations.MBean;
+import org.jgroups.annotations.ManagedAttribute;
+import org.jgroups.annotations.ManagedOperation;
+import org.jgroups.annotations.Property;
 import org.jgroups.stack.Protocol;
 import org.jgroups.util.*;
 
@@ -16,71 +20,108 @@ import java.util.concurrent.locks.ReentrantLock;
 
 
 /**
- * Failure detection based on simple heartbeat protocol. Regularly polls members for
- * liveness. Multicasts SUSPECT messages when a member is not reachable. The simple
- * algorithms works as follows: the membership is known and ordered. Each HB protocol
- * periodically sends an 'are-you-alive' message to its *neighbor*. A neighbor is the next in
- * rank in the membership list, which is recomputed upon a view change. When a response hasn't
- * been received for n milliseconds and m tries, the corresponding member is suspected (and
- * eventually excluded if faulty).<p>
- * FD starts when it detects (in a view change notification) that there are at least
- * 2 members in the group. It stops running when the membership drops below 2.<p>
- * When a message is received from the monitored neighbor member, it causes the pinger thread to
- * 'skip' sending the next are-you-alive message. Thus, traffic is reduced.<p>
- * When we receive a ping from a member that's not in the membership list, we shun it by sending it a
- * NOT_MEMBER message. That member will then leave the group (and possibly rejoin). This is only done if
- * <code>shun</code> is true.
+ * Failure detection based on simple heartbeat protocol. Regularly polls members
+ * for liveness. Multicasts SUSPECT messages when a member is not reachable. The
+ * simple algorithms works as follows: the membership is known and ordered. Each
+ * HB protocol periodically sends an 'are-you-alive' message to its *neighbor*.
+ * A neighbor is the next in rank in the membership list, which is recomputed
+ * upon a view change. When a response hasn't been received for n milliseconds
+ * and m tries, the corresponding member is suspected (and eventually excluded
+ * if faulty).
+ * <p>
+ * FD starts when it detects (in a view change notification) that there are at
+ * least 2 members in the group. It stops running when the membership drops
+ * below 2.
+ * <p>
+ * When a message is received from the monitored neighbor member, it causes the
+ * pinger thread to 'skip' sending the next are-you-alive message. Thus, traffic
+ * is reduced.
+ * <p>
+ * When we receive a ping from a member that's not in the membership list, we
+ * shun it by sending it a NOT_MEMBER message. That member will then leave the
+ * group (and possibly rejoin). This is only done if <code>shun</code> is
+ * true.
+ * 
  * @author Bela Ban
- * @version $Id: FD.java,v 1.58.2.3 2008/05/22 13:23:06 belaban Exp $
+ * @version $Id: FD.java,v 1.73 2008/10/21 12:16:15 vlada Exp $
  */
+@MBean(description="Failure detection based on simple heartbeat protocol")
 public class FD extends Protocol {
-    Address               local_addr=null;
-    long                  timeout=3000;  // number of millisecs to wait for an are-you-alive msg
-    long                  last_ack=System.currentTimeMillis();
-    int                   num_tries=0;
-    int                   max_tries=2;   // number of times to send a are-you-alive msg (tot time= max_tries*timeout)
+    
+    private final static String name="FD";
 
-    protected final Lock  lock=new ReentrantLock();
+    /* -----------------------------------------    Properties     -------------------------------------------------- */
+
+    @Property(description="Timeout to suspect a node P if neither a heartbeat nor data were received from P. Default is 3000 msec")
+    @ManagedAttribute(description="Timeout in msec to suspect a node P if neither a heartbeat nor data were received from P", writable=true)
+    long timeout=3000; 
+
+    @Property(description="Shun switch. Default is true")
+    @ManagedAttribute(description="Shun switch", writable=true)
+    boolean shun=true;
+
+    @Property(description="Number of times to send heartbeat. Default is 2")
+    @ManagedAttribute(description="Number of times to send a are-you-alive msg", writable=true)
+    int max_tries=2;
+    
+    
+    
+    /* ---------------------------------------------   JMX      ------------------------------------------------------ */
+
+    protected int num_heartbeats=0;
+    
+    protected int num_suspect_events=0;   
+
+    protected final BoundedList<Address> suspect_history=new BoundedList<Address>(20);
+
+
+    
+    
+    /* --------------------------------------------- Fields ------------------------------------------------------ */
+
+    
+    protected Address local_addr=null;
+    
+    private long last_ack=System.currentTimeMillis();
+    
+    protected int num_tries=0;
+
+    protected final Lock lock=new ReentrantLock();
 
     @GuardedBy("lock")
-    Address               ping_dest=null;
+    protected Address ping_dest=null;
 
     @GuardedBy("lock")
-    final List<Address>   members=new ArrayList<Address>();
+    protected final List<Address> members=new ArrayList<Address>();
 
     /** Members from which we select ping_dest. may be subset of {@link #members} */
     @GuardedBy("lock")
-    final List<Address>   pingable_mbrs=new ArrayList<Address>();
-
-    // number of pings from suspected mbrs
+    protected final List<Address> pingable_mbrs=new ArrayList<Address>();
+   
     @GuardedBy("lock")
-    final Map<Address,Integer>  invalid_pingers=new HashMap<Address,Integer>(7);
+    private final Map<Address,Integer> invalid_pingers=new HashMap<Address,Integer>(7);
 
-    boolean               shun=true;
-    TimeScheduler         timer=null;
+    private TimeScheduler timer=null;
 
     @GuardedBy("lock")
-    private Future        monitor_future=null;  // task that performs the actual monitoring for failure detection
-
-    protected int         num_heartbeats=0;
-    protected int         num_suspect_events=0;
-
+    private Future<?> monitor_future=null; // task that performs the actual monitoring for failure detection
+    
     /** Transmits SUSPECT message until view change or UNSUSPECT is received */
-    protected final Broadcaster  bcast_task=new Broadcaster();
-    final static String   name="FD";
-
-    final BoundedList<Address>   suspect_history=new BoundedList<Address>(20);
-
-
-
-
+    protected final Broadcaster bcast_task=new Broadcaster();
+    
 
     public String getName() {return name;}
+    @ManagedAttribute(description="Member address")
     public String getLocalAddress() {return local_addr != null? local_addr.toString() : "null";}
+    @ManagedAttribute(description="List of cluster members")
     public String getMembers() {return members != null? members.toString() : "null";}
+    @ManagedAttribute(description="List of pingable members of a cluster")
     public String getPingableMembers() {return pingable_mbrs != null? pingable_mbrs.toString() : "null";}
+    @ManagedAttribute(description="Ping destination")
     public String getPingDest() {return ping_dest != null? ping_dest.toString() : "null";}
+    @ManagedAttribute(description="Number of heartbeats sent")    
     public int getNumberOfHeartbeatsSent() {return num_heartbeats;}
+    @ManagedAttribute(description="Number of suspect events received")
     public int getNumSuspectEventsGenerated() {return num_suspect_events;}
     public long getTimeout() {return timeout;}
     public void setTimeout(long timeout) {this.timeout=timeout;}
@@ -89,43 +130,14 @@ public class FD extends Protocol {
     public int getCurrentNumTries() {return num_tries;}
     public boolean isShun() {return shun;}
     public void setShun(boolean flag) {this.shun=flag;}
+    @ManagedOperation(description="Print suspect history")
     public String printSuspectHistory() {
         StringBuilder sb=new StringBuilder();
         for(Address addr: suspect_history) {
             sb.append(new Date()).append(": ").append(addr).append("\n");
         }
         return sb.toString();
-    }
-
-
-    public boolean setProperties(Properties props) {
-        String str;
-
-        super.setProperties(props);
-        str=props.getProperty("timeout");
-        if(str != null) {
-            timeout=Long.parseLong(str);
-            props.remove("timeout");
-        }
-
-        str=props.getProperty("max_tries");  // before suspecting a member
-        if(str != null) {
-            max_tries=Integer.parseInt(str);
-            props.remove("max_tries");
-        }
-
-        str=props.getProperty("shun");
-        if(str != null) {
-            shun=Boolean.valueOf(str).booleanValue();
-            props.remove("shun");
-        }
-
-        if(!props.isEmpty()) {
-            log.error("the following properties are not recognized: " + props);
-            return false;
-        }
-        return true;
-    }
+    }   
 
     public void resetStats() {
         num_heartbeats=num_suspect_events=0;
@@ -136,7 +148,7 @@ public class FD extends Protocol {
     public void init() throws Exception {
         timer=getTransport().getTimer();
         if(timer == null)
-            throw new Exception("timer cannot be retrieved from protocol stack");
+            throw new Exception("timer cannot be retrieved");
     }
 
 
@@ -447,12 +459,12 @@ public class FD extends Protocol {
             else {
                 out.writeBoolean(true);
                 out.writeInt(mbrs.size());
-                for(Iterator it=mbrs.iterator(); it.hasNext();) {
-                    Address addr=(Address)it.next();
-                    Marshaller.write(addr, out);
+                for(Iterator<Address> it=mbrs.iterator(); it.hasNext();) {
+                    Address addr=it.next();
+                    out.writeObject(addr);
                 }
             }
-            Marshaller.write(from, out);
+            out.writeObject(from);
         }
 
         public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
@@ -462,11 +474,11 @@ public class FD extends Protocol {
                 int len=in.readInt();
                 mbrs=new Vector<Address>(11);
                 for(int i=0; i < len; i++) {
-                    Address addr=(Address)Marshaller.read(in);
+                    Address addr=(Address)in.readObject();
                     mbrs.add(addr);
                 }
             }
-            from=(Address)Marshaller.read(in);
+            from=(Address)in.readObject();
         }
 
 
@@ -567,12 +579,12 @@ public class FD extends Protocol {
         final Vector<Address> suspected_mbrs=new Vector<Address>(7);
         final Lock bcast_lock=new ReentrantLock();
         @GuardedBy("bcast_lock")
-        Future bcast_future=null;
+        Future<?> bcast_future=null;
         @GuardedBy("bcast_lock")
         BroadcastTask task;
 
 
-        Vector getSuspectedMembers() {
+        Vector<Address> getSuspectedMembers() {
             return suspected_mbrs;
         }
 
@@ -640,7 +652,7 @@ public class FD extends Protocol {
 
 
         /** Removes all elements from suspected_mbrs that are <em>not</em> in the new membership */
-        void adjustSuspectedMembers(List new_mbrship) {
+        void adjustSuspectedMembers(List<Address> new_mbrship) {
             if(new_mbrship == null || new_mbrship.isEmpty()) return;
             synchronized(suspected_mbrs) {
                 suspected_mbrs.retainAll(new_mbrship);
