@@ -5,6 +5,9 @@ import org.jgroups.Address;
 import org.jgroups.Event;
 import org.jgroups.Message;
 import org.jgroups.View;
+import org.jgroups.annotations.MBean;
+import org.jgroups.annotations.ManagedAttribute;
+import org.jgroups.annotations.Property;
 import org.jgroups.stack.Protocol;
 import org.jgroups.util.ExposedByteArrayOutputStream;
 import org.jgroups.util.Util;
@@ -13,39 +16,65 @@ import java.io.ByteArrayInputStream;
 import java.io.DataInputStream;
 import java.io.DataOutputStream;
 import java.util.*;
+import java.util.Map.Entry;
 
 
 
 /**
- * Fragmentation layer. Fragments messages larger than FRAG_SIZE into smaller packets.
- * Reassembles fragmented packets into bigger ones. The fragmentation number is prepended
- * to the messages as a header (and removed at the receiving side).<p>
- * Each fragment is identified by (a) the sender (part of the message to which the header is appended),
- * (b) the fragmentation ID (which is unique per FRAG layer (monotonically increasing) and (c) the
- * fragement ID which ranges from 0 to number_of_fragments-1.<p>
- * Requirement: lossless delivery (e.g. NAK, ACK). No requirement on ordering. Works for both unicast and
- * multicast messages.
+ * Fragmentation layer. Fragments messages larger than FRAG_SIZE into smaller
+ * packets. Reassembles fragmented packets into bigger ones. The fragmentation
+ * number is prepended to the messages as a header (and removed at the receiving
+ * side).
+ * <p>
+ * Each fragment is identified by (a) the sender (part of the message to which
+ * the header is appended), (b) the fragmentation ID (which is unique per FRAG
+ * layer (monotonically increasing) and (c) the fragement ID which ranges from 0
+ * to number_of_fragments-1.
+ * <p>
+ * Requirement: lossless delivery (e.g. NAK, ACK). No requirement on ordering.
+ * Works for both unicast and multicast messages.
+ * 
  * @author Bela Ban
  * @author Filip Hanik
- * @version $Id: FRAG.java,v 1.39.2.1 2008/03/27 16:33:39 belaban Exp $
+ * @version $Id: FRAG.java,v 1.46 2008/07/22 14:07:45 belaban Exp $
  */
+@MBean(description="Fragments messages larger than fragmentation size into smaller packets")
 public class FRAG extends Protocol {
-    private int frag_size=8192;  // conservative value
+    
+    private final static String name="FRAG";
 
+    /* -----------------------------------------    Properties     -------------------------------------------------- */
+
+    @Property(description="The max number of bytes in a message. Larger messages will be fragmented. Default is 8192 bytes")
+    @ManagedAttribute(description="Fragmentation size", writable=true)
+    private int frag_size=8192; // conservative value
+
+    @Property(description="The max size in bytes for the byte array output buffer")
+    @ManagedAttribute(description="The max size in bytes the byte array output buffer should be, " + "otherwise we keep memory occupied unnecessarily", writable=true)
     private int max_retained_buffer=70000;
 
+    
+    /* --------------------------------------------- Fields ------------------------------------------------------ */
+
+    
     /*the fragmentation list contains a fragmentation table per sender
      *this way it becomes easier to clean up if a sender (member) leaves or crashes
      */
-    private final FragmentationList     fragment_list=new FragmentationList();
-    private int                         curr_id=1;
+    private final FragmentationList fragment_list=new FragmentationList();
+    private int curr_id=1;
     private final ExposedByteArrayOutputStream bos=new ExposedByteArrayOutputStream(1024);
-    private final Vector<Address>       members=new Vector<Address>(11);
-    private final static String         name="FRAG";
+    private final Vector<Address> members=new Vector<Address>(11);
+    
+    
+ 
 
+    @ManagedAttribute(description="Number of sent messages")
     long num_sent_msgs=0;
+    @ManagedAttribute(description="Number of sent fragments")
     long num_sent_frags=0;
+    @ManagedAttribute(description="Number of received messages")
     long num_received_msgs=0;
+    @ManagedAttribute(description="Number of received fragments")
     long num_received_frags=0;
 
 
@@ -59,39 +88,14 @@ public class FRAG extends Protocol {
     public long getNumberOfSentFragments() {return num_sent_frags;}
     public long getNumberOfReceivedMessages() {return num_received_msgs;}
     public long getNumberOfReceivedFragments() {return num_received_frags;}
-
-    /**
-     * Setup the Protocol instance acording to the configuration string
-     */
-    public boolean setProperties(Properties props) {
-        String str;
-        
-        super.setProperties(props);
-        str=props.getProperty("frag_size");
-        if(str != null) {
-            frag_size=Integer.parseInt(str);
-            props.remove("frag_size");
-        }
-
-        str=props.getProperty("max_retained_buffer");
-        if(str != null) {
-            max_retained_buffer=Integer.parseInt(str);
-            props.remove("max_retained_buffer");
-        }
-
-        if(!props.isEmpty()) {
-            log.error("the following properties are not recognized: " + props);
-            return false;
-        }
-        return true;
-    }
+  
 
     public void init() throws Exception {
         super.init();
         Map<String,Object> info=new HashMap<String,Object>(1);
         info.put("frag_size", frag_size);
-        up_prot.up(new Event(Event.INFO, info));
-        down_prot.down(new Event(Event.INFO, info));
+        up_prot.up(new Event(Event.CONFIG, info));
+        down_prot.down(new Event(Event.CONFIG, info));
     }
 
     public void resetStats() {
@@ -107,48 +111,30 @@ public class FRAG extends Protocol {
     public Object down(Event evt) {
         switch(evt.getType()) {
 
-        case Event.MSG:
-            Message msg=(Message)evt.getArg();
-            long size=msg.size();
-            num_sent_msgs++;
-            if(size > frag_size) {
-                if(log.isTraceEnabled()) {
-                    StringBuilder sb=new StringBuilder("message size is ");
-                    sb.append(size).append(", will fragment (frag_size=").append(frag_size).append(')');
-                    log.trace(sb.toString());
+            case Event.MSG:
+                Message msg=(Message)evt.getArg();
+                long size=msg.size();
+                num_sent_msgs++;
+                if(size > frag_size) {
+                    if(log.isTraceEnabled()) {
+                        StringBuilder sb=new StringBuilder("message size is ");
+                        sb.append(size).append(", will fragment (frag_size=").append(frag_size).append(')');
+                        log.trace(sb.toString());
+                    }
+                    fragment(msg);  // Fragment and pass down
+                    return null;
                 }
-                fragment(msg);  // Fragment and pass down
-                return null;
-            }
-            break;
+                break;
 
-        case Event.VIEW_CHANGE:
-            //don't do anything if this dude is sending out the view change
-            //we are receiving a view change,
-            //in here we check for the
-            View view=(View)evt.getArg();
-            Vector new_mbrs=view.getMembers(), left_mbrs;
-            Address mbr;
+            case Event.VIEW_CHANGE:
+                handleViewChange((View)evt.getArg());
+                break;
 
-            left_mbrs=Util.determineLeftMembers(members, new_mbrs);
-            members.clear();
-            members.addAll(new_mbrs);
-
-            for(int i=0; i < left_mbrs.size(); i++) {
-                mbr=(Address)left_mbrs.elementAt(i);
-                //the new view doesn't contain the sender, he must have left,
-                //hence we will clear all his fragmentation tables
-                fragment_list.remove(mbr);
-                if(log.isTraceEnabled())
-                    log.trace("[VIEW_CHANGE] removed " + mbr + " from fragmentation table");
-            }
-            break;
-
-        case Event.CONFIG:
-            Object ret=down_prot.down(evt);
-            if(log.isDebugEnabled()) log.debug("received CONFIG event: " + evt.getArg());
-            handleConfigEvent((Map<String,Object>)evt.getArg());
-            return ret;
+            case Event.CONFIG:
+                Object ret=down_prot.down(evt);
+                if(log.isDebugEnabled()) log.debug("received CONFIG event: " + evt.getArg());
+                handleConfigEvent((Map<String,Object>)evt.getArg());
+                return ret;
         }
 
         return down_prot.down(evt);  // Pass on to the layer below us
@@ -161,29 +147,48 @@ public class FRAG extends Protocol {
     public Object up(Event evt) {
         switch(evt.getType()) {
 
-        case Event.MSG:
-            Message msg=(Message)evt.getArg();
-            FragHeader hdr=(FragHeader)msg.getHeader(name);
-            if(hdr != null) { // needs to be defragmented
-                unfragment(msg, hdr); // Unfragment and possibly pass up
-                return null;
-            }
-            else {
-                num_received_msgs++;
-            }
-            break;
+            case Event.MSG:
+                Message msg=(Message)evt.getArg();
+                FragHeader hdr=(FragHeader)msg.getHeader(name);
+                if(hdr != null) { // needs to be defragmented
+                    unfragment(msg, hdr); // Unfragment and possibly pass up
+                    return null;
+                }
+                else {
+                    num_received_msgs++;
+                }
+                break;
 
-        case Event.CONFIG:
-            Object ret=up_prot.up(evt);
-            if(log.isDebugEnabled()) log.debug("received CONFIG event: " + evt.getArg());
-            handleConfigEvent((Map<String,Object>)evt.getArg());
-            return ret;
+            case Event.VIEW_CHANGE:
+                handleViewChange((View)evt.getArg());
+                break;
+
+            case Event.CONFIG:
+                Object ret=up_prot.up(evt);
+                if(log.isDebugEnabled()) log.debug("received CONFIG event: " + evt.getArg());
+                handleConfigEvent((Map<String,Object>)evt.getArg());
+                return ret;
         }
 
         return up_prot.up(evt); // Pass up to the layer above us by default
     }
 
 
+    private void handleViewChange(View view) {
+        Vector<Address> new_mbrs=view.getMembers(), left_mbrs;
+        left_mbrs=Util.determineLeftMembers(members, new_mbrs);
+        members.clear();
+        members.addAll(new_mbrs);
+
+        for(Address mbr: left_mbrs){
+            //the new view doesn't contain the sender, he must have left,
+            //hence we will clear all his fragmentation tables
+            fragment_list.remove(mbr);
+            if(log.isTraceEnabled())
+                log.trace("[VIEW_CHANGE] removed " + mbr + " from fragmentation table");
+        }
+    }
+
     /**
      * Send all fragments as separate messages (with same ID !).
      * Example:
@@ -319,7 +324,7 @@ public class FRAG extends Protocol {
          * 11 is the best growth capacity to start with<br/>
          * HashMap<Address,FragmentationTable>
          */
-        private final HashMap frag_tables=new HashMap(11);
+        private final HashMap<Address,FragmentationTable> frag_tables=new HashMap<Address,FragmentationTable>(11);
 
 
         /**
@@ -331,10 +336,9 @@ public class FRAG extends Protocol {
          * @throws IllegalArgumentException if an entry for this sender already exist
          */
         public void add(Address sender, FragmentationTable table) throws IllegalArgumentException {
-            FragmentationTable healthCheck;
-
+            
             synchronized(frag_tables) {
-                healthCheck=(FragmentationTable)frag_tables.get(sender);
+                FragmentationTable healthCheck=frag_tables.get(sender);
                 if(healthCheck == null) {
                     frag_tables.put(sender, table);
                 }
@@ -351,7 +355,7 @@ public class FRAG extends Protocol {
          */
         public FragmentationTable get(Address sender) {
             synchronized(frag_tables) {
-                return (FragmentationTable)frag_tables.get(sender);
+                return frag_tables.get(sender);
             }
         }
 
@@ -393,20 +397,19 @@ public class FRAG extends Protocol {
 
             synchronized(frag_tables) {
                 result=new Address[frag_tables.size()];
-                for(Iterator it=frag_tables.keySet().iterator(); it.hasNext();) {
-                    result[index++]=(Address)it.next();
+                for(Iterator<Address> it=frag_tables.keySet().iterator(); it.hasNext();) {
+                    result[index++]=it.next();
                 }
             }
             return result;
         }
 
-        public String toString() {
-            Map.Entry entry;
+        public String toString() {          
             StringBuilder buf=new StringBuilder("Fragmentation list contains ");
             synchronized(frag_tables) {
                 buf.append(frag_tables.size()).append(" tables\n");
-                for(Iterator it=frag_tables.entrySet().iterator(); it.hasNext();) {
-                    entry=(Map.Entry)it.next();
+                for(Iterator<Entry<Address,FragmentationTable>> it=frag_tables.entrySet().iterator(); it.hasNext();) {
+                    Entry<Address,FragmentationTable> entry=it.next();
                     buf.append(entry.getKey()).append(": " ).append(entry.getValue()).append("\n");
                 }
             }
@@ -424,7 +427,7 @@ public class FRAG extends Protocol {
     static class FragmentationTable {
         private final Address sender;
         /* the hashtable that holds the fragmentation entries for this sender*/
-        private final Hashtable h=new Hashtable(11);  // keys: frag_ids, vals: Entrys
+        private final Hashtable<Long,FragEntry> h=new Hashtable<Long,FragEntry>(11);  // keys: frag_ids, vals: Entrys
 
 
         FragmentationTable(Address sender) {
@@ -438,7 +441,7 @@ public class FRAG extends Protocol {
          * once all the byte buffer entries have been filled
          * the fragmentation is considered complete.
          */
-        static class Entry {
+        static class FragEntry {
             //the total number of fragment in this message
             int tot_frags=0;
             // each fragment is a byte buffer
@@ -453,7 +456,7 @@ public class FRAG extends Protocol {
              *
              * @param tot_frags the number of fragments to expect for this message
              */
-            Entry(long msg_id, int tot_frags) {
+            FragEntry(long msg_id, int tot_frags) {
                 this.msg_id=msg_id;
                 this.tot_frags=tot_frags;
                 fragments=new byte[tot_frags][];
@@ -532,10 +535,10 @@ public class FRAG extends Protocol {
             /*initialize the return value to default not complete */
             byte[] retval=null;
 
-            Entry e=(Entry)h.get(new Long(id));
+            FragEntry e=h.get(new Long(id));
 
             if(e == null) {   // Create new entry if not yet present
-                e=new Entry(id, tot_frags);
+                e=new FragEntry(id, tot_frags);
                 h.put(new Long(id), e);
             }
 
@@ -553,9 +556,9 @@ public class FRAG extends Protocol {
 
         public String toString() {
             StringBuilder buf=new StringBuilder("Fragmentation Table Sender:").append(sender).append("\n\t");
-            java.util.Enumeration e=this.h.elements();
+            Enumeration<FragEntry> e=this.h.elements();
             while(e.hasMoreElements()) {
-                Entry entry=(Entry)e.nextElement();
+                FragEntry entry=e.nextElement();
                 int count=0;
                 for(int i=0; i < entry.fragments.length; i++) {
                     if(entry.fragments[i] != null) {
