@@ -3,9 +3,10 @@ package org.jgroups.protocols;
 import org.jgroups.Event;
 import org.jgroups.Global;
 import org.jgroups.Message;
-import org.jgroups.util.Buffer;
-import org.jgroups.util.ExposedByteArrayOutputStream;
-import org.jgroups.util.Util;
+import org.jgroups.annotations.ManagedAttribute;
+import org.jgroups.annotations.Property;
+import org.jgroups.conf.PropertyConverters;
+import org.jgroups.util.*;
 
 import java.io.*;
 import java.net.*;
@@ -19,53 +20,93 @@ import java.util.*;
  * back via the regular transport (e.g. TCP) to the sender (discovery request contained sender's regular address,
  * e.g. 192.168.0.2:7800).
  * @author Bela Ban
- * @version $Id: MPING.java,v 1.29.2.5 2008/11/11 10:54:49 belaban Exp $
+ * @version $Id: MPING.java,v 1.46 2009/01/05 08:24:06 belaban Exp $
  */
 public class MPING extends PING implements Runnable {
-    MulticastSocket        mcast_sock=null;
-
-    /** If we have multiple mcast send sockets, e.g. send_interfaces or send_on_all_interfaces enabled */
-    MulticastSocket[]      mcast_send_sockets=null;
-    Thread                 receiver=null;
-    InetAddress            bind_addr=null;
-    int                    ip_ttl=8;
-    InetAddress            mcast_addr=null;
-    int                    mcast_port=7555;
-
-     /** If true, the transport should use all available interfaces to receive multicast messages */
-    boolean                receive_on_all_interfaces=false;
-
-     /** List<NetworkInterface> of interfaces to receive multicasts on. The multicast receive socket will listen
-     * on all of these interfaces. This is a comma-separated list of IP addresses or interface names. E.g.
-     * "192.168.5.1,eth1,127.0.0.1". Duplicates are discarded; we only bind to an interface once.
-     * If this property is set, it override receive_on_all_interfaces.
-     */
-    List<NetworkInterface> receive_interfaces=null;
-
-    /** If true, the transport should use all available interfaces to send multicast messages. This means
-     * the same multicast message is sent N times, so use with care */
-    boolean                send_on_all_interfaces=false;
-
-    /** List<NetworkInterface> of interfaces to send multicasts on. The multicast send socket will send the
-     * same multicast message on all of these interfaces. This is a comma-separated list of IP addresses or
-     * interface names. E.g. "192.168.5.1,eth1,127.0.0.1". Duplicates are discarded.
-     * If this property is set, it override send_on_all_interfaces.
-     */
-    List<NetworkInterface> send_interfaces=null;
-
-    /** Pre-allocated byte stream. Used for serializing datagram packets. Will grow as needed */
-    final ExposedByteArrayOutputStream out_stream=new ExposedByteArrayOutputStream(128);
-    byte                receive_buf[]=new byte[1024];
-
-
-    private static final boolean is_linux; // are we running on Linux ?
+    
+    private static final boolean can_bind_to_mcast_addr; // are we running on Linux ?
 
 
     static {
-        is_linux=Util.checkForLinux();
+        can_bind_to_mcast_addr=Util.checkForLinux() || Util.checkForSolaris();
     }
 
 
+    /* -----------------------------------------    Properties     -------------------------------------------------- */
+
+    @ManagedAttribute(description="Bind address for multicast socket", writable=true)
+    @Property(converter=PropertyConverters.BindAddress.class, description="Bind address for multicast socket")
+    InetAddress bind_addr=null;
+    
+    @Property(description="Time to live for discovery packets. Default is 8")
+    @ManagedAttribute(description="Time to live for discovery packets", writable=true)
+    int ip_ttl=8;
+
+    @ManagedAttribute(description="Multicast address for discovery packets", writable=true)
+    InetAddress mcast_addr=null;
+
+    @Property(description="Multicast port for discovery packets. Default is 7555")
+    @ManagedAttribute(description="Multicast port for discovery packets", writable=true)
+    int mcast_port=7555;
+
+    @Property(name="bind_to_all_interfaces", deprecatedMessage="bind_to_all_interfaces has been deprecated; use receive_on_all_interfaces instead", description="If true, the transport should use all available interfaces to receive multicast messages. Default is false")
+    boolean receive_on_all_interfaces=false;
+
+    /**
+     * List<NetworkInterface> of interfaces to receive multicasts on. The
+     * multicast receive socket will listen on all of these interfaces. This is
+     * a comma-separated list of IP addresses or interface names. E.g.
+     * "192.168.5.1,eth1,127.0.0.1". Duplicates are discarded; we only bind to
+     * an interface once. If this property is set, it override
+     * receive_on_all_interfaces.
+     */
+    @Property(converter=PropertyConverters.NetworkInterfaceList.class, description="List of interfaces to receive multicasts on")
+    List<NetworkInterface> receive_interfaces=null;
+
+    /**
+     * If true, the transport should use all available interfaces to send
+     * multicast messages. This means the same multicast message is sent N
+     * times, so use with care
+     */
+    @Property(description="Whether send messages are sent on all interfaces. Default is false")
+    boolean send_on_all_interfaces=false;
+
+    /**
+     * List<NetworkInterface> of interfaces to send multicasts on. The
+     * multicast send socket will send the same multicast message on all of
+     * these interfaces. This is a comma-separated list of IP addresses or
+     * interface names. E.g. "192.168.5.1,eth1,127.0.0.1". Duplicates are
+     * discarded. If this property is set, it override send_on_all_interfaces.
+     */
+    @Property(converter=PropertyConverters.NetworkInterfaceList.class, description="List of interfaces to send multicasts on")
+    List<NetworkInterface> send_interfaces=null;
+    
+    
+    
+    /* --------------------------------------------- Fields ------------------------------------------------------ */
+
+    
+    private MulticastSocket mcast_sock=null;
+
+    /**
+     * If we have multiple mcast send sockets, e.g. send_interfaces or
+     * send_on_all_interfaces enabled
+     */
+    private MulticastSocket[] mcast_send_sockets=null;
+    
+    private volatile Thread receiver=null;
+
+    /**
+     * Pre-allocated byte stream. Used for serializing datagram packets. Will
+     * grow as needed
+     */
+    private final ExposedByteArrayOutputStream out_stream=new ExposedByteArrayOutputStream(128);
+    
+    private final byte receive_buf[]=new byte[1024];
+
+
+    public MPING() {        
+    }
 
     public String getName() {
         return "MPING";
@@ -79,18 +120,22 @@ public class MPING extends PING implements Runnable {
         this.bind_addr=bind_addr;
     }
 
+    @ManagedAttribute(description="Receive interfaces")
     public List<NetworkInterface> getReceiveInterfaces() {
         return receive_interfaces;
     }
 
+    @ManagedAttribute(description="Send interfaces")
     public List<NetworkInterface> getSendInterfaces() {
         return send_interfaces;
     }
 
+    @ManagedAttribute
     public boolean isReceiveOnAllInterfaces() {
         return receive_on_all_interfaces;
     }
 
+    @ManagedAttribute
     public boolean isSendOnAllInterfaces() {
         return send_on_all_interfaces;
     }
@@ -111,6 +156,11 @@ public class MPING extends PING implements Runnable {
         this.mcast_addr=mcast_addr;
     }
 
+    @Property(name="mcast_addr")
+    public void setMulticastAddress(String addr) throws UnknownHostException {
+        mcast_addr=InetAddress.getByName(addr);
+    }
+
     public int getMcastPort() {
         return mcast_port;
     }
@@ -120,100 +170,6 @@ public class MPING extends PING implements Runnable {
     }
 
 
-    public boolean setProperties(Properties props) {
-        String str;
-
-        try {
-            bind_addr=Util.getBindAddress(props);
-        }
-        catch(UnknownHostException unknown) {
-            log.fatal("failed getting bind_addr", unknown);
-            return false;
-        }
-        catch(SocketException ex) {
-            log.fatal("failed getting bind_addr", ex);
-            return false;
-        }
-
-        str=Util.getProperty(new String[]{Global.MPING_MCAST_ADDR}, props, "mcast_addr", false, "230.5.6.7");
-        if(str != null) {
-            try {
-                mcast_addr=InetAddress.getByName(str);
-            }
-            catch(UnknownHostException e) {
-                log.error("could not resolve " + str, e);
-                return false;
-            }
-            props.remove("mcast_addr");
-        }
-
-        str=Util.getProperty(new String[]{Global.MPING_MCAST_PORT}, props, "mcast_port", false, "7555");
-        if(str != null) {
-            mcast_port=Integer.parseInt(str);
-            props.remove("mcast_port");
-        }
-
-        str=Util.getProperty(new String[]{Global.MPING_IP_TTL}, props, "ip_ttl", false, "16");
-        if(str != null) {
-            ip_ttl=Integer.parseInt(str);
-            props.remove("ip_ttl");
-        }
-
-        str=props.getProperty("bind_to_all_interfaces");
-        if(str != null) {
-            receive_on_all_interfaces=Boolean.parseBoolean(str);
-            props.remove("bind_to_all_interfaces");
-            log.warn("bind_to_all_interfaces has been deprecated; use receive_on_all_interfaces instead");
-            props.remove("bind_to_all_interfaces");
-        }
-
-        str=props.getProperty("receive_on_all_interfaces");
-        if(str != null) {
-            receive_on_all_interfaces=Boolean.parseBoolean(str);
-            props.remove("receive_on_all_interfaces");
-        }
-
-        str=props.getProperty("receive_interfaces");
-        if(str != null) {
-            try {
-                receive_interfaces=Util.parseInterfaceList(str);
-                props.remove("receive_interfaces");
-            }
-            catch(Exception e) {
-                log.error("error determining interfaces (" + str + ")", e);
-                return false;
-            }
-        }
-
-        str=props.getProperty("send_on_all_interfaces");
-        if(str != null) {
-            send_on_all_interfaces=Boolean.parseBoolean(str);
-            props.remove("send_on_all_interfaces");
-        }
-
-        str=props.getProperty("send_interfaces");
-        if(str != null) {
-            try {
-                send_interfaces=Util.parseInterfaceList(str);
-                props.remove("send_interfaces");
-            }
-            catch(Exception e) {
-                log.error("error determining interfaces (" + str + ")", e);
-                return false;
-            }
-        }
-
-        if(mcast_addr == null) {
-            try {
-                mcast_addr=InetAddress.getByName("230.5.6.7");
-            }
-            catch(UnknownHostException e) {
-                log.error("failed getting default mcast address", e);
-                return false;
-            }
-        }
-        return super.setProperties(props);
-    }
 
 
     public Object up(Event evt) {
@@ -228,12 +184,32 @@ public class MPING extends PING implements Runnable {
     }
 
 
+    public void init() throws Exception {
+        super.init();
+
+        String str=Util.getProperty(new String[]{Global.MPING_MCAST_ADDR}, null, "mcast_addr", false, null);
+        if(str != null)
+            mcast_addr=InetAddress.getByName(str);
+
+        str=Util.getProperty(new String[]{Global.MPING_MCAST_PORT}, null, "mcast_port", false, null);
+        if(str != null)
+            mcast_port=Integer.parseInt(str);
+
+        str=Util.getProperty(new String[]{Global.MPING_IP_TTL}, null, "ip_ttl", false, null);
+        if(str != null)
+            ip_ttl=Integer.parseInt(str);
+
+        if(mcast_addr == null) {
+            mcast_addr=InetAddress.getByName("230.5.6.7");
+        }
+    }
+
     public void start() throws Exception {
-        if(is_linux) // https://jira.jboss.org/jira/browse/JGRP-836 - prevent cross talking on Linux
+        if(can_bind_to_mcast_addr) // https://jira.jboss.org/jira/browse/JGRP-836 - prevent cross talking on Linux
             mcast_sock=Util.createMulticastSocket(mcast_addr, mcast_port, log);
         else
             mcast_sock=new MulticastSocket(mcast_port);
-
+        
         mcast_sock.setTimeToLive(ip_ttl);
 
         if(receive_on_all_interfaces || (receive_interfaces != null && !receive_interfaces.isEmpty())) {
@@ -364,7 +340,7 @@ public class MPING extends PING implements Runnable {
             try {
                 mcast_sock.receive(packet);
                 data=packet.getData();
-                inp_stream=new ByteArrayInputStream(data, 0, data.length);
+                inp_stream=new ExposedByteArrayInputStream(data, 0, data.length);
                 inp=new DataInputStream(inp_stream);
                 msg=new Message();
                 msg.readFrom(inp);
@@ -377,16 +353,12 @@ public class MPING extends PING implements Runnable {
                 log.error("failed receiving packet (from " + packet.getSocketAddress() + ")", ex);
             }
             finally {
-                closeInputStream(inp);
-                closeInputStream(inp_stream);
+                Util.close(inp);
             }
         }
         if(log.isTraceEnabled())
             log.trace("receiver thread terminated");
     }
 
-    private static void closeInputStream(InputStream inp) {
-        if(inp != null)
-            try {inp.close();} catch(Throwable e) {}
-    }
+
 }
