@@ -1,4 +1,4 @@
-// $Id: TUNNEL.java,v 1.45.2.4 2008/10/30 14:02:54 belaban Exp $
+// $Id: TUNNEL.java,v 1.51 2008/10/31 08:38:44 belaban Exp $
 
 package org.jgroups.protocols;
 
@@ -6,6 +6,7 @@ import org.jgroups.Address;
 import org.jgroups.Event;
 import org.jgroups.Message;
 import org.jgroups.annotations.GuardedBy;
+import org.jgroups.annotations.Property;
 import org.jgroups.stack.RouterStub;
 import org.jgroups.stack.IpAddress;
 import org.jgroups.util.Util;
@@ -13,8 +14,6 @@ import org.jgroups.util.Util;
 import java.io.DataInputStream;
 import java.io.IOException;
 import java.net.SocketException;
-import java.util.Enumeration;
-import java.util.Properties;
 import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.locks.Lock;
@@ -39,13 +38,24 @@ import java.util.concurrent.locks.ReentrantLock;
  * @author Bela Ban
  */
 public class TUNNEL extends TP {
+    
+    /* -----------------------------------------    Properties     -------------------------------------------------- */
+    
+    @Property(description="Router host address")
     private String router_host = null;
 
-    private int router_port = 0;
+    @Property(description="Router port")
+    private int router_port = 0;    
 
+    @Property(description="Interval in msec to attempt connecting back to router in case of torn connection. Default is 5000 msec")
+    private long reconnect_interval = 5000;
+    
+    
+    
+    /* --------------------------------------------- Fields ------------------------------------------------------ */
+    
+    
     private RouterStub stub;
-
-    long reconnect_interval = 5000;
     
     /*
      * flag indicating if tunnel was destroyed intentionally (disconnect, channel destroy etc)
@@ -99,6 +109,14 @@ public class TUNNEL extends TP {
         super.init();
         if(timer == null)
             throw new Exception("TUNNEL.init(): timer cannot be retrieved from protocol stack");
+        
+        if(log.isDebugEnabled()){
+            log.debug("router_host=" + router_host + ";router_port=" + router_port);
+        }
+
+        if(router_host == null || router_port == 0){
+            throw new Exception("both router_host and router_port have to be set !");            
+        }
     }
 
     public void start() throws Exception {
@@ -120,55 +138,6 @@ public class TUNNEL extends TP {
         local_addr = null;
     }
 
-    /** Setup the Protocol instance acording to the configuration string */
-    public boolean setProperties(Properties props) {
-        String str;
-
-        super.setProperties(props);
-        str = props.getProperty("router_host");
-        if(str != null){
-            router_host = str;
-            props.remove("router_host");
-        }
-
-        str = props.getProperty("router_port");
-        if(str != null){
-            router_port = Integer.parseInt(str);
-            props.remove("router_port");
-        }
-
-        if(log.isDebugEnabled()){
-            log.debug("router_host=" + router_host + ";router_port=" + router_port);
-        }
-
-        if(router_host == null || router_port == 0){
-            if(log.isErrorEnabled()){
-                log.error("both router_host and router_port have to be set !");
-                return false;
-            }
-        }
-
-        str = props.getProperty("reconnect_interval");
-        if(str != null){
-            reconnect_interval = Long.parseLong(str);
-            props.remove("reconnect_interval");
-        }
-
-        if(!props.isEmpty()){
-            StringBuilder sb = new StringBuilder();
-            for(Enumeration e = props.propertyNames();e.hasMoreElements();){
-                sb.append(e.nextElement().toString());
-                if(e.hasMoreElements()){
-                    sb.append(", ");
-                }
-            }
-            if(log.isErrorEnabled())
-                log.error("The following properties are not recognized: " + sb);
-            return false;
-        }
-        return true;
-    }
-
     /** Tears the TCP connection to the router down */
     void teardownTunnel() {
         intentionallyTornDown = true;
