@@ -1,7 +1,11 @@
-
 package org.jgroups.protocols;
 
 import org.jgroups.*;
+import org.jgroups.annotations.MBean;
+import org.jgroups.annotations.ManagedAttribute;
+import org.jgroups.annotations.ManagedOperation;
+import org.jgroups.annotations.Property;
+import org.jgroups.conf.PropertyConverters;
 import org.jgroups.stack.AckReceiverWindow;
 import org.jgroups.stack.AckSenderWindow;
 import org.jgroups.stack.Protocol;
@@ -12,9 +16,13 @@ import org.jgroups.util.TimeScheduler;
 import org.jgroups.util.Util;
 
 import java.io.*;
-import java.util.*;
-import java.util.concurrent.locks.ReentrantLock;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.Vector;
 import java.util.concurrent.atomic.AtomicInteger;
+import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.concurrent.locks.ReentrantLock;
 
 
 /**
@@ -34,57 +42,112 @@ import java.util.concurrent.atomic.AtomicInteger;
  * whenever a message is received: the new message is added and then we try to remove as many messages as
  * possible (until we stop at a gap, or there are no more messages).
  * @author Bela Ban
- * @version $Id: UNICAST.java,v 1.91.2.14 2008/06/17 08:21:26 belaban Exp $
+ * @version $Id: UNICAST.java,v 1.118 2008/10/21 08:19:08 vlada Exp $
  */
+@MBean(description="Reliable unicast layer")
 public class UNICAST extends Protocol implements AckSenderWindow.RetransmitCommand {
-    private final Vector<Address> members=new Vector<Address>(11);
-    private final HashMap<Address,Entry> connections=new HashMap<Address,Entry>(11);
-    private long[]                timeouts={400,800,1600,3200};  // for AckSenderWindow: max time to wait for missing acks
-    private Address               local_addr=null;
-    private TimeScheduler         timer=null;                    // used for retransmissions (passed to AckSenderWindow)
-    private Map<Thread,ReentrantLock> locks;
 
-    // if UNICAST is used without GMS, don't consult the membership on retransmit() if use_gms=false
-    // default is true
-    private boolean          use_gms=true;
-    private boolean          started=false;
+    
+    private static final String name="UNICAST";
+    
+    private static final long DEFAULT_FIRST_SEQNO=1;   
+    
+    
+    /* ------------------------------------------ Properties  ------------------------------------------ */    
+    
+    
+    @Property(description="If GMS should be consulted for the membership on retransmit. Default is true")
+    private boolean use_gms=true;
 
-    // ack a message before it is processed by the application to limit unnecessary retransmits
-    private boolean          immediate_ack=false;
+    @Property(description="Acks a message before it is processed by the application to limit unnecessary retransmits. Default is false")
+    private boolean immediate_ack=false;
 
-    /** whether to loop back messages sent to self (will be removed in the future, default=false) */
-    private boolean          loopback=false;
+    @Property(description="Whether to loop back messages sent to self. Default is false")
+    private boolean loopback=false;
+    
+    
+    private long[] timeout= { 400, 800, 1600, 3200 }; // for AckSenderWindow: max time to wait for missing acks
 
     /**
-     * By default, we release the lock on the sender in up() after the up() method call passed up the stack returns.
-     * However, with eager_lock_release enabled (default), we release the lock as soon as the application calls
-     * Channel.down() <em>within</em> the receive() callback. This leads to issues as the one described in
-     * http://jira.jboss.com/jira/browse/JGRP-656. Note that ordering is <em>still correct </em>, but messages from self
-     * might get delivered concurrently. This can be turned off by setting eager_lock_release to false.
+     * By default, we release the lock on the sender in up() after the up()
+     * method call passed up the stack returns. However, with eager_lock_release
+     * enabled (default), we release the lock as soon as the application calls
+     * Channel.down() <em>within</em> the receive() callback. This leads to
+     * issues as the one described in
+     * http://jira.jboss.com/jira/browse/JGRP-656. Note that ordering is
+     * <em>still correct </em>, but messages from self might get delivered
+     * concurrently. This can be turned off by setting eager_lock_release to
+     * false.
      */
+    @Property(description="See http://jira.jboss.com/jira/browse/JGRP-656. Default is true")
     private boolean eager_lock_release=true;
+    
+    
+    
 
-    /** A list of members who left, used to determine when to prevent sending messages to left mbrs */
-    private final BoundedList<Address> previous_members=new BoundedList<Address>(50);
-    /** Contains all members that were enabled for unicasts by Event.ENABLE_UNICAST_TO */
-    private final BoundedList<Address> enabled_members=new BoundedList<Address>(100);
-
-    private final static String name="UNICAST";
-    private static final long DEFAULT_FIRST_SEQNO=1;
-
+    /* --------------------------------------------- JMX  ---------------------------------------------- */    
+    
+    
     private long num_msgs_sent=0, num_msgs_received=0, num_bytes_sent=0, num_bytes_received=0;
     private long num_acks_sent=0, num_acks_received=0, num_xmit_requests_received=0;
 
+    
+    
+    /* --------------------------------------------- Fields ------------------------------------------------ */
+
+    
+    
+    private final Vector<Address> members=new Vector<Address>(11);
+    
+    private final HashMap<Address,Entry> connections=new HashMap<Address,Entry>(11);      
+    
+    private Address local_addr=null;
+    
+    private TimeScheduler timer=null; // used for retransmissions (passed to AckSenderWindow)
+    
+    private Map<Thread,ReentrantLock> locks;
+
+    private boolean started=false;
+
+    /**
+     * A list of members who left, used to determine when to prevent sending
+     * messages to left mbrs
+     */
+    private final BoundedList<Address> previous_members=new BoundedList<Address>(50);
+    /**
+     * Contains all members that were enabled for unicasts by
+     * Event.ENABLE_UNICAST_TO
+     */
+    private final BoundedList<Address> enabled_members=new BoundedList<Address>(100);
+    
 
     /** <em>Regular</em> messages which have been added, but not removed */
     private final AtomicInteger undelivered_msgs=new AtomicInteger(0);
 
+    
+    
+    
+    @ManagedAttribute
+    public int getUndeliveredMessages() {
+        return undelivered_msgs.get();
+    }
 
     /** All protocol names have to be unique ! */
     public String  getName() {return name;}
 
+    public long[] getTimeout() {return timeout;}
+
+    @Property(name="timeout",converter=PropertyConverters.LongArray.class)
+    public void setTimeout(long[] val) {
+        if(val != null)
+            timeout=val;
+    }
+
+    @ManagedAttribute
     public String getLocalAddress() {return local_addr != null? local_addr.toString() : "null";}
+    @ManagedAttribute
     public String getMembers() {return members != null? members.toString() : "[]";}
+    @ManagedOperation
     public String printConnections() {
         StringBuilder sb=new StringBuilder();
         for(Map.Entry<Address,Entry> entry: connections.entrySet()) {
@@ -93,36 +156,43 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
         return sb.toString();
     }
 
-
+    @ManagedAttribute
     public long getNumMessagesSent() {
         return num_msgs_sent;
     }
 
+    @ManagedAttribute
     public long getNumMessagesReceived() {
         return num_msgs_received;
     }
 
+    @ManagedAttribute
     public long getNumBytesSent() {
         return num_bytes_sent;
     }
 
+    @ManagedAttribute
     public long getNumBytesReceived() {
         return num_bytes_received;
     }
-
+    
+    @ManagedAttribute
     public long getNumAcksSent() {
         return num_acks_sent;
     }
 
+    @ManagedAttribute
     public long getNumAcksReceived() {
         return num_acks_received;
     }
 
+    @ManagedAttribute
     public long getNumberOfRetransmitRequestsReceived() {
         return num_xmit_requests_received;
     }
 
     /** The number of messages in all Entry.sent_msgs tables (haven't received an ACK yet) */
+    @ManagedAttribute
     public int getNumberOfUnackedMessages() {
         int num=0;
         synchronized(connections) {
@@ -134,8 +204,8 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
         return num;
     }
 
-
-    public String getUnackedMessages() {
+    @ManagedOperation
+    public String printUnackedMessages() {
         StringBuilder sb=new StringBuilder();
         Entry e;
         Object member;
@@ -152,6 +222,7 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
     }
 
 
+    @ManagedAttribute
     public int getNumberOfMessagesInReceiveWindows() {
         int num=0;
         synchronized(connections) {
@@ -170,75 +241,12 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
 
 
     public Map<String, Object> dumpStats() {
-        Map<String,Object> m=new HashMap<String,Object>();
-        m.put("num_msgs_sent", new Long(num_msgs_sent));
-        m.put("num_msgs_received", new Long(num_msgs_received));
-        m.put("num_bytes_sent", new Long(num_bytes_sent));
-        m.put("num_bytes_received", new Long(num_bytes_received));
-        m.put("num_acks_sent", new Long(num_acks_sent));
-        m.put("num_acks_received", new Long(num_acks_received));
-        m.put("num_xmit_requests_received", new Long(num_xmit_requests_received));
-        m.put("num_unacked_msgs", new Long(getNumberOfUnackedMessages()));
-        m.put("unacked_msgs", getUnackedMessages());
-        m.put("num_msgs_in_recv_windows", new Long(getNumberOfMessagesInReceiveWindows()));
+        Map<String, Object> m=super.dumpStats();       
+        m.put("unacked_msgs", printUnackedMessages());        
         return m;
     }
 
-    public boolean setProperties(Properties props) {
-        String     str;
-        long[]     tmp;
 
-        super.setProperties(props);
-        str=props.getProperty("timeout");
-        if(str != null) {
-        tmp=Util.parseCommaDelimitedLongs(str);
-        if(tmp != null && tmp.length > 0)
-        timeouts=tmp;
-            props.remove("timeout");
-        }
-
-        str=props.getProperty("window_size");
-        if(str != null) {
-            props.remove("window_size");
-            log.warn("window_size is deprecated and will be ignored");
-        }
-
-        str=props.getProperty("min_threshold");
-        if(str != null) {
-            props.remove("min_threshold");
-            log.warn("min_threshold is deprecated and will be ignored");
-        }
-
-        str=props.getProperty("use_gms");
-        if(str != null) {
-            use_gms=Boolean.valueOf(str).booleanValue();
-            props.remove("use_gms");
-        }
-
-        str=props.getProperty("immediate_ack");
-        if(str != null) {
-        	immediate_ack=Boolean.valueOf(str).booleanValue();
-            props.remove("immediate_ack");
-        }
-
-        str=props.getProperty("loopback");
-        if(str != null) {
-            loopback=Boolean.valueOf(str).booleanValue();
-            props.remove("loopback");
-        }
-
-        str=props.getProperty("eager_lock_release");
-        if(str != null) {
-            eager_lock_release=Boolean.valueOf(str).booleanValue();
-            props.remove("eager_lock_release");
-        }
-
-        if(!props.isEmpty()) {
-            log.error("these properties are not recognized: " + props);
-            return false;
-        }
-        return true;
-    }
 
     public void start() throws Exception {
         timer=getTransport().getTimer();
@@ -349,7 +357,7 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
                         seqno=entry.sent_msgs_seqno;
                         UnicastHeader hdr=new UnicastHeader(UnicastHeader.DATA, seqno);
                         if(entry.sent_msgs == null) { // first msg to peer 'dst'
-                            entry.sent_msgs=new AckSenderWindow(this, new StaticInterval(timeouts), timer, this.local_addr); // use the global timer
+                            entry.sent_msgs=new AckSenderWindow(this, new StaticInterval(timeout), timer, this.local_addr); // use the global timer
                         }
                         msg.putHeader(name, hdr);
                         if(log.isTraceEnabled())
@@ -398,9 +406,7 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
                 // See DESIGN for details
                 boolean rc;
                 if(use_gms && !left_members.isEmpty()) {
-                    Address mbr;
-                    for(int i=0; i < left_members.size(); i++) {
-                        mbr=left_members.elementAt(i);
+                    for(Address mbr: left_members) {
                         rc=removeConnection(mbr); // adds to previous_members
                         if(rc && log.isTraceEnabled())
                             log.trace("removed " + mbr + " from connection table, member(s) " + left_members + " left");
@@ -497,6 +503,18 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
     }
 
 
+    private void removeConnections(List<Address> mbrs) {
+        Entry entry;
+        synchronized(connections) {
+            for(Address mbr: mbrs) {
+                entry=connections.remove(mbr);
+                if(entry != null)
+                    entry.reset();
+            }
+        }
+    }
+
+
 
     /** Called by AckSenderWindow to resend messages for which no ACK has been received yet */
     public void retransmit(long seqno, Message msg) {
@@ -591,9 +609,14 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
             return true; // ack the message, because this will stop retransmissions (which are unreliable) !
         }
 
+        final AtomicBoolean processing=win.getProcessing();
+        if(!processing.compareAndSet(false, true)) {
+            return true;
+        }
 
         // Try to remove (from the AckReceiverWindow) as many messages as possible as pass them up
         Message  m;
+        boolean released_processing=false;
         short removed_regular_msgs=0;
 
         // Prevents concurrent passing up of messages by different threads (http://jira.jboss.com/jira/browse/JGRP-198);
@@ -603,11 +626,17 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
         // delivery of P1, Q1, Q2, P2: FIFO (implemented by UNICAST) says messages need to be delivered only in the
         // order in which they were sent by their senders
         ReentrantLock lock=win.getLock();
-        lock.lock(); // we don't block on entry any more (http://jira.jboss.com/jira/browse/JGRP-485)
         try {
             if(eager_lock_release)
                 locks.put(Thread.currentThread(), lock);
-            while((m=win.remove()) != null) {
+            lock.lock(); // we don't block on entry any more (http://jira.jboss.com/jira/browse/JGRP-485)
+            while(true) {
+                m=win.remove(processing);
+                if(m == null) {
+                    released_processing=true;
+                    return true;
+                }
+
                 // discard OOB msg as it has already been delivered (http://jira.jboss.com/jira/browse/JGRP-377)
                 if(m.isFlagSet(Message.OOB)) {
                     continue;
@@ -617,6 +646,8 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
             }
         }
         finally {
+            if(!released_processing)
+                processing.set(false);
             if(eager_lock_release)
                 locks.remove(Thread.currentThread());
             if(lock.isHeldByCurrentThread())
@@ -633,7 +664,6 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
                 undelivered_msgs.addAndGet(-(removed_regular_msgs -num_msgs_added));
             }
         }
-        return true; // msg was successfully received - send an ack back to the sender
     }
 
 
@@ -659,7 +689,9 @@ public class UNICAST extends Protocol implements AckSenderWindow.RetransmitComma
 
     private void sendAck(Address dst, long seqno) {
         Message ack=new Message(dst);
-        ack.setFlag(Message.OOB);
+        // commented Jan 23 2008 (bela): TP.enable_unicast_bundling should decide whether we bundle or not, and *not*
+        // the OOB flag ! Bundling UNICAST ACKs should be really fast
+        // ack.setFlag(Message.OOB);
         ack.putHeader(name, new UnicastHeader(UnicastHeader.ACK, seqno));
         if(log.isTraceEnabled())
             log.trace(new StringBuilder().append(local_addr).append(" --> ACK(").append(dst).
