@@ -1,7 +1,12 @@
 package org.jgroups.protocols.pbcast;
 
 import org.jgroups.*;
+import org.jgroups.annotations.DeprecatedProperty;
 import org.jgroups.annotations.GuardedBy;
+import org.jgroups.annotations.MBean;
+import org.jgroups.annotations.ManagedAttribute;
+import org.jgroups.annotations.ManagedOperation;
+import org.jgroups.annotations.Property;
 import org.jgroups.stack.Protocol;
 import org.jgroups.util.Digest;
 import org.jgroups.util.Promise;
@@ -40,9 +45,46 @@ import java.util.concurrent.atomic.AtomicInteger;
  * @version $Id$
  * @since 2.4
  */
+@MBean(description="Flushes the cluster")
+@DeprecatedProperty(names={"auto_flush_conf"})
 public class FLUSH extends Protocol {
+    
     public static final String NAME = "FLUSH";
+    
+    
+    /* ------------------------------------------ Properties  ------------------------------------------ */
+    @Property(description="Max time to keep channel blocked in flush. Default is 8000 msec")
+    private long timeout = 8000;
+     
+    @Property(description="Timeout (per atttempt) to quiet the cluster during the first flush phase. Default is 2500 msec")
+    private long start_flush_timeout = 1500;
+    
+    @Property(description="Retry timeout after an unsuccessful attempt to quiet the cluster (first flush phase). Default is 3000 msec")
+    private long retry_timeout = 2000;
 
+    @Property(description="Reconcilliation phase toggle. Default is true")
+    private boolean enable_reconciliation = true;
+    
+    @Property(description="Max number of attempts to quiet the cluster during first flush phase. Default is 4")
+    private int flush_retry_count = 4;
+
+    
+    /* --------------------------------------------- JMX  ---------------------------------------------- */
+    
+    
+    private long startFlushTime;
+
+    private long totalTimeInFlush;
+
+    private int numberOfFlushes;
+
+    private double averageFlushDuration;
+    
+    
+    
+    /* --------------------------------------------- Fields ------------------------------------------------------ */
+    
+    
     @GuardedBy("sharedLock")
     private View currentView;
 
@@ -58,12 +100,17 @@ public class FLUSH extends Protocol {
 
     @GuardedBy("sharedLock")
     private final List<Address> flushMembers;
+    
+    private final AtomicInteger viewCounter = new AtomicInteger(0);  
 
     @GuardedBy("sharedLock")
     private final Map<Address, Digest> flushCompletedMap;    
 
     @GuardedBy("sharedLock")
     private final Set<Address> suspected;
+    
+    @GuardedBy("sharedLock")
+    private final List<Address> reconcileOks;
 
     private final Object sharedLock = new Object();
 
@@ -78,33 +125,9 @@ public class FLUSH extends Protocol {
     
     @GuardedBy("sharedLock")
     private boolean flushCompleted = false;
-
-    /**
-     * Default timeout for a group member to be in
-     * <code>isBlockingFlushDown</code>
-     */
-    private long timeout = 8000;
-     
-    private long start_flush_timeout = 2500;
-    
-    private long retry_timeout = 3000;
-
-    private boolean enable_reconciliation = true;
-    
-    private int flush_retry_count = 4;
-    
-    private final AtomicInteger viewCounter = new AtomicInteger(0);   
     
     private volatile boolean allowMessagesToPassUp = false;
 
-    private long startFlushTime;
-
-    private long totalTimeInFlush;
-
-    private int numberOfFlushes;
-
-    private double averageFlushDuration;
-
     private final Promise<Boolean> flush_promise = new Promise<Boolean>();    
     
     private final AtomicBoolean flushInProgress = new AtomicBoolean(false);
@@ -113,13 +136,11 @@ public class FLUSH extends Protocol {
     
     private final AtomicBoolean sentUnblock = new AtomicBoolean(false);
 
-    @GuardedBy("sharedLock")
-    private final List<Address> reconcileOks = new ArrayList<Address>();
-
     public FLUSH(){
         super();
         currentView = new View(new ViewId(), new Vector<Address>());      
         flushCompletedMap = new HashMap<Address, Digest>();        
+        reconcileOks = new ArrayList<Address>();
         flushMembers = new ArrayList<Address>();
         suspected = new TreeSet<Address>();
     }
@@ -128,29 +149,6 @@ public class FLUSH extends Protocol {
         return NAME;
     }
 
-    public boolean setProperties(Properties props) {
-        super.setProperties(props);
-
-        timeout = Util.parseLong(props, "timeout", timeout);
-        flush_retry_count = Util.parseInt(props, "flush_retry_count", flush_retry_count);
-        start_flush_timeout = Util.parseLong(props, "start_flush_timeout", start_flush_timeout);        
-        retry_timeout = Util.parseLong(props, "retry_timeout", retry_timeout);
-        enable_reconciliation = Util.parseBoolean(props,
-                                                  "enable_reconciliation",
-                                                  enable_reconciliation);
-        String str = props.getProperty("auto_flush_conf");
-        if(str != null){
-            log.warn("auto_flush_conf has been deprecated and its value will be ignored");
-            props.remove("auto_flush_conf");
-        }
-
-        if(!props.isEmpty()){
-            log.error("the following properties are not recognized: " + props);
-            return false;
-        }
-        return true;
-    }
-
     public void start() throws Exception {
         Map<String, Object> map = new HashMap<String, Object>();
         map.put("flush_supported", Boolean.TRUE);
@@ -161,6 +159,7 @@ public class FLUSH extends Protocol {
         synchronized(blockMutex){
             isBlockingFlushDown = true;
         }
+        flush_promise.setResult(Boolean.FALSE);
         allowMessagesToPassUp = false;
     }
 
@@ -176,18 +175,22 @@ public class FLUSH extends Protocol {
 
     /* -------------------JMX attributes and operations --------------------- */
 
+    @ManagedAttribute
     public double getAverageFlushDuration() {
         return averageFlushDuration;
     }
 
+    @ManagedAttribute
     public long getTotalTimeInFlush() {
         return totalTimeInFlush;
     }
 
+    @ManagedAttribute
     public int getNumberOfFlushes() {
         return numberOfFlushes;
     }
 
+    @ManagedOperation(description="Request cluster flush")
     public boolean startFlush() {        
         return startFlush(new Event(Event.SUSPEND));
     }
@@ -203,37 +206,38 @@ public class FLUSH extends Protocol {
     private boolean startFlush(List<Address> flushParticipants, int numberOfAttempts, boolean force) {
         boolean successfulFlush = false;
         if(!flushInProgress.get() || force){
-            flush_promise.reset();                                 
-                       
-            onSuspend(flushParticipants);
-            try{
-                Boolean r = flush_promise.getResultWithTimeout(start_flush_timeout);
-                successfulFlush = r.booleanValue();
-            }catch(TimeoutException e){
-                if(log.isDebugEnabled())
-                    log.debug("At " + localAddress
-                              + " timed out waiting for flush responses after "
-                              + start_flush_timeout
-                            + " msec");
+        	Boolean result=flush_promise.getResult(100);
+            successfulFlush = result !=null && result.booleanValue();
+            if(!successfulFlush){
+	            flush_promise.reset();                                 
+	                       
+	            onSuspend(flushParticipants);
+	            try{
+	                Boolean r = flush_promise.getResultWithTimeout(start_flush_timeout);
+	                successfulFlush = r.booleanValue();
+	            }catch(TimeoutException e){
+	                if(log.isDebugEnabled())
+	                    log.debug("At " + localAddress
+	                              + " timed out waiting for flush responses after "
+	                              + start_flush_timeout
+	                            + " msec");
+	            }
             }
         }
 
         if(!successfulFlush && numberOfAttempts > 0) {
             waitForFlushCompletion(retry_timeout);
-            Boolean result=flush_promise.getResult(100);
-            successfulFlush = result !=null && result.booleanValue();
-            if(!successfulFlush){
-                if(log.isDebugEnabled()) {
-                    log.debug("Retrying FLUSH at " + localAddress
-                              + ". Attempts left "
-                              + numberOfAttempts);
-                }
-                successfulFlush=startFlush(flushParticipants, --numberOfAttempts, true);
+            if(log.isDebugEnabled()) {
+                log.debug("Retrying FLUSH at " + localAddress
+                          + ". Attempts left "
+                          + numberOfAttempts);
             }
+            successfulFlush=startFlush(flushParticipants, --numberOfAttempts, true);
         }
         return successfulFlush;
     }
 
+    @ManagedOperation(description="Request end of flush in a cluster")
     public void stopFlush() {
         down(new Event(Event.RESUME));
     }
@@ -446,7 +450,8 @@ public class FLUSH extends Protocol {
     private boolean waitForFlushCompletion(long timeout){
         long start_time = System.currentTimeMillis(), backofftime = timeout;
         while (backofftime > 0 && flushInProgress.get()) {
-            Util.sleep(100);
+        	//sleep min 100 msec and max 600 msec
+            Util.sleep(100 + Util.random(500));
             backofftime = timeout - (System.currentTimeMillis() - start_time);            
         }
         return backofftime < 0; 
@@ -623,6 +628,7 @@ public class FLUSH extends Protocol {
             sendUnBlockUpToChannel();                  
         }                
         flushInProgress.set(false);
+        flush_promise.setResult(Boolean.FALSE);
     }
 
     private void onSuspend(List<Address> members) {
@@ -727,7 +733,9 @@ public class FLUSH extends Protocol {
         boolean needsReconciliationPhase = false;
         synchronized(sharedLock){
             flushCompletedMap.put(address, digest);
-            flushCompleted=flushCompletedMap.size() >=  flushMembers.size() && !flushMembers.isEmpty() && flushCompletedMap.keySet().containsAll(flushMembers);           
+            flushCompleted = flushCompletedMap.size() >= flushMembers.size()
+					&& !flushMembers.isEmpty()
+					&& flushCompletedMap.keySet().containsAll(flushMembers);           
 
             if(log.isDebugEnabled())
                 log.debug("At " + localAddress
@@ -933,20 +941,8 @@ public class FLUSH extends Protocol {
         public void writeTo(DataOutputStream out) throws IOException {
             out.writeByte(type);
             out.writeLong(viewID);
-            if(flushParticipants != null && !flushParticipants.isEmpty()){
-                out.writeShort(flushParticipants.size());
-                for (Address participant : flushParticipants) {
-                    Util.writeAddress(participant, out);    
-                }                       
-            }else{
-                out.writeShort(0);
-            }
-            if(digest != null){
-                out.writeBoolean(true);
-                Util.writeStreamable(digest, out);
-            }else{
-                out.writeBoolean(false);
-            }
+            Util.writeAddresses(flushParticipants, out);           
+            Util.writeStreamable(digest, out);            
         }
 
         public void readFrom(DataInputStream in) throws IOException,
@@ -954,17 +950,8 @@ public class FLUSH extends Protocol {
                                                 InstantiationException {
             type = in.readByte();
             viewID = in.readLong();
-            int flushParticipantsSize = in.readShort();
-            if(flushParticipantsSize > 0){
-                flushParticipants = new ArrayList<Address>(flushParticipantsSize);
-                for(int i = 0;i < flushParticipantsSize;i++){
-                    flushParticipants.add(Util.readAddress(in));
-                }
-            }
-            boolean hasDigest = in.readBoolean();
-            if(hasDigest){
-                digest = (Digest) Util.readStreamable(Digest.class, in);
-            }
+            flushParticipants = Util.readAddresses(in, ArrayList.class);
+            digest = (Digest) Util.readStreamable(Digest.class, in);            
         }
     }
 }
