@@ -4,99 +4,152 @@ package org.jgroups.protocols.pbcast;
 
 import org.apache.commons.logging.Log;
 import org.jgroups.*;
+import org.jgroups.annotations.*;
+import org.jgroups.protocols.pbcast.GmsImpl.Request;
 import org.jgroups.stack.Protocol;
 import org.jgroups.util.*;
 import org.jgroups.util.Queue;
 
 import java.io.*;
 import java.util.*;
-import java.util.List;
 import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
-import org.jgroups.protocols.pbcast.GmsImpl.Request;
 
 
 /**
- * Group membership protocol. Handles joins/leaves/crashes (suspicions) and emits new views
- * accordingly. Use VIEW_ENFORCER on top of this layer to make sure new members don't receive
- * any messages until they are members
+ * Group membership protocol. Handles joins/leaves/crashes (suspicions) and
+ * emits new views accordingly. Use VIEW_ENFORCER on top of this layer to make
+ * sure new members don't receive any messages until they are members
+ * 
  * @author Bela Ban
- * @version $Id: GMS.java,v 1.126.2.18 2008/10/31 15:03:42 belaban Exp $
+ * @version $Id: GMS.java,v 1.154 2008/12/05 09:21:13 belaban Exp $
  */
+@MBean(description="Group membership protocol")
+@DeprecatedProperty(names={"join_retry_timeout","digest_timeout","use_flush","flush_timeout"})
 public class GMS extends Protocol {
-    private GmsImpl           impl=null;
-    Address                   local_addr=null;
-    final Membership          members=new Membership();     // real membership
-    private final Membership  tmp_members=new Membership(); // base for computing next view
+    
+    public static final String name="GMS";
+
+    private static final String CLIENT="Client";
+    private static final String COORD="Coordinator";
+    private static final String PART="Participant";
+
+    /* ------------------------------------------ Properties  ------------------------------------------ */
+
+    @ManagedAttribute(description="Join timeout", writable=true)
+    @Property(description="Join timeout. Default is 5000 msec")
+    long join_timeout=5000;
+
+    @Property(description="Leave timeout. Default is 5000 msec")
+    long leave_timeout=5000;
+    
+    @Property(description="Timeout to complete merge. Default is 10000 msec")
+    long merge_timeout=10000; // time to wait for all MERGE_RSPS
+    
+    @ManagedAttribute(description="Shunning toggle", writable=true)
+    @Property(description="Shunning toggle. Default is false")
+    private boolean shun=false;
+    
+    @Property(description="If true this member is a designated merge leader. Default is false")
+    boolean merge_leader=false; // can I initiate a merge ?
+    
+    @Property(description="Print local address of this member after connect. Default is true")
+    private boolean print_local_addr=true;
+    
+    @Property(description="If true this member can never become coordinator. Default is false")
+    boolean disable_initial_coord=false; // can the member become a coord on startup or not ?
+    
+    /**
+     * Setting this to false disables concurrent startups. This is only used by
+     * unit testing code for testing merging. To everybody else: don't change it
+     * to false !
+     */
+    @Property(description="Temporary switch. Default is true and should not be changed")
+    boolean handle_concurrent_startup=true;
+    
+    /**
+     * Whether view bundling (http://jira.jboss.com/jira/browse/JGRP-144) should
+     * be enabled or not. Setting this to false forces each JOIN/LEAVE/SUPSECT
+     * request to be handled separately. By default these requests are processed
+     * together if they are queued at approximately the same time
+     */
+    @ManagedAttribute(description="View bundling toggle", writable=true)
+    @Property(description="Should views be bundled? Default is true")    
+    private boolean view_bundling=true;
+    
+    @Property(description="Max view bundling timeout if view bundling is turned on. Default is 50 msec")
+    @ManagedAttribute
+    private long max_bundling_time=50; // 50ms max to wait for other JOIN, LEAVE or SUSPECT requests
+    
+    @Property(description="Max number of old members to keep in history. Default is 50")
+    protected int num_prev_mbrs=50;
+    
+    @Property(description="Time in ms to wait for all VIEW acks (0 == wait forever. Default is 2000 msec" )
+    long view_ack_collection_timeout=2000;
+ 
+    @Property(description="Timeout to resume ViewHandler. Default is 20000 msec")
+    long resume_task_timeout=20000;
+
+    /**
+     * If we receive a JOIN request from P and P is already in the current
+     * membership, then we send back a JOIN response with an error message when
+     * this property is set to true (Channel.connect() will fail). Otherwise, we
+     * return the current view
+     */
+    @Property(description="Should repeated join requests from existing members be treated as errors. Default is true")
+    boolean reject_join_from_existing_member=true;
+    
+    @ManagedAttribute(writable=true, description="Logs failures for collecting all view acks if true")
+    boolean log_collect_msgs=true;   
+    
+    
+    /* --------------------------------------------- JMX  ---------------------------------------------- */
+    
+    
+    private int num_views=0;
+
+    /** Stores the last 20 views */
+    private final BoundedList<View> prev_views=new BoundedList<View>(20);
+
+
+    /* --------------------------------------------- Fields ------------------------------------------------ */
+
+    private GmsImpl impl=null;
+    private final Object impl_mutex=new Object(); // synchronizes event entry into impl
+    private final Hashtable<String,GmsImpl> impls=new Hashtable<String,GmsImpl>(3);
+    
+    protected Address local_addr=null;
+    protected final Membership members=new Membership(); // real membership
+    
+    private final Membership tmp_members=new Membership(); // base for computing next view
 
     /** Members joined but for which no view has been received yet */
     private final Vector<Address> joining=new Vector<Address>(7);
 
     /** Members excluded from group, but for which no view has been received yet */
-    private final Vector<Address>  leaving=new Vector<Address>(7);
-
-    View                      view=null;
-    ViewId                    view_id=null;
-    private long              ltime=0;
-    long                      join_timeout=5000;
-    long                      leave_timeout=5000;
-    long                      merge_timeout=5000;           // time to wait for all MERGE_RSPS
-    private final Object      impl_mutex=new Object();       // synchronizes event entry into impl
-    private final Hashtable<String,GmsImpl>   impls=new Hashtable<String,GmsImpl>(3);
-    private boolean           shun=false;
-    boolean                   merge_leader=false;         // can I initiate a merge ?
-    private boolean           print_local_addr=true;
-    boolean                   disable_initial_coord=false; // can the member become a coord on startup or not ?
-    /** Setting this to false disables concurrent startups. This is only used by unit testing code
-     * for testing merging. To everybody else: don't change it to false ! */
-    boolean                   handle_concurrent_startup=true;
-    /** Whether view bundling (http://jira.jboss.com/jira/browse/JGRP-144) should be enabled or not. Setting this to
-     * false forces each JOIN/LEAVE/SUPSECT request to be handled separately. By default these requests are processed
-     * together if they are queued at approximately the same time */
-    private boolean           view_bundling=true;
-    private long              max_bundling_time=50; // 50ms max to wait for other JOIN, LEAVE or SUSPECT requests
-    static final String       CLIENT="Client";
-    static final String       COORD="Coordinator";
-    static final String       PART="Participant";
-    TimeScheduler             timer=null;
-
-    /** Max number of old members to keep in history */
-    protected int             num_prev_mbrs=50;
+    private final Vector<Address> leaving=new Vector<Address>(7);
 
     /** Keeps track of old members (up to num_prev_mbrs) */
-    BoundedList<Address>      prev_members=null;
+    private BoundedList<Address> prev_members=null;
 
-    /** If we receive a JOIN request from P and P is already in the current membership, then we send back a JOIN
-     * response with an error message when this property is set to true (Channel.connect() will fail). Otherwise,
-     * we return the current view */
-    boolean                   reject_join_from_existing_member=true;
-
-    int                       num_views=0;
-
-    /** Stores the last 20 views */
-    BoundedList<View>         prev_views=new BoundedList<View>(20);
+    protected View view=null;
+    
+    protected ViewId view_id=null;
+    
+    protected long ltime=0;
 
+    protected TimeScheduler timer=null;    
 
     /** Class to process JOIN, LEAVE and MERGE requests */
     private final ViewHandler view_handler=new ViewHandler();
 
     /** To collect VIEW_ACKs from all members */
-    final AckCollector        ack_collector=new AckCollector();
-      
+    protected final AckCollector ack_collector=new AckCollector();
+
     //[JGRP-700] - FLUSH: flushing should span merge
-    final AckCollector        merge_ack_collector=new AckCollector();
-
-
-    /** Time in ms to wait for all VIEW acks (0 == wait forever) */
-    long                      view_ack_collection_timeout=2000;
-
-    /** How long should a Resumer wait until resuming the ViewHandler */
-    long                      resume_task_timeout=20000;
-
-    boolean                   flushProtocolInStack=false;
-
-    public static final String       name="GMS";
+    protected final AckCollector merge_ack_collector=new AckCollector();
 
+    boolean flushProtocolInStack=false;   
 
 
     public GMS() {
@@ -108,11 +161,15 @@ public class GMS extends Protocol {
         return name;
     }
 
-
+    @ManagedAttribute
     public String getView() {return view_id != null? view_id.toString() : "null";}
+    @ManagedAttribute
     public int getNumberOfViews() {return num_views;}
+    @ManagedAttribute
     public String getLocalAddress() {return local_addr != null? local_addr.toString() : "null";}
+    @ManagedAttribute
     public String getMembers() {return members != null? members.toString() : "[]";}
+    @ManagedAttribute
     public int getNumMembers() {return members != null? members.size() : 0;}
     public long getJoinTimeout() {return join_timeout;}
     public void setJoinTimeout(long t) {join_timeout=t;}
@@ -122,15 +179,7 @@ public class GMS extends Protocol {
     public void setJoinRetryTimeout(long t) {}
     public boolean isShun() {return shun;}
     public void setShun(boolean s) {shun=s;}
-
-    public boolean isPrintLocalAddr() {
-        return print_local_addr;
-    }
-
-    public void setPrintLocalAddr(boolean print_local_addr) {
-        this.print_local_addr=print_local_addr;
-    }
-
+    @ManagedOperation
     public String printPreviousMembers() {
         StringBuilder sb=new StringBuilder();
         if(prev_members != null) {
@@ -141,6 +190,9 @@ public class GMS extends Protocol {
         return sb.toString();
     }
 
+    public void setPrintLocalAddress(boolean flag) {print_local_addr=flag;}
+    public void setPrintLocalAddr(boolean flag) {setPrintLocalAddress(flag);}
+
     public long getViewAckCollectionTimeout() {
         return view_ack_collection_timeout;
     }
@@ -161,21 +213,28 @@ public class GMS extends Protocol {
         return max_bundling_time;
     }
 
+
     public void setMaxBundlingTime(long max_bundling_time) {
         this.max_bundling_time=max_bundling_time;
     }
 
-    public int viewHandlerSize() {return view_handler.size();}
+    @ManagedAttribute
+    public int getViewHandlerSize() {return view_handler.size();}
+    @ManagedAttribute
     public boolean isViewHandlerSuspended() {return view_handler.suspended();}
+    @ManagedOperation
     public String dumpViewHandlerQueue() {
         return view_handler.dumpQueue();
     }
+    @ManagedOperation
     public String dumpViewHandlerHistory() {
         return view_handler.dumpHistory();
     }
+    @ManagedOperation
     public void suspendViewHandler() {
         view_handler.suspend(null);
     }
+    @ManagedOperation
     public void resumeViewHandler() {
         view_handler.resumeForce();
     }
@@ -184,6 +243,7 @@ public class GMS extends Protocol {
 
     ViewHandler getViewHandler() {return view_handler;}
 
+    @ManagedOperation
     public String printPreviousViews() {
         StringBuilder sb=new StringBuilder();
         for(View view: prev_views) {
@@ -198,6 +258,14 @@ public class GMS extends Protocol {
     }
 
 
+    public void setLogCollectMessages(boolean flag) {
+        log_collect_msgs=flag;
+    }
+
+    public boolean getLogCollectMessages() {
+        return log_collect_msgs;
+    }
+
     public void resetStats() {
         super.resetStats();
         num_views=0;
@@ -399,17 +467,19 @@ public class GMS extends Protocol {
                           + new_view.getVid());
         }
         catch(TimeoutException e) {
-            log.warn(local_addr + " failed to collect all ACKs (" + ack_collector.size()
-                     + ") for mcasted view "
-                     + new_view
-                     + " after "
-                     + view_ack_collection_timeout
-                     + "ms, missing ACKs from "
-                     + ack_collector.printMissing()
-                     + " (received="
-                     + ack_collector.printReceived()
-                     + "), local_addr="
-                     + local_addr);
+            if(log_collect_msgs && log.isWarnEnabled()) {
+                log.warn(local_addr + " failed to collect all ACKs (" + ack_collector.size()
+                        + ") for view "
+                        + new_view
+                        + " after "
+                        + view_ack_collection_timeout
+                        + "ms, missing ACKs from "
+                        + ack_collector.printMissing()
+                        + " (received="
+                        + ack_collector.printReceived()
+                        + "), local_addr="
+                        + local_addr);
+            }
         }   
         
         if(jr != null && (newMembers != null && !newMembers.isEmpty())) {
@@ -425,17 +495,19 @@ public class GMS extends Protocol {
                               + new_view.getVid());
             }
             catch(TimeoutException e) {
-                log.warn(local_addr + " failed to collect all ACKs (" + ack_collector.size()
-                         + ") for unicasted view "
-                         + new_view
-                         + " after "
-                         + view_ack_collection_timeout
-                         + "ms, missing ACKs from "
-                         + ack_collector.printMissing()
-                         + " (received="
-                         + ack_collector.printReceived()
-                         + "), local_addr="
-                         + local_addr);
+                if(log_collect_msgs && log.isWarnEnabled()) {
+                    log.warn(local_addr + " failed to collect all ACKs (" + ack_collector.size()
+                            + ") for unicasted view "
+                            + new_view
+                            + " after "
+                            + view_ack_collection_timeout
+                            + "ms, missing ACKs from "
+                            + ack_collector.printMissing()
+                            + " (received="
+                            + ack_collector.printReceived()
+                            + "), local_addr="
+                            + local_addr);
+                }
             }
         }           
     }
@@ -769,11 +841,6 @@ public class GMS extends Protocol {
 
             case Event.SET_LOCAL_ADDRESS:
                 local_addr=(Address)evt.getArg();
-                if(print_local_addr) {
-                    System.out.println("\n-------------------------------------------------------\n" +
-                                       "GMS: address is " + local_addr +
-                                       "\n-------------------------------------------------------");
-                }
                 break;                               // pass up
 
             case Event.SUSPECT:
@@ -825,6 +892,11 @@ public class GMS extends Protocol {
         Object arg=null;
         switch(evt.getType()) {            
             case Event.CONNECT:
+                if(print_local_addr) {
+                    System.out.println("\n---------------------------------------------------------\n" +
+                            "GMS: address is " + local_addr + " (cluster=" + evt.getArg() + ")" +
+                            "\n---------------------------------------------------------");
+                }
                 down_prot.down(evt);
                 if(local_addr == null)
                     if(log.isFatalEnabled()) log.fatal("[CONNECT] local_addr is null");
@@ -868,130 +940,6 @@ public class GMS extends Protocol {
     }
 
 
-    /** Setup the Protocol instance according to the configuration string */
-    public boolean setProperties(Properties props) {
-        String str;
-
-        super.setProperties(props);
-        str=props.getProperty("shun");
-        if(str != null) {
-            shun=Boolean.valueOf(str).booleanValue();
-            props.remove("shun");
-        }
-
-        str=props.getProperty("merge_leader");
-        if(str != null) {
-            merge_leader=Boolean.valueOf(str).booleanValue();
-            props.remove("merge_leader");
-        }
-
-        str=props.getProperty("print_local_addr");
-        if(str != null) {
-            print_local_addr=Boolean.valueOf(str).booleanValue();
-            props.remove("print_local_addr");
-        }
-
-        str=props.getProperty("join_timeout");           // time to wait for JOIN
-        if(str != null) {
-            join_timeout=Long.parseLong(str);
-            props.remove("join_timeout");
-        }
-
-        str=props.getProperty("join_retry_timeout");     // time to wait between JOINs
-        if(str != null) {
-            props.remove("join_retry_timeout");
-            if(log.isWarnEnabled())
-                log.warn("join_retry_timeout has been deprecated and its value will be ignored");
-        }
-
-        str=props.getProperty("leave_timeout");           // time to wait until coord responds to LEAVE req.
-        if(str != null) {
-            leave_timeout=Long.parseLong(str);
-            props.remove("leave_timeout");
-        }
-
-        str=props.getProperty("merge_timeout");           // time to wait for MERGE_RSPS from subgroup coordinators
-        if(str != null) {
-            merge_timeout=Long.parseLong(str);
-            props.remove("merge_timeout");
-        }
-
-        str=props.getProperty("digest_timeout");          // time to wait for GET_DIGEST_OK from PBCAST
-        if(str != null) {
-            log.warn("digest_timeout has been deprecated and its value will be ignored");
-            props.remove("digest_timeout");
-        }
-
-        str=props.getProperty("view_ack_collection_timeout");
-        if(str != null) {
-            view_ack_collection_timeout=Long.parseLong(str);
-            props.remove("view_ack_collection_timeout");
-        }
-
-        str=props.getProperty("resume_task_timeout");
-        if(str != null) {
-            resume_task_timeout=Long.parseLong(str);
-            props.remove("resume_task_timeout");
-        }
-
-        str=props.getProperty("disable_initial_coord");
-        if(str != null) {
-            disable_initial_coord=Boolean.valueOf(str).booleanValue();
-            props.remove("disable_initial_coord");
-            if(log.isWarnEnabled())
-                log.warn("disable_initial_coord has been deprecated and will be phased out by 3.0, please don't use it anymore");
-        }
-
-        str=props.getProperty("handle_concurrent_startup");
-        if(str != null) {
-            handle_concurrent_startup=Boolean.valueOf(str).booleanValue();
-            props.remove("handle_concurrent_startup");
-        }
-
-        str=props.getProperty("num_prev_mbrs");
-        if(str != null) {
-            num_prev_mbrs=Integer.parseInt(str);
-            props.remove("num_prev_mbrs");
-        }
-
-        str=props.getProperty("reject_join_from_existing_member");
-        if(str != null) {
-            reject_join_from_existing_member=Boolean.parseBoolean(str);
-            props.remove("reject_join_from_existing_member");
-        }
-        
-        str=props.getProperty("use_flush");
-        if(str != null) {
-            log.warn("use_flush has been deprecated and its value will be ignored");
-            props.remove("use_flush");
-        } 
-        str=props.getProperty("flush_timeout");   
-        if(str != null) {
-            log.warn("flush_timeout has been deprecated and its value will be ignored");
-            props.remove("flush_timeout");
-        }  
-
-        str=props.getProperty("view_bundling");
-        if(str != null) {
-            view_bundling=Boolean.valueOf(str).booleanValue();
-            props.remove("view_bundling");
-        }
-
-        str=props.getProperty("max_bundling_time");
-        if(str != null) {
-            max_bundling_time=Long.parseLong(str);
-            props.remove("max_bundling_time");
-        }
-
-
-        if(!props.isEmpty()) {
-            log.error("the following properties are not recognized: " + props);
-            return false;
-        }
-        return true;
-    }
-
-
 
     /* ------------------------------- Private Methods --------------------------------- */
 
@@ -1223,7 +1171,7 @@ public class GMS extends Protocol {
     /**
      * Class which processes JOIN, LEAVE and MERGE requests. Requests are queued and processed in FIFO order
      * @author Bela Ban
-     * @version $Id: GMS.java,v 1.126.2.18 2008/10/31 15:03:42 belaban Exp $
+     * @version $Id: GMS.java,v 1.154 2008/12/05 09:21:13 belaban Exp $
      */
     class ViewHandler implements Runnable {
         volatile Thread                    thread;
@@ -1459,9 +1407,6 @@ public class GMS extends Protocol {
             }
             merge_id=null;
             // resumeForce();
-
-            // commented as this is already done in waitUnitlCompleted()
-            // thread = null; // Workaround for potential classloader leak to our thread
         }
     }
 
