@@ -1,13 +1,20 @@
-// $Id: GossipRouter.java,v 1.26.2.3 2008/11/12 08:16:53 belaban Exp $
 
 package org.jgroups.stack;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.jgroups.Address;
-import org.jgroups.conf.ClassConfigurator;
+import org.jgroups.annotations.ManagedAttribute;
+import org.jgroups.annotations.ManagedOperation;
+import org.jgroups.annotations.Property;
+import org.jgroups.jmx.JmxConfigurator;
+import org.jgroups.util.DefaultThreadFactory;
+import org.jgroups.util.ShutdownRejectedExecutionHandler;
+import org.jgroups.util.ThreadFactory;
+import org.jgroups.util.ThreadManagerThreadPoolExecutor;
 import org.jgroups.util.Util;
 
+import javax.management.MBeanServer;
 import java.io.DataInputStream;
 import java.io.DataOutputStream;
 import java.io.IOException;
@@ -15,11 +22,20 @@ import java.net.InetAddress;
 import java.net.ServerSocket;
 import java.net.Socket;
 import java.util.*;
-import java.util.concurrent.ConcurrentMap;
+import java.util.Map.Entry;
+import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ConcurrentMap;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+import java.util.concurrent.LinkedBlockingQueue;
+import java.util.concurrent.RejectedExecutionHandler;
+import java.util.concurrent.SynchronousQueue;
+import java.util.concurrent.ThreadPoolExecutor;
+import java.util.concurrent.TimeUnit;
 
 /**
- * Router for TCP based group communication (using layer TCP instead of UDP).
+ * Router for TCP based group comunication (using layer TCP instead of UDP).
  * Instead of the TCP layer sending packets point-to-point to each other
  * member, it sends the packet to the router which - depending on the target
  * address - multicasts or unicasts it to the group / or single member.<p>
@@ -37,6 +53,7 @@ import java.util.concurrent.ConcurrentHashMap;
  * additional administrative effort on the part of the user.<p>
  * @author Bela Ban
  * @author Ovidiu Feodorov <ovidiuf@users.sourceforge.net>
+ * @version $Id: GossipRouter.java,v 1.40 2008/12/18 12:23:37 vlada Exp $
  * @since 2.1.1
  */
 public class GossipRouter {
@@ -55,53 +72,88 @@ public class GossipRouter {
     public static final long GOSSIP_REQUEST_TIMEOUT=1000;
     public static final long ROUTING_CLIENT_REPLY_TIMEOUT=120000;
 
+    @ManagedAttribute(description="server port on which the GossipRouter accepts client connections", writable=true)
     private int port;
+
+    @ManagedAttribute(description="address to which the GossipRouter should bind", writable=true, name="bindAddress")
     private String bindAddressString;
 
-    // time (in msecs) until a cached 'gossip' member entry expires
-    private long expiryTime;
+    @ManagedAttribute(description="time (in msecs) until a cached 'gossip' member entry expires", writable=true)
+    private long expiryTime=30000;
 
-    // number of millisecs the main thread waits to receive a gossip request
-    // after connection was established; upon expiration, the router initiates
-    // the routing protocol on the connection. Don't set the interval too big, 
-    // otherwise the router will appear slow in answering routing requests.
+    @ManagedAttribute(description="number of millisecs the main thread waits to receive a gossip request " +
+            "after connection was established; upon expiration, the router initiates " +
+            "the routing protocol on the connection. Don't set the interval too big, " +
+            "otherwise the router will appear slow in answering routing requests.", writable=true)
     private long gossipRequestTimeout;
 
-    // time (in ms) main thread waits for a router client to send the routing 
-    // request type and the group afiliation before it declares the request
-    // failed.
+    @ManagedAttribute(description="time (in ms) main thread waits for a router client to send the routing " +
+            "request type and the group afiliation before it declares the request " +
+            "failed.", writable=true)
     private long routingClientReplyTimeout;
 
-    // HashMap<String, Map<Address,AddressEntry> >. Maintains associations between groups and their members. Keys=group
-    // names, values = maps of logical address / AddressEntry associations
+    // Maintains associations between groups and their members
     private final ConcurrentMap<String,ConcurrentMap<Address,AddressEntry>> routingTable=new ConcurrentHashMap<String,ConcurrentMap<Address,AddressEntry>>();
 
     private ServerSocket srvSock=null;
     private InetAddress bindAddress=null;
 
-    /** Time (in millis) for setting SO_LINGER on sockets returned from accept(). 0 means don't set SO_LINGER */
+    @Property(description="Time (in millis) for setting SO_LINGER on sockets returned from accept(). " +
+            "0 means do not set SO_LINGER")
     private long linger_timeout=2000L;
 
-    /** Time (in millis) for SO_TIMEOUT on sockets returned from accept(). 0 means don't set SO_TIMEOUT */
+    @Property(description="Time (in millis) for SO_TIMEOUT on sockets returned from accept(). " +
+            "0 means don't set SO_TIMEOUT")
     private long sock_read_timeout=3000L;
 
-    /** The max queue size of backlogged connections */
+    @Property(description="The max queue size of backlogged connections")
     private int backlog=1000;
 
+    @ManagedAttribute(description="operational status", name="running")
     private boolean up=true;
 
-    /** whether to discard message sent to self */
+    @ManagedAttribute(description="whether to discard message sent to self", writable=true)
     private boolean discard_loopbacks=false;
+    
+    @ManagedAttribute(description="Minimum thread pool size for incoming connections. Default is 2")
+    protected int thread_pool_min_threads=2;
+
+	@ManagedAttribute(description="Maximum thread pool size for incoming connections. Default is 10")
+    protected int thread_pool_max_threads=10;
+   
+    
+    @ManagedAttribute(description="Timeout in milliseconds to remove idle thread from regular pool. Default is 30000")
+    protected long thread_pool_keep_alive_time=30000;
+
+    @ManagedAttribute(description="Switch for enabling thread pool for incoming connections. Default true")
+    protected boolean thread_pool_enabled=false;
+  
+    @ManagedAttribute(description="Use queue to enqueue incoming connections")
+    protected boolean thread_pool_queue_enabled=true;
+
+    
+    @ManagedAttribute(description="Maximum queue size for incoming connections")
+    protected int thread_pool_queue_max_size=50;
+
+    @ManagedAttribute(description="Thread rejection policy. Possible values are Abort, Discard, DiscardOldest and Run Default is Run")
+    protected String thread_pool_rejection_policy="Run";
+    
+    protected ExecutorService thread_pool;
+    
+    protected BlockingQueue<Runnable> thread_pool_queue=null;
+    
+    protected ThreadFactory default_thread_factory = new DefaultThreadFactory(Util.getGlobalThreadGroup(), "gossip-handlers", true, true);
 
 
     // the cache sweeper
-    Timer timer=null;
+    protected Timer timer=null;
 
     protected final Log log=LogFactory.getLog(this.getClass());
 
-    //
-    // JMX INSTRUMENTATION - MANAGEMENT INTERFACE
-    //
+    private boolean jmx=false;
+	private boolean registered= false;
+   
+
 
     public GossipRouter() {
         this(PORT);
@@ -115,11 +167,8 @@ public class GossipRouter {
         this(port, bindAddressString, EXPIRY_TIME);
     }
 
-    public GossipRouter(int port, String bindAddressString,
-                        long expiryTime) {
-        this(port, bindAddressString, expiryTime,
-             GOSSIP_REQUEST_TIMEOUT,
-             ROUTING_CLIENT_REPLY_TIMEOUT);
+    public GossipRouter(int port, String bindAddressString, long expiryTime) {
+        this(port, bindAddressString, expiryTime, GOSSIP_REQUEST_TIMEOUT, ROUTING_CLIENT_REPLY_TIMEOUT);
     }
 
     public GossipRouter(int port, String bindAddressString,
@@ -132,9 +181,13 @@ public class GossipRouter {
         this.routingClientReplyTimeout=routingClientReplyTimeout;
     }
 
-    //
-    // MANAGED ATTRIBUTES
-    //
+    public GossipRouter(int port, String bindAddressString,
+                        long expiryTime, long gossipRequestTimeout,
+                        long routingClientReplyTimeout, boolean jmx) {
+        this(port, bindAddressString, expiryTime, gossipRequestTimeout, routingClientReplyTimeout);
+        this.jmx=jmx;
+    }
+
 
     public void setPort(int port) {
         this.port=port;
@@ -184,6 +237,7 @@ public class GossipRouter {
         return routingClientReplyTimeout;
     }
 
+    @ManagedAttribute(description="status")
     public boolean isStarted() {
         return srvSock != null;
     }
@@ -211,6 +265,73 @@ public class GossipRouter {
     public void setSocketReadTimeout(long sock_read_timeout) {
         this.sock_read_timeout=sock_read_timeout;
     }
+    
+    public ThreadFactory getDefaultThreadPoolThreadFactory() {
+        return default_thread_factory;
+    }
+
+    public void setDefaultThreadPoolThreadFactory(ThreadFactory factory) {
+        default_thread_factory=factory;
+        if(thread_pool instanceof ThreadPoolExecutor)
+            ((ThreadPoolExecutor)thread_pool).setThreadFactory(factory);
+    }
+    
+    public int getThreadPoolMinThreads() {
+		return thread_pool_min_threads;
+	}
+
+	public void setThreadPoolMinThreads(int thread_pool_min_threads) {
+		this.thread_pool_min_threads = thread_pool_min_threads;
+	}
+
+	public int getThreadPoolMaxThreads() {
+		return thread_pool_max_threads;
+	}
+
+	public void setThreadPoolMaxThreads(int thread_pool_max_threads) {
+		this.thread_pool_max_threads = thread_pool_max_threads;
+	}
+
+	public long getThreadPoolKeepAliveTime() {
+		return thread_pool_keep_alive_time;
+	}
+
+	public void setThreadPoolKeepAliveTime(long thread_pool_keep_alive_time) {
+		this.thread_pool_keep_alive_time = thread_pool_keep_alive_time;
+	}
+
+	public boolean isThreadPoolEnabled() {
+		return thread_pool_enabled;
+	}
+
+	public void setThreadPoolEnabled(boolean thread_pool_enabled) {
+		this.thread_pool_enabled = thread_pool_enabled;
+	}
+
+	public boolean isThreadPoolQueueEnabled() {
+		return thread_pool_queue_enabled;
+	}
+
+	public void setThreadPoolQueueEnabled(boolean thread_pool_queue_enabled) {
+		this.thread_pool_queue_enabled = thread_pool_queue_enabled;
+	}
+
+	public int getThreadPoolQueueMaxSize() {
+		return thread_pool_queue_max_size;
+	}
+
+	public void setThreadPoolQueueMaxSize(int thread_pool_queue_max_size) {
+		this.thread_pool_queue_max_size = thread_pool_queue_max_size;
+	}
+
+	public String getThreadPoolRejectionPolicy() {
+		return thread_pool_rejection_policy;
+	}
+
+	public void setThreadPoolRejectionPolicy(String thread_pool_rejection_policy) {
+		this.thread_pool_rejection_policy = thread_pool_rejection_policy;
+	}
+
 
     public static String type2String(int type) {
         switch(type) {
@@ -237,28 +358,27 @@ public class GossipRouter {
         }
     }
 
-    //
-    // JBoss MBean LIFECYCLE OPERATIONS
-    //
 
-
-    /**
-     * JBoss MBean lifecycle operation.
-     */
     public void create() throws Exception {
-        // not used
     }
 
     /**
-     * JBoss MBean lifecycle operation. Called after create(). When this method
-     * is called, the managed attributes have already been set.<br>
-     * Brings the Router in fully functional state.
+     * Lifecycle operation. Called after create(). When this method is called, the managed attributes have already
+     * been set.<br> Brings the Router into a fully functional state.
      */
+    @ManagedOperation(description="Lifecycle operation. Called after create(). When this method is called, " +
+            "the managed attributes have already been set. Brings the Router into a fully functional state.")
     public void start() throws Exception {
         if(srvSock != null) {
             throw new Exception("Router already started.");
         }
 
+        if(jmx && !registered) {
+            MBeanServer server=Util.getMBeanServer();
+            JmxConfigurator.register(this, server, "jgroups:name=GossipRouter");
+            registered = true;
+        }
+
         if(bindAddressString != null) {
             bindAddress=InetAddress.getByName(bindAddressString);
             srvSock=new ServerSocket(port, backlog, bindAddress);
@@ -268,13 +388,37 @@ public class GossipRouter {
         }
 
         up=true;
+        
+        if (thread_pool_enabled) {
+			if (thread_pool_queue_enabled) {
+				thread_pool_queue = new LinkedBlockingQueue<Runnable>(
+						thread_pool_queue_max_size);
+			} 
+			else {
+				thread_pool_queue = new SynchronousQueue<Runnable>();
+			}
+			thread_pool = createThreadPool(thread_pool_min_threads,
+					thread_pool_max_threads, thread_pool_keep_alive_time,
+					thread_pool_rejection_policy, thread_pool_queue,
+					default_thread_factory);
+		} 
+        else { 
+			thread_pool = Executors.newSingleThreadExecutor(default_thread_factory);
+		}
+
+        Runtime.getRuntime().addShutdownHook(new Thread() {
+            public void run() {
+                cleanup();
+                GossipRouter.this.stop();
+            }
+        });
 
         // start the main server thread
         new Thread(new Runnable() {
             public void run() {
-                mainLoop();
-                cleanup();
-            }
+                    mainLoop();
+                    cleanup();
+                }
         }, "GossipRouter").start();
 
         // starts the cache sweeper as daemon thread, so we won't block on it
@@ -288,43 +432,29 @@ public class GossipRouter {
     }
 
     /**
-     * JBoss MBean lifecycle operation. The JMX agent allways calls this method
-     * before destroy(). Close connections and frees resources.
+     * Always called before destroy(). Close connections and frees resources.
      */
+    @ManagedOperation(description="Always called before destroy(). Closes connections and frees resources")
     public void stop() {
         up=false;
-
-        if(srvSock == null) {
-            if(log.isWarnEnabled()) log.warn("router already stopped");
-            return;
-        }
+        thread_pool.shutdownNow();
 
         timer.cancel();
-        shutdown();
-        try {
-            srvSock.close();
+        if(srvSock != null) {
+            shutdown();
+            Util.close(srvSock);
+            // exiting the mainLoop will clean the tables
+            srvSock=null;
         }
-        catch(Exception e) {
-            if(log.isErrorEnabled()) log.error("Failed to close server socket: " + e);
-        }
-        // exiting the mainLoop will clean the tables
-        srvSock=null;
         if(log.isInfoEnabled()) log.info("router stopped");
     }
 
-    /**
-     * JBoss MBean lifecycle operation.
-     */
     public void destroy() {
-        // not used
     }
 
-    //
-    // ORDINARY OPERATIONS
-    //
-
 
 
+    @ManagedOperation(description="dumps the contents of the routing table")
     public String dumpRoutingTable() {
         String label="routing";
         StringBuilder sb=new StringBuilder();
@@ -333,10 +463,10 @@ public class GossipRouter {
             sb.append("empty ").append(label).append(" table");
         }
         else {
-            for(Iterator i=routingTable.keySet().iterator(); i.hasNext();) {
-                String gname=(String)i.next();
+            for(Iterator<String> i=routingTable.keySet().iterator(); i.hasNext();) {
+                String gname=i.next();
                 sb.append("GROUP: '" + gname + "'\n");
-                Map map=routingTable.get(gname);
+                Map<Address,AddressEntry> map=routingTable.get(gname);
                 if(map == null) {
                     sb.append("\tnull list of addresses\n");
                 }
@@ -344,10 +474,8 @@ public class GossipRouter {
                     sb.append("\tempty list of addresses\n");
                 }
                 else {
-                    AddressEntry ae;
-                    for(Iterator j=map.values().iterator(); j.hasNext();) {
-                        ae=(AddressEntry)j.next();
-                        sb.append('\t').append(ae).append('\n');
+                    for(Iterator<AddressEntry> j=map.values().iterator(); j.hasNext();) {
+                        sb.append('\t').append(i.next()).append('\n');
                     }
                 }
             }
@@ -356,166 +484,197 @@ public class GossipRouter {
     }
 
 
-    /**
-     * The main server loop. Runs on the JGroups Router Main Thread.
-     */
     private void mainLoop() {
-        Socket sock=null;
-        DataInputStream input=null;
-        DataOutputStream output=null;
-        Address peer_addr=null, mbr, logical_addr;
 
-        if(bindAddress == null) {
-            bindAddress=srvSock.getInetAddress();
-        }
-        System.out.println("GossipRouter started at " + new Date() +
-                "\nListening on port " + port + " bound on address " + bindAddress + '\n');
+		if (bindAddress == null) {
+			bindAddress = srvSock.getInetAddress();
+		}
+		System.out.println("GossipRouter started at " + new Date()
+				+ "\nListening on port " + port + " bound on address "
+				+ bindAddress + '\n');
 
-        GossipData req;
-        String group;
+		while (up && srvSock != null) {
+			try {
+				final Socket sock = srvSock.accept();
+				if (linger_timeout > 0) {
+					int linger = Math.min(1, (int) (linger_timeout / 1000));
+					sock.setSoLinger(true, linger);
+				}
+				if (sock_read_timeout > 0) {
+					sock.setSoTimeout((int) sock_read_timeout);
+				}
 
-        while(up && srvSock != null) {
-            try {
-                sock=srvSock.accept();
-                if(linger_timeout > 0) {
-                    int linger=Math.min(1, (int)(linger_timeout / 1000));
-                    sock.setSoLinger(true, linger);
-                }
-                if(sock_read_timeout > 0) {
-                    sock.setSoTimeout((int)sock_read_timeout);
-                }
+				final DataInputStream input = new DataInputStream(sock.getInputStream());
+				Runnable task = new Runnable(){
 
-                input=new DataInputStream(sock.getInputStream());
-                // if(log.isTraceEnabled())
-                   // log.trace("accepted connection from " + sock);
+					public void run() {
+						DataOutputStream output =null;
+						Address peer_addr = null, mbr, logical_addr;
+						String group;
+						GossipData req = new GossipData();
+						try {
+							req.readFrom(input);
 
-                req=new GossipData();
-                req.readFrom(input);
+							switch (req.getType()) {
+							case GossipRouter.REGISTER:
+								mbr = req.getAddress();
+								group = req.getGroup();
+								if (log.isTraceEnabled())
+									log.trace("REGISTER(" + group + ", " + mbr + ")");
+								if (group == null || mbr == null) {
+									if (log.isErrorEnabled())
+										log.error("group or member is null, cannot register member");
+								} else
+									addGossipEntry(group, mbr, new AddressEntry(mbr));
+								Util.close(input);
+								Util.close(sock);
+								break;
 
-                switch(req.getType()) {
-                    case GossipRouter.REGISTER:
-                        mbr=req.getAddress();
-                        group=req.getGroup();
-                        if(log.isTraceEnabled())
-                            log.trace("REGISTER(" + group + ", " + mbr + ")");
-                        if(group == null || mbr == null) {
-                            if(log.isErrorEnabled()) log.error("group or member is null, cannot register member");
-                        }
-                        else
-                            addGossipEntry(group, mbr, new AddressEntry(mbr));
-                        Util.close(input);
-                        Util.close(sock);
-                        break;
+							case GossipRouter.UNREGISTER:
+								mbr = req.getAddress();
+								group = req.getGroup();
+								if (log.isTraceEnabled())
+									log.trace("UNREGISTER(" + group + ", " + mbr + ")");
+								if (group == null || mbr == null) {
+									if (log.isErrorEnabled())
+										log.error("group or member is null, cannot unregister member");
+								} else
+									removeGossipEntry(group, mbr);
+								Util.close(input);
+								Util.close(output);
+								Util.close(sock);
+								break;
 
-                    case GossipRouter.UNREGISTER:
-                        mbr=req.getAddress();
-                        group=req.getGroup();
-                        if(log.isTraceEnabled())
-                            log.trace("UNREGISTER(" + group + ", " + mbr + ")");
-                        if(group == null || mbr == null) {
-                            if(log.isErrorEnabled()) log.error("group or member is null, cannot unregister member");
-                        }
-                        else
-                            removeGossipEntry(group, mbr);
-                        Util.close(input);
-                        Util.close(output);
-                        Util.close(sock);
-                        break;
+							case GossipRouter.GOSSIP_GET:
+								group = req.getGroup();
+								List<Address> mbrs = null;
+								Map<Address, AddressEntry> map;
+								map = routingTable.get(group);
+								if (map != null) {
+									mbrs = new LinkedList<Address>(map.keySet());
+								}
 
-                    case GossipRouter.GOSSIP_GET:
-                        group=req.getGroup();
-                        List<Address> mbrs=null;
-                        Map<Address,AddressEntry> map;
-                        map=routingTable.get(group);
-                        if(map != null) {
-                            mbrs=new LinkedList<Address>(map.keySet());
-                        }
+								if (log.isTraceEnabled())
+									log.trace("GOSSIP_GET(" + group + ") --> " + mbrs);
+								output = new DataOutputStream(sock.getOutputStream());
+								GossipData rsp = new GossipData(GossipRouter.GET_RSP,group, null, mbrs);
+								rsp.writeTo(output);
+								Util.close(input);
+								Util.close(output);
+								Util.close(sock);
+								break;
 
-                        if(log.isTraceEnabled())
-                            log.trace("GOSSIP_GET(" + group + ") --> " + mbrs);
-                        output=new DataOutputStream(sock.getOutputStream());
-                        GossipData rsp=new GossipData(GossipRouter.GET_RSP, group, null, mbrs);
-                        rsp.writeTo(output);
-                        Util.close(input);
-                        Util.close(output);
-                        Util.close(sock);
-                        break;
+							case GossipRouter.ROUTER_GET:
+								group = req.getGroup();
+								output = new DataOutputStream(sock.getOutputStream());
 
-                    case GossipRouter.ROUTER_GET:
-                        group=req.getGroup();
-                        output=new DataOutputStream(sock.getOutputStream());
+								List<Address> ret = null;
+								map = routingTable.get(group);
+								if (map != null) {
+									ret = new LinkedList<Address>(map.keySet());
+								} else
+									ret = new LinkedList<Address>();
+								if (log.isTraceEnabled())
+									log.trace("ROUTER_GET(" + group + ") --> " + ret);
+								rsp = new GossipData(GossipRouter.GET_RSP, group, null,
+										ret);
+								rsp.writeTo(output);
+								Util.close(input);
+								Util.close(output);
+								Util.close(sock);
+								break;
 
-                        List<Address> ret=null;
-                        map=routingTable.get(group);
-                        if(map != null) {
-                            ret=new LinkedList<Address>(map.keySet());
-                        }
-                        else
-                            ret=new LinkedList<Address>();
-                        if(log.isTraceEnabled())
-                            log.trace("ROUTER_GET(" + group + ") --> " + ret);
-                        rsp=new GossipData(GossipRouter.GET_RSP, group, null, ret);
-                        rsp.writeTo(output);
-                        Util.close(input);
-                        Util.close(output);
-                        Util.close(sock);
-                        break;
+							case GossipRouter.DUMP:
+								output = new DataOutputStream(sock.getOutputStream());
+								output.writeUTF(dumpRoutingTable());
+								Util.close(input);
+								Util.close(output);
+								Util.close(sock);
+								break;
 
-                    case GossipRouter.DUMP:
-                        output=new DataOutputStream(sock.getOutputStream());
-                        output.writeUTF(dumpRoutingTable());
-                        Util.close(input);
-                        Util.close(output);
-                        Util.close(sock);
-                        break;
+							case GossipRouter.CONNECT:
+								sock.setSoTimeout(0); // we have to disable this here
+								output = new DataOutputStream(sock.getOutputStream());
+								peer_addr = new IpAddress(sock.getInetAddress(), sock.getPort());
+								logical_addr = req.getAddress();
+								String group_name = req.getGroup();
 
-                    case GossipRouter.CONNECT:
-                        sock.setSoTimeout(0); // we have to disable this here
-                        output=new DataOutputStream(sock.getOutputStream());
-                        peer_addr=new IpAddress(sock.getInetAddress(), sock.getPort());
-                        logical_addr=req.getAddress();
-                        String group_name=req.getGroup();
+								if (log.isTraceEnabled())
+									log.trace("CONNECT(" + group_name + ", "+ logical_addr + ")");
+								SocketThread st = new SocketThread(sock, input,group_name, logical_addr);
+								addEntry(group_name, logical_addr, new AddressEntry(logical_addr, peer_addr, sock, st, output));
+								st.start();
+								break;
 
-                        if(log.isTraceEnabled())
-                            log.trace("CONNECT(" + group_name + ", " + logical_addr + ")");
-                        SocketThread st=new SocketThread(sock, input, group_name, logical_addr);
-                        addEntry(group_name, logical_addr, new AddressEntry(logical_addr, peer_addr, sock, st, output));
-                        st.start();
-                        break;
+							case GossipRouter.DISCONNECT:
+								Address addr = req.getAddress();
+								group_name = req.getGroup();
+								removeEntry(group_name, addr);
+								if (log.isTraceEnabled())
+									log.trace("DISCONNECT(" + group_name + ", " + addr+ ")");
+								Util.close(input);
+								Util.close(output);
+								Util.close(sock);
+								break;
 
-                    case GossipRouter.DISCONNECT:
-                        Address addr=req.getAddress();
-                        group_name=req.getGroup();
-                        removeEntry(group_name, addr);
-                        if(log.isTraceEnabled())
-                            log.trace("DISCONNECT(" + group_name + ", " + addr + ")");
-                        Util.close(input);
-                        Util.close(output);
-                        Util.close(sock);
-                        break;
+							case GossipRouter.SHUTDOWN:
+								if (log.isInfoEnabled())
+									log.info("router shutting down");
+								Util.close(input);
+								Util.close(output);
+								Util.close(sock);
+								up = false;
+								break;
+							default:
+								if (log.isWarnEnabled())
+									log.warn("received unkown gossip request (gossip="+ req + ')');
+								break;
+							}
+						} catch (Exception e) {
+							if (up)
+								if (log.isErrorEnabled())
+									log.error("failure handling a client request", e);
+							Util.close(input);
+							Util.close(output);
+							Util.close(sock);
+						}
+					}};
+					if(!thread_pool.isShutdown())
+						thread_pool.submit(task);
+					else
+						task.run();
+			} catch (Exception exc) {
+				if (log.isErrorEnabled())
+					log.error("failure receiving and setting up a client request", exc);
+			}
+		}
+	}
+    
+    protected ExecutorService createThreadPool(int min_threads,
+			int max_threads, long keep_alive_time, String rejection_policy,
+			BlockingQueue<Runnable> queue, final ThreadFactory factory) {
+
+		ThreadPoolExecutor pool = new ThreadManagerThreadPoolExecutor(
+				min_threads, max_threads, keep_alive_time,
+				TimeUnit.MILLISECONDS, queue);
+		pool.setThreadFactory(factory);
+
+		// default
+		RejectedExecutionHandler handler = new ThreadPoolExecutor.CallerRunsPolicy();
+		if (rejection_policy != null) {
+			if (rejection_policy.equals("abort"))
+				handler = new ThreadPoolExecutor.AbortPolicy();
+			else if (rejection_policy.equals("discard"))
+				handler = new ThreadPoolExecutor.DiscardPolicy();
+			else if (rejection_policy.equals("discardoldest"))
+				handler = new ThreadPoolExecutor.DiscardOldestPolicy();
+		}
+		pool.setRejectedExecutionHandler(new ShutdownRejectedExecutionHandler(
+				handler));
+
+		return pool;
+	}
 
-                    case GossipRouter.SHUTDOWN:
-                        if(log.isInfoEnabled()) log.info("router shutting down");
-                        Util.close(input);
-                        Util.close(output);
-                        Util.close(sock);
-                        up=false;
-                        break;
-                    default:
-                        if(log.isWarnEnabled())
-                            log.warn("received unkown gossip request (gossip=" + req + ')');
-                        break;
-                }
-            }
-            catch(Exception e) {
-                if(up)
-                    if(log.isErrorEnabled()) log.error("failure handling a client request", e);
-                Util.close(input);
-                Util.close(output);
-                Util.close(sock);
-            }
-        }
-    }
 
 
     /**
@@ -564,20 +723,17 @@ public class GossipRouter {
     private void sweep() {
         long diff, currentTime=System.currentTimeMillis();
         int num_entries_removed=0;
-
-        Map.Entry entry, entry2;
-        Map map;
-        AddressEntry ae;
-        for(Iterator it=routingTable.entrySet().iterator(); it.hasNext();) {
-            entry=(Map.Entry)it.next();
-            map=(Map)entry.getValue();
+       
+        for(Iterator<Entry<String,ConcurrentMap<Address,AddressEntry>>> it=routingTable.entrySet().iterator(); it.hasNext();) {
+        	Entry <String,ConcurrentMap<Address,AddressEntry>> entry=it.next();
+        	Map <Address,AddressEntry> map=entry.getValue();
             if(map == null || map.isEmpty()) {
                 it.remove();
                 continue;
             }
-            for(Iterator it2=map.entrySet().iterator(); it2.hasNext();) {
-                entry2=(Map.Entry)it2.next();
-                ae=(GossipRouter.AddressEntry)entry2.getValue();
+            for(Iterator<Entry<Address,AddressEntry>> it2=map.entrySet().iterator(); it2.hasNext();) {
+            	Entry<Address,AddressEntry>entry2=it2.next();
+                AddressEntry ae=entry2.getValue();
                 diff=currentTime - ae.timestamp;
                 if(diff > expiryTime) {
                     it2.remove();
@@ -671,12 +827,11 @@ public class GossipRouter {
 
 
     private void removeEntry(String groupname, Address logical_addr) {
-        Map val;
-        val=routingTable.get(groupname);
+        Map<Address,AddressEntry>val=routingTable.get(groupname);
         if(val == null)
             return;
         synchronized(val) {
-            AddressEntry entry=(AddressEntry)val.get(logical_addr);
+            AddressEntry entry=val.get(logical_addr);
             if(entry != null) {
                 entry.destroy();
                 val.remove(logical_addr);
@@ -691,7 +846,7 @@ public class GossipRouter {
     private AddressEntry findAddressEntry(String group_name, Address logical_addr) {
         if(group_name == null || logical_addr == null)
             return null;
-        Map val=routingTable.get(group_name);
+        Map<Address,AddressEntry> val=routingTable.get(group_name);
         if(val == null)
             return null;
         return (AddressEntry)val.get(logical_addr);
@@ -716,17 +871,14 @@ public class GossipRouter {
 
 
     private void sendToAllMembersInGroup(String groupname, byte[] msg, Address sender) {
-        Map val;
-        val=routingTable.get(groupname);
+        Map<Address,AddressEntry> val = routingTable.get(groupname);
         if(val == null || val.isEmpty())
             return;
-
-        Map.Entry tmp;
-        AddressEntry entry;
+        
         synchronized(val) {
-            for(Iterator i=val.entrySet().iterator(); i.hasNext();) {
-                tmp=(Map.Entry)i.next();
-                entry=(GossipRouter.AddressEntry)tmp.getValue();
+            for(Iterator<Entry<Address,AddressEntry>> i=val.entrySet().iterator(); i.hasNext();) {
+            	Entry<Address,AddressEntry> tmp=i.next();
+                AddressEntry entry=tmp.getValue();
                 DataOutputStream dos=entry.output;
 
                 if(dos != null) {
@@ -735,7 +887,7 @@ public class GossipRouter {
                         sendToMember(null, dos, msg, sender);
                     }
                     catch(Exception e) {
-                        if(log.isTraceEnabled()) log.trace("cannot send to " + entry.logical_addr + ": " + e.getMessage());
+                        if(log.isWarnEnabled()) log.warn("cannot send to " + entry.logical_addr + ": " + e.getMessage());
                         entry.destroy(); // this closes the socket
                         i.remove();
                     }
@@ -864,7 +1016,19 @@ public class GossipRouter {
             int len;
             Address dst_addr=null;
             String gname;
-
+            
+            DataOutputStream output = null;
+            try {
+                output=new DataOutputStream(sock.getOutputStream());
+                output.writeBoolean(true);
+            }
+            catch(IOException e1) {
+                try {
+                    output.writeBoolean(false);
+                }
+                catch(IOException e) {}
+            }
+            
             while(active) {
                 try {
                     // 1. Group name is first
@@ -914,6 +1078,7 @@ public class GossipRouter {
         long routingTimeout=GossipRouter.ROUTING_CLIENT_REPLY_TIMEOUT;
         GossipRouter router=null;
         String bind_addr=null;
+        boolean jmx=false;
 
         for(int i=0; i < args.length; i++) {
             arg=args[i];
@@ -936,20 +1101,34 @@ public class GossipRouter {
             if("-rtimeout".equals(arg)) {
                 routingTimeout=Long.parseLong(args[++i]);
                 continue;
+            }
+             if("-jmx".equals(arg)) {
+                jmx=true;
+                continue;
             }
             help();
             return;
         }
-        System.out.println("GossipRouter is starting...");
+        System.out.println("GossipRouter is starting. Enter quit to exit JVM");
 
         try {
-            ClassConfigurator.getInstance(true);
-            router=new GossipRouter(port, bind_addr, expiry, timeout, routingTimeout);
+            router=new GossipRouter(port, bind_addr, expiry, timeout, routingTimeout, jmx);
             router.start();
         }
         catch(Exception e) {
             System.err.println(e);
         }
+        String quit = "";
+        while (!quit.startsWith("quit")) {
+			Scanner in = new Scanner(System.in);
+			try{
+				quit = in.nextLine();
+			}
+			catch(Exception e){}
+		}
+        
+        router.stop();
+        router.cleanup();
     }
 
     static void help() {
@@ -961,6 +1140,8 @@ public class GossipRouter {
         System.out.println("                            a gossip request after connection was established;");
         System.out.println("                            upon expiration, the router initiates the routing");
         System.out.println("                            protocol on the connection.");
+        System.out.println("        -rtimeout  <mcsec> - Routing timeout");
+        System.out.println("        -jmx               - Expose attrs and operations via JMX");
     }
 
 
