@@ -1,73 +1,92 @@
-// $Id: NakReceiverWindowTest.java,v 1.1.4.1 2008/06/09 09:23:15 belaban Exp $
+// $Id: NakReceiverWindowTest.java,v 1.9 2008/09/23 14:50:29 belaban Exp $
 
 package org.jgroups.tests;
 
 
-import junit.framework.Test;
-import junit.framework.TestCase;
-import junit.framework.TestSuite;
-import org.jgroups.Message;
 import org.jgroups.Address;
-import org.jgroups.stack.NakReceiverWindow;
+import org.jgroups.Global;
+import org.jgroups.Message;
 import org.jgroups.stack.IpAddress;
+import org.jgroups.stack.NakReceiverWindow;
 import org.jgroups.stack.Retransmitter;
+import org.jgroups.util.TimeScheduler;
+import org.testng.Assert;
+import org.testng.annotations.AfterMethod;
+import org.testng.annotations.BeforeClass;
+import org.testng.annotations.BeforeMethod;
+import org.testng.annotations.Test;
+
+import java.util.List;
 
 
-public class NakReceiverWindowTest extends TestCase {
-
+@Test(groups=Global.FUNCTIONAL)
+public class NakReceiverWindowTest {
     private Address sender;
     private MyRetransmitCommand cmd=new MyRetransmitCommand();
+    private TimeScheduler timer;
 
-    public NakReceiverWindowTest(String name) {
-        super(name);
+    @BeforeMethod
+    void initTimer() {
+        timer=new TimeScheduler();
+    }
+
+    @AfterMethod
+    void destroyTimer() throws InterruptedException {
+        timer.stop();
     }
 
 
+    @BeforeClass
     protected void setUp() throws Exception {
-        super.setUp();
         sender=new IpAddress("127.0.0.1", 5555);
     }
 
+
     public void test1() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 1);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 1, timer);
         check(win, 0, 1, 1);
-        assertNull(win.get(23));
+        assert win.get(23) == null;
     }
 
+
     public void test2() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100, timer);
         check(win, 0, 100, 100);
     }
 
+
     public void test3() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         win.add(1, new Message());
-        assertNotNull(win.get(1));
+        assert win.get(1) != null;
         check(win, 0, 1, 0);
         win.add(2, new Message());
         check(win, 0, 2, 0);
-        assertNotNull(win.get(2));
+        assert win.get(2) != null;
         win.remove();
         check(win, 0, 2, 1);
         win.remove();
         check(win, 0, 2, 2);
     }
 
+
     public void test4() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 1);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 1, timer);
         win.add(2, new Message());
         check(win, 0, 2, 1);
     }
 
+
     public void test5() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100, timer);
         win.add(101, new Message());
         win.add(100, new Message());
         check(win, 0, 101, 100);
     }
 
+
     public void test6() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100, timer);
         win.add(101, new Message());
         System.out.println("win: " + win);
         win.add(100, new Message());
@@ -81,8 +100,9 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
+
     public void testLowerBounds() {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100, 50, null);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100, 50, timer);
         win.add(101, new Message());
         System.out.println("win: " + win);
         win.add(100, new Message());
@@ -95,8 +115,9 @@ public class NakReceiverWindowTest extends TestCase {
         check(win, 50, 101, 101);
     }
 
+
     public void test7() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         win.add(1, new Message());
         win.add(2, new Message());
         win.add(3, new Message());
@@ -112,8 +133,9 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
+
     public void testLowerBounds2() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100, 50, null);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100, 50, timer);
         win.add(100, new Message());
         win.add(101, new Message());
         win.add(102, new Message());
@@ -130,8 +152,9 @@ public class NakReceiverWindowTest extends TestCase {
         check(win, 103, 103, 103);
     }
 
+
     public void test8() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         win.add(1, new Message());
         win.add(2, new Message());
         win.add(3, new Message());
@@ -153,8 +176,9 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
+
     public void testAdd() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         check(win, 0, 0, 0);
         win.add(0, new Message()); // discarded, next expected is 1
         check(win, 0, 0, 0);
@@ -177,8 +201,9 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
+
     public void test9() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         win.add(1, new Message());
         win.add(2, new Message());
         win.add(3, new Message());
@@ -188,7 +213,7 @@ public class NakReceiverWindowTest extends TestCase {
         while((win.remove()) != null) ;
         win.stable(6); // 6 is ignore as it is >= highest delivered message
         System.out.println("win: " + win);
-        assertNotNull(win.get(2));
+        assert win.get(2) != null;
         check(win, 0, 6, 4);
         win.add(5, new Message());
         check(win, 0, 6, 4);
@@ -199,8 +224,9 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
+
     public void testHighestDelivered() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         win.add(1, new Message());
         win.add(2, new Message());
         win.add(3, new Message());
@@ -227,8 +253,9 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
+
     public void testMissingMessages() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         win.add(1, new Message());
         win.add(5, new Message());
         check(win, 0, 5, 0);
@@ -238,8 +265,9 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
+
     public void testMissingMessages2() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         win.add(1, new Message());
         win.add(5, new Message());
         check(win, 0, 5, 0);
@@ -251,8 +279,9 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
+
     public void testMissingMessages3() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         win.add(1, new Message());
         win.add(5, new Message());
         check(win, 0, 5, 0);
@@ -278,8 +307,9 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
+
     public void testMissingMessages4() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100, timer);
         win.add(101, new Message());
         win.add(105, new Message());
         check(win, 0, 105, 100);
@@ -305,8 +335,9 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
+
     public void testMissingMessages5() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 100, timer);
         win.add(101, new Message());
         check(win, 0, 101, 100);
         win.add(108, new Message());
@@ -332,8 +363,9 @@ public class NakReceiverWindowTest extends TestCase {
         System.out.println("win: " + win);
     }
 
+
     public void test10() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         win.add(1, new Message());
         win.add(2, new Message());
         win.add(3, new Message());
@@ -342,8 +374,9 @@ public class NakReceiverWindowTest extends TestCase {
         check(win, 0, 4, 4);
     }
 
+
     public void test10a() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         win.add(1, new Message());
         win.add(2, new Message());
         win.add(3, new Message());
@@ -354,8 +387,9 @@ public class NakReceiverWindowTest extends TestCase {
 
     }
 
+
     public void test11() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         win.add(1, new Message());
         win.add(2, new Message());
         win.add(3, new Message());
@@ -366,21 +400,23 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
+
     public void test12() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
 
         win.add(1, new Message(null, null, new Integer(1)));
         win.add(2, new Message(null, null, new Integer(2)));
         win.add(3, new Message(null, null, new Integer(3)));
 
-        assertEquals(1, ((Integer)win.remove().getObject()).intValue());
-        assertEquals(2, ((Integer)win.remove().getObject()).intValue());
-        assertEquals(3, ((Integer)win.remove().getObject()).intValue());
+        Assert.assertEquals(1, ((Integer)win.remove().getObject()).intValue());
+        Assert.assertEquals(2, ((Integer)win.remove().getObject()).intValue());
+        Assert.assertEquals(3, ((Integer)win.remove().getObject()).intValue());
     }
 
 
+
     public void test13() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         win.add(1, new Message());
         win.add(2, new Message());
         win.add(3, new Message());
@@ -397,50 +433,51 @@ public class NakReceiverWindowTest extends TestCase {
 
 
 
+
     public void testAddOOBAtHead() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         boolean rc;
         rc=win.add(0, oob());
-        assertFalse(rc);
+        assert !(rc);
         rc=win.add(1, oob());
-        assertTrue(rc);
+        assert rc;
         rc=win.add(1, oob());
-        assertFalse(rc);
+        assert !(rc);
     }
 
 
+
     public void testAddOOBAtTail() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         boolean rc;
         rc=win.add(1, oob());
-        assertTrue(rc);
+        assert rc;
         rc=win.add(2, oob());
-        assertTrue(rc);
+        assert rc;
         rc=win.add(2, oob());
-        assertFalse(rc);
+        assert !(rc);
     }
 
 
+
     public void testAddOOBInTheMiddle() throws Exception {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         boolean rc;
         rc=win.add(3, oob());
-        assertTrue(rc);
+        assert rc;
         rc=win.add(3, oob());
-        assertFalse(rc);
+        assert !(rc);
         rc=win.add(1, oob());
-        assertTrue(rc);
+        assert rc;
         rc=win.add(1, oob());
-        assertFalse(rc);
+        assert !(rc);
         rc=win.add(2, oob());
-        assertTrue(rc);
+        assert rc;
         rc=win.add(2, oob());
-        assertFalse(rc);
+        assert !(rc);
     }
 
 
-
-
     public void testUpdateHighestSeen() {
         add(1000);
         add(2000);
@@ -453,50 +490,109 @@ public class NakReceiverWindowTest extends TestCase {
         add(30000);
     }
 
+
     public void test1000() {
         add(1000);
     }
 
+
     public void test10000() {
         add(10000);
     }
 
 
     public void testHasMessagesToRemove() {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
-        assertFalse(win.hasMessagesToRemove());
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
+        assert !win.hasMessagesToRemove();
         win.add(2, new Message());
-        assertFalse(win.hasMessagesToRemove());
+        assert !win.hasMessagesToRemove();
         win.add(1, oob());
-        assertTrue(win.hasMessagesToRemove());
+        assert win.hasMessagesToRemove();
         win.remove();
-        assertTrue(win.hasMessagesToRemove());
+
+        assert win.hasMessagesToRemove();
         win.remove();
-        assertFalse(win.hasMessagesToRemove());
+        assert ! win.hasMessagesToRemove();
     }
 
     public void testRemoveOOBMessage() {
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
         System.out.println("win = " + win);
-        win.add(2, new Message());
+        win.add(2, msg());
         System.out.println("win = " + win);
-        assertNull(win.removeOOBMessage());
-        assertNull(win.remove());
+        assert win.removeOOBMessage() == null;
+        assert win.remove() == null;
         win.add(1, oob());
         System.out.println("win = " + win);
-        assertNotNull(win.removeOOBMessage());
+        assert win.removeOOBMessage() != null;
         System.out.println("win = " + win);
-        assertNull(win.removeOOBMessage());
-        assertNotNull(win.remove());
-        assertNull(win.remove());
-        assertNull(win.removeOOBMessage());
+        assert win.removeOOBMessage() == null;
+        assert win.remove() != null;
+        assert win.remove() == null;
+        assert win.removeOOBMessage() == null;
     }
 
+    public void testRemoveRegularAndOOBMessages() {
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
+        win.add(1, msg());
+        System.out.println("win = " + win);
+        win.remove();
+        System.out.println("win = " + win);
+        assert win.getHighestDelivered() == 1;
+
+        win.add(3, msg());
+        win.remove();
+        System.out.println("win = " + win);
+        assert win.getHighestDelivered() == 1;
+
+        win.add(2, oob());
+        win.removeOOBMessage();
+        System.out.println("win = " + win);
+
+        assert win.getHighestDelivered() == 2;
+    }
+
+    public void testRemoveMany() {
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
+        win.add(1, msg());
+        win.add(2, msg());
+        win.add(3, msg());
+        win.add(5, msg());
+        win.add(6, msg());
+        System.out.println("win = " + win);
+        List<Message> msgs=win.removeMany();
+        System.out.println("msgs = " + msgs);
+        assert msgs.size() == 3;
+
+        win.add(4, msg());
+        msgs=win.removeMany();
+        System.out.println("msgs = " + msgs);
+        assert msgs.size() == 3;
+    }
+
+
+    public void testRetransmitter() {
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 0, timer);
+        win.add(1, msg());
+        win.add(2, msg());
+        win.add(3, msg());
+        win.add(5, msg());
+        win.add(6, msg());
+        System.out.println("win = " + win);
+        int num_pending_xmits=win.getPendingXmits();
+        assert num_pending_xmits == 1;
+
+        win.add(4, msg());
+        num_pending_xmits=win.getPendingXmits();
+        assert num_pending_xmits == 0;
+    }
+
+
 
     private void add(int num_msgs) {
         long start, stop;
         double time_per_msg;
-        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 1);
+        NakReceiverWindow win=new NakReceiverWindow(sender, cmd, 1, timer);
         start=System.currentTimeMillis();
         for(int i=1; i < 1 + num_msgs; i++) {
             win.add(i, new Message());
@@ -507,20 +603,21 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
-    private Message oob() {
+    private static Message oob() {
         Message retval=new Message();
         retval.setFlag(Message.OOB);
         return retval;
     }
 
+    private static Message msg() {
+        return new Message();
+    }
 
 
-    private void check(NakReceiverWindow win, long lowest, long highest_received, long highest_delivered) {
-        assertEquals("lowest=" + lowest + ", win.lowest=" + win.getLowestSeen(), lowest, win.getLowestSeen());
-        assertEquals("highest_received=" + highest_received + ", win.highest_received=" + win.getHighestReceived(),
-                     highest_received, win.getHighestReceived());
-        assertEquals("highest_delivered=" + highest_delivered + ", win.highest_delivered=" + win.getHighestDelivered(), 
-                     highest_delivered, win.getHighestDelivered());
+    private static void check(NakReceiverWindow win, long lowest, long highest_received, long highest_delivered) {
+        Assert.assertEquals(win.getLowestSeen(), lowest, "lowest=" + lowest + ", win.lowest=" + win.getLowestSeen());
+        Assert.assertEquals(win.getHighestReceived(), highest_received, "highest_received=" + highest_received + ", win.highest_received=" + win.getHighestReceived());
+        Assert.assertEquals(win.getHighestDelivered(), highest_delivered, "highest_delivered=" + highest_delivered + ", win.highest_delivered=" + win.getHighestDelivered());
     }
 
 
@@ -531,11 +628,4 @@ public class NakReceiverWindowTest extends TestCase {
     }
 
 
-    public static Test suite() {
-        return new TestSuite(NakReceiverWindowTest.class);
-    }
-
-    public static void main(String[] args) {
-        junit.textui.TestRunner.run(suite());
-    }
 }
