@@ -1,19 +1,21 @@
 package org.jgroups.tests;
 
-import junit.framework.Test;
-import junit.framework.TestSuite;
+
+import org.testng.annotations.*;
 import org.jgroups.Address;
-import org.jgroups.MergeView;
+import org.jgroups.JChannel;
 import org.jgroups.Message;
 import org.jgroups.View;
+import org.jgroups.Global;
 import org.jgroups.util.Util;
+import org.testng.Assert;
+import org.testng.annotations.BeforeMethod;
 
 import java.io.*;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
-import java.util.Vector;
 import java.util.concurrent.Semaphore;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
@@ -22,24 +24,24 @@ import java.util.concurrent.atomic.AtomicInteger;
  * Tests concurrent startup with state transfer.
  * 
  * @author bela
- * @version $Id: ConcurrentStartupTest.java,v 1.32.2.8 2008/10/31 15:00:35 vlada Exp $
+ * @version $Id: ConcurrentStartupTest.java,v 1.48 2008/11/21 14:41:07 vlada Exp $
  */
+@Test(groups={Global.FLUSH},sequential=true)
 public class ConcurrentStartupTest extends ChannelTestBase {
-
     private AtomicInteger mod = new AtomicInteger(1);
 
-    public void setUp() throws Exception {
-        super.setUp();
-        mod.set(1);
-        CHANNEL_CONFIG = System.getProperty("channel.conf.flush", "flush-udp.xml");
-    }
 
-    public boolean useBlocking() {
-        return true;
+    @BeforeMethod
+    protected void setUp() throws Exception {
+        mod.set(1);
     }   
 
-    public void testConcurrentStartupState() {
-        concurrentStartupHelper(true);
+    public void testConcurrentStartupLargeState() {
+        concurrentStartupHelper(true, false);
+    }
+
+    public void testConcurrentStartupSmallState() {
+        concurrentStartupHelper(false, true);
     }
 
     /**
@@ -55,82 +57,139 @@ public class ConcurrentStartupTest extends ChannelTestBase {
      * 
      * 
      */
-    protected void concurrentStartupHelper(boolean useDispatcher) {
-        String[] names = new String[] { "A", "B", "C", "D" };
+    protected void concurrentStartupHelper(boolean largeState, boolean useDispatcher) {
+        String[] names=null;
 
-        int count = names.length;
+        names=new String[] { "A", "B", "C", "D" };
 
-        ConcurrentStartupChannel[] channels = new ConcurrentStartupChannel[count];
-        try{
+        int count=names.length;
+
+        ConcurrentStartupChannel[] channels=new ConcurrentStartupChannel[count];
+        try {
             // Create a semaphore and take all its permits
-            Semaphore semaphore = new Semaphore(count);
+            Semaphore semaphore=new Semaphore(count);
             semaphore.acquire(count);
 
             // Create activation threads that will block on the semaphore
             for(int i=0;i < count;i++) {
-
-                channels[i]=new ConcurrentStartupChannel(names[i], semaphore, useDispatcher);
+                if(largeState) {
+                    if(i == 0)
+                        channels[i]=new ConcurrentStartupChannelWithLargeState(semaphore,
+                                                                               names[i],
+                                                                               useDispatcher);
+                    else
+                        channels[i]=new ConcurrentStartupChannelWithLargeState((JChannel)channels[0].getChannel(),
+                                                                               semaphore,
+                                                                               names[i],
+                                                                               useDispatcher);
+                }
+                else {
+                    if(i == 0)
+                        channels[i]=new ConcurrentStartupChannel(names[i], semaphore, useDispatcher);
+                    else
+                        channels[i]=new ConcurrentStartupChannel((JChannel)channels[0].getChannel(),
+                                                                 names[i],
+                                                                 semaphore,
+                                                                 useDispatcher);
+                }
 
                 // Release one ticket at a time to allow the thread to start
                 // working
                 channels[i].start();
-                semaphore.release(1);                                
+                semaphore.release(1);
+                //sleep at least a second and max second and a half
+                Util.sleepRandom(1500);
             }
 
             // Make sure everyone is in sync
+
             blockUntilViewsReceived(channels, 60000);
 
-
-            for(ConcurrentStartupChannel c: channels) {
-                View view=c.getChannel().getView();
-                System.out.println("view = " + view);
-            }
-
             // Sleep to ensure the threads get all the semaphore tickets
-            Util.sleep(1000);
+            Util.sleep(2000);
 
             // Reacquire the semaphore tickets; when we have them all
             // we know the threads are done
-            boolean acquired = semaphore.tryAcquire(count, 20, TimeUnit.SECONDS);
-            if(!acquired){
+            boolean acquired=semaphore.tryAcquire(count, 20, TimeUnit.SECONDS);
+            if(!acquired) {
                 log.warn("Most likely a bug, analyse the stack below:");
                 log.warn(Util.dumpThreads());
             }
 
             // Sleep to ensure async message arrive
-            Util.sleep(1000);
+            Util.sleep(3000);
 
             // do test verification            
-            for (ConcurrentStartupChannel channel : channels) {
-                log.info(channel.getName() +"=" +channel.getList());  
+            for(ConcurrentStartupChannel channel:channels) {
+                log.info(channel.getName() + "=" + channel.getList());
             }
-            for (ConcurrentStartupChannel channel : channels) {
-                log.info(channel.getName() +"=" +channel.getModifications());  
+            for(ConcurrentStartupChannel channel:channels) {
+                log.info(channel.getName() + "=" + channel.getModifications());
             }
-            
-            
-            for (ConcurrentStartupChannel channel : channels) {
-                assertEquals(channel.getName() + " should have " + count + " elements", count, channel.getList().size());
+
+            for(ConcurrentStartupChannel channel:channels) {
+                Assert.assertEquals(channel.getList().size(),
+                                    count,
+                                    channel.getName() + " should have " + count + " elements");
             }
-            
-            for(ConcurrentStartupChannel channel:channels){                
+        }
+        catch(Exception ex) {
+            log.warn("Exception encountered during test", ex);
+            assert false:ex.getLocalizedMessage();
+        }
+        finally {
+            for(ConcurrentStartupChannel channel:channels) {
+                channel.cleanup();
+                Util.sleep(2000); // remove before 2.6 GA
+            }
+
+            for(ConcurrentStartupChannel channel:channels) {
                 checkEventStateTransferSequence(channel);
             }
-        }catch(Exception ex){
-            log.warn("Exception encountered during test", ex);
-            fail(ex.getLocalizedMessage());
-        }finally{
-            for(int i = count-1;i >=0;i--){
-                channels[i].cleanup();
-                Util.sleep(250);
-            }                
         }
     }
 
     protected int getMod() {       
         return mod.incrementAndGet();
-    } 
-    
+    }  
+
+    protected class ConcurrentStartupChannelWithLargeState extends ConcurrentStartupChannel {
+        
+    	//depends on retry_timeout parameter in FLUSH
+    	private static final long TRANSFER_TIME = 1000; 
+        public ConcurrentStartupChannelWithLargeState(Semaphore semaphore,
+                                                      String name,
+                                                      boolean useDispatcher) throws Exception{
+            super(name, semaphore, useDispatcher);
+        }
+        
+        public ConcurrentStartupChannelWithLargeState(JChannel ch, Semaphore semaphore,
+                                                      String name,
+                                                      boolean useDispatcher) throws Exception{
+            super(ch,name, semaphore, useDispatcher);
+        }
+      
+        public void setState(byte[] state) {
+            Util.sleep(TRANSFER_TIME);
+            super.setState(state);
+        }
+
+        public byte[] getState() {
+            Util.sleep(TRANSFER_TIME);
+            return super.getState();
+        }
+
+        public void getState(OutputStream ostream) {
+            Util.sleep(TRANSFER_TIME);
+            super.getState(ostream);
+        }
+
+        public void setState(InputStream istream) {
+            Util.sleep(TRANSFER_TIME);
+            super.setState(istream);
+        }
+    }
+
     protected class ConcurrentStartupChannel extends PushChannelApplicationWithSemaphore {
         private final List<Address> l = new LinkedList<Address>();       
 
@@ -139,12 +198,14 @@ public class ConcurrentStartupTest extends ChannelTestBase {
         public ConcurrentStartupChannel(String name,Semaphore semaphore,boolean useDispatcher) throws Exception{
             super(name, semaphore, useDispatcher);
         }
+        
+        public ConcurrentStartupChannel(JChannel ch,String name,Semaphore semaphore,boolean useDispatcher) throws Exception{
+            super(ch,name, semaphore, useDispatcher);
+        }
 
         public void useChannel() throws Exception {
             channel.connect("test", null, null, 25000);
-            LinkedList<Address> l =new LinkedList<Address>();
-            l.add(channel.getLocalAddress());
-            channel.send(null, null, l);
+            channel.send(null, null, channel.getLocalAddress());
         }
 
         List<Address> getList() {
@@ -158,10 +219,10 @@ public class ConcurrentStartupTest extends ChannelTestBase {
         public void receive(Message msg) {
             if(msg.getBuffer() == null)
                 return;
-            List<Address> obj = (List)msg.getObject();
+            Address obj = (Address)msg.getObject();
             log.info("-- [#" + getName() + " (" + channel.getLocalAddress() + ")]: received " + obj);
             synchronized(this){
-                l.addAll(obj);
+                l.add(obj);
                 Integer key = new Integer(getMod());
                 mods.put(key, obj);
             }
@@ -169,33 +230,9 @@ public class ConcurrentStartupTest extends ChannelTestBase {
 
         public void viewAccepted(View new_view) {
             super.viewAccepted(new_view);
-            synchronized(this) {
-                Integer key=new Integer(getMod());
+            synchronized(this){
+                Integer key = new Integer(getMod());
                 mods.put(key, new_view.getVid());
-
-                if(new_view instanceof MergeView) {
-                    MergeView mv=(MergeView)new_view;
-                    Vector<View> subgroups=mv.getSubgroups();
-                    boolean amISubgroupLeader=false;
-                    for(View view:subgroups) {
-                        Address subCoord=view.getMembers().firstElement();
-                        amISubgroupLeader=getLocalAddress().equals(subCoord);
-                        if(amISubgroupLeader) {
-                            for(View view2:subgroups) {
-                                if(!getLocalAddress().equals(view2.getMembers().firstElement())) {
-                                    for(Address member:view2.getMembers()) {
-                                        Message m=new Message(member, null, (Serializable)l);
-                                        try {
-                                            channel.send(m);
-                                        }
-                                        catch(Exception e) {
-                                        }
-                                    }
-                                }
-                            }
-                        }
-                    }
-                }
             }
         }
 
@@ -276,12 +313,5 @@ public class ConcurrentStartupTest extends ChannelTestBase {
         }
     }
 
-    public static Test suite() {
-        return new TestSuite(ConcurrentStartupTest.class);
-    }
 
-    public static void main(String[] args) {
-        String[] testCaseName = { ConcurrentStartupTest.class.getName() };
-        junit.textui.TestRunner.main(testCaseName);
-    }
 }
