@@ -1,10 +1,18 @@
 package org.jgroups.tests;
 
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.ObjectInputStream;
-import java.io.ObjectOutputStream;
-import java.io.OutputStream;
+
+import org.testng.annotations.*;
+import org.jgroups.Address;
+import org.jgroups.Channel;
+import org.jgroups.Global;
+import org.jgroups.JChannel;
+import org.jgroups.Message;
+import org.jgroups.View;
+import org.jgroups.util.Util;
+import org.testng.Assert;
+import org.testng.annotations.BeforeMethod;
+
+import java.io.*;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
@@ -13,40 +21,29 @@ import java.util.concurrent.Semaphore;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 
-import junit.framework.Test;
-import junit.framework.TestSuite;
-
-import org.jgroups.Address;
-import org.jgroups.Channel;
-import org.jgroups.Message;
-import org.jgroups.View;
-import org.jgroups.util.Util;
-
 /**
  * Tests concurrent state transfer with flush.
  * 
  * @author bela
- * @version $Id: ConcurrentStateTransferTest.java,v 1.2.2.4 2008/10/30 15:03:49 belaban Exp $
+ * @version $Id: ConcurrentStateTransferTest.java,v 1.15 2008/11/21 15:47:18 vlada Exp $
  */
+@Test(groups={Global.FLUSH},sequential=true)
 public class ConcurrentStateTransferTest extends ChannelTestBase {
 
     private final AtomicInteger mod = new AtomicInteger(1);
 
+    @BeforeMethod
     public void setUp() throws Exception {
-        super.setUp();
         mod.set(1);
-        CHANNEL_CONFIG = System.getProperty("channel.conf.flush", "flush-udp.xml");
-    }
-
-    public boolean useBlocking() {
-        return true;
-    }  
+    }    
 
     
+    @Test
     public void testConcurrentLargeStateTransfer() {
         concurrentStateTranferHelper(true, false);
     }
 
+    @Test
     public void testConcurrentSmallStateTranfer() {
         concurrentStateTranferHelper(false, true);
     }
@@ -60,80 +57,94 @@ public class ConcurrentStateTransferTest extends ChannelTestBase {
      * 
      */
     protected void concurrentStateTranferHelper(boolean largeState, boolean useDispatcher) {
-        String[] names = new String[] { "A", "B", "C", "D" };
+        String[] names=new String[] { "A", "B", "C", "D" };
 
-        int count = names.length;
-        ConcurrentStateTransfer[] channels = new ConcurrentStateTransfer[count];
+        int count=names.length;
+        ConcurrentStateTransfer[] channels=new ConcurrentStateTransfer[count];
 
         // Create a semaphore and take all its tickets
-        Semaphore semaphore = new Semaphore(count);
+        Semaphore semaphore=new Semaphore(count);
 
-        try{
+        try {
 
             semaphore.acquire(count);
             // Create activation threads that will block on the semaphore
-            for(int i = 0;i < count;i++){
-                if(largeState){
-                    channels[i] = new ConcurrentLargeStateTransfer(names[i],
-                                                                   semaphore,
-                                                                   useDispatcher);                                     
-                }else{                   
-                    channels[i] = new ConcurrentStateTransfer(names[i],
-                                                              semaphore,
-                                                              useDispatcher);                   
+            for(int i=0;i < count;i++) {
+                if(largeState) {
+                    if(i == 0)
+                        channels[i]=new ConcurrentLargeStateTransfer(names[i],
+                                                                     semaphore,
+                                                                     useDispatcher);
+                    else
+                        channels[i]=new ConcurrentLargeStateTransfer((JChannel)channels[0].getChannel(),
+                                                                     names[i],
+                                                                     semaphore,
+                                                                     useDispatcher);
+
+                }
+                else {
+                    if(i == 0)
+                        channels[i]=new ConcurrentStateTransfer(names[i], semaphore, useDispatcher);
+                    else
+                        channels[i]=new ConcurrentStateTransfer((JChannel)channels[0].getChannel(),
+                                                                names[i],
+                                                                semaphore,
+                                                                useDispatcher);
                 }
 
                 // Start threads and let them join the channel
                 channels[i].start();
-                Util.sleep(500);
+                //Util.sleep(2000);
             }
 
             // Make sure everyone is in sync
 
             blockUntilViewsReceived(channels, 60000);
 
-            Util.sleep(1000);
+            Util.sleep(2000);
             // Unleash hell !
             semaphore.release(count);
 
             // Sleep to ensure the threads get all the semaphore tickets
-            Util.sleep(1000);
+            Util.sleep(2000);
 
             // Reacquire the semaphore tickets; when we have them all
             // we know the threads are done
-            boolean acquired = semaphore.tryAcquire(count, 30, TimeUnit.SECONDS);
-            if(!acquired){
+            boolean acquired=semaphore.tryAcquire(count, 30, TimeUnit.SECONDS);
+            if(!acquired) {
                 log.warn("Most likely a bug, analyse the stack below:");
                 log.warn(Util.dumpThreads());
             }
 
             // Sleep to ensure async message arrive
             Util.sleep(2000);
-            
+
             // do test verification            
-            for (ConcurrentStateTransfer channel : channels) {
-                log.info(channel.getName() +"=" +channel.getList());  
+            for(ConcurrentStateTransfer channel:channels) {
+                log.info(channel.getName() + "=" + channel.getList());
             }
-            for (ConcurrentStateTransfer channel : channels) {
-                log.info(channel.getName() +"=" +channel.getModifications());  
+            for(ConcurrentStateTransfer channel:channels) {
+                log.info(channel.getName() + "=" + channel.getModifications());
             }
-            
-            
-            for (ConcurrentStateTransfer channel : channels) {
-                assertEquals(channel.getName() + " should have " + count + " elements", count, channel.getList().size());
+
+            for(ConcurrentStateTransfer channel:channels) {
+                Assert.assertEquals(channel.getList().size(),
+                                    count,
+                                    channel.getName() + " should have " + count + " elements");
             }
-            
-            for(ConcurrentStateTransfer channel:channels){                
-                checkEventStateTransferSequence(channel);
-            }   
-        }catch(Exception ex){
+        }
+        catch(Exception ex) {
             log.warn("Exception encountered during test", ex);
-            fail(ex.getLocalizedMessage());
-        }finally{
-            for(int i = count-1;i >=0;i--){
-                channels[i].cleanup();
-                Util.sleep(250);
-            }                               
+            assert false:ex.getLocalizedMessage();
+        }
+        finally {
+            for(ConcurrentStateTransfer channel:channels) {
+                channel.cleanup();
+                Util.sleep(2000); 
+            }
+            for(ConcurrentStateTransfer channel:channels) {
+                checkEventStateTransferSequence(channel);
+            }
         }
     }
     
@@ -151,7 +162,12 @@ public class ConcurrentStateTransferTest extends ChannelTestBase {
 
         public ConcurrentStateTransfer(String name,Semaphore semaphore,boolean useDispatcher) throws Exception{
             super(name, semaphore, useDispatcher);
-            channel.connect("test");
+            channel.connect("ConcurrentStateTransfer");
+        }
+        
+        public ConcurrentStateTransfer(JChannel ch, String name,Semaphore semaphore,boolean useDispatcher) throws Exception{
+            super(ch,name, semaphore, useDispatcher);
+            channel.connect("ConcurrentStateTransfer");
         }
 
         public void useChannel() throws Exception {
@@ -269,11 +285,16 @@ public class ConcurrentStateTransferTest extends ChannelTestBase {
     }
 
     protected class ConcurrentLargeStateTransfer extends ConcurrentStateTransfer {
-        private static final long TRANSFER_TIME = 2500;
+    	//depends on retry_timeout parameter in FLUSH
+        private static final long TRANSFER_TIME = 1000;
         public ConcurrentLargeStateTransfer(String name,Semaphore semaphore,boolean useDispatcher) throws Exception{
             super(name, semaphore, useDispatcher);
         }
         
+        public ConcurrentLargeStateTransfer(JChannel channel,String name,Semaphore semaphore,boolean useDispatcher) throws Exception{
+            super(channel,name, semaphore, useDispatcher);
+        }
+        
         public void setState(byte[] state) {
             Util.sleep(TRANSFER_TIME);
             super.setState(state);
@@ -295,12 +316,4 @@ public class ConcurrentStateTransferTest extends ChannelTestBase {
         }
     }
     
-    public static Test suite() {
-        return new TestSuite(ConcurrentStateTransferTest.class);
-    }
-
-    public static void main(String[] args) {
-        String[] testCaseName = { ConcurrentStateTransferTest.class.getName() };
-        junit.textui.TestRunner.main(testCaseName);
-    }
 }
