@@ -1,15 +1,17 @@
 package org.jgroups.tests;
 
-import junit.framework.Test;
-import junit.framework.TestSuite;
+
 import org.jgroups.*;
-import org.jgroups.stack.Protocol;
 import org.jgroups.util.Util;
+import org.testng.annotations.Test;
 
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.LinkedList;
+import java.util.List;
 import java.util.concurrent.Semaphore;
 import java.util.concurrent.TimeUnit;
 
@@ -18,47 +20,12 @@ import java.util.concurrent.TimeUnit;
  * configured to use FLUSH
  * 
  * @author Bela Ban
- * @version $Id: FlushTest.java,v 1.58.2.4 2008/10/30 15:03:49 belaban Exp $
+ * @version $Id: FlushTest.java,v 1.76 2008/11/20 17:54:49 vlada Exp $
  */
+@Test(groups=Global.FLUSH,sequential=true)
 public class FlushTest extends ChannelTestBase {
-    private JChannel c1, c2;
-
-    public FlushTest(){
-        super();
-    }
-
-    public FlushTest(String name){
-        super(name);
-    }
-
-    public void setUp() throws Exception {
-        super.setUp();
-        CHANNEL_CONFIG = System.getProperty("channel.conf.flush", "flush-udp.xml");
-    }
-
-    public void tearDown() throws Exception {
-        if(c2 != null){
-            c2.close();
-            assertFalse(c2.isOpen());
-            assertFalse(c2.isConnected());
-            c2 = null;
-        }
-
-        if(c1 != null){
-            c1.close();
-            assertFalse(c1.isOpen());
-            assertFalse(c1.isConnected());
-            c1 = null;
-        }
-
-        Util.sleep(500);
-        super.tearDown();
-    }
-
-    public boolean useBlocking() {
-        return true;
-    }
-
+    
+    @Test
     public void testSingleChannel() throws Exception {
         Semaphore s = new Semaphore(1);
         FlushTestReceiver receivers[] = new FlushTestReceiver[] { new FlushTestReceiver("c1",
@@ -81,72 +48,197 @@ public class FlushTest extends ChannelTestBase {
         Util.sleep(1000);
 
         checkEventStateTransferSequence(receivers[0]);
-
     }
 
     /**
      * Tests issue #1 in http://jira.jboss.com/jira/browse/JGRP-335
+     * @throws Exception 
      */
-    public void testJoinFollowedByUnicast() throws ChannelException {
-        c1 = createChannel();
-        c1.setReceiver(new SimpleReplier(c1, true));
-        c1.connect("test");
+    @Test
+    public void testJoinFollowedByUnicast() throws Exception {
+        JChannel c1=null;
+        JChannel c2=null;
+        try {
+            c1=createChannel(true, 2);
+            c1.setReceiver(new SimpleReplier(c1, true));
+            c1.connect("testJoinFollowedByUnicast");
 
-        Address target = c1.getLocalAddress();
-        Message unicast_msg = new Message(target);
+            Address target=c1.getLocalAddress();
+            Message unicast_msg=new Message(target);
 
-        c2 = createChannel();
-        c2.setReceiver(new SimpleReplier(c2, false));
-        c2.connect("test");
+            c2=createChannel(c1);
+            c2.setReceiver(new SimpleReplier(c2, false));
+            c2.connect("testJoinFollowedByUnicast");
 
-        // now send unicast, this might block as described in the case
-        c2.send(unicast_msg);
-        // if we don't get here this means we'd time out
+            // now send unicast, this might block as described in the case
+            c2.send(unicast_msg);
+            // if we don't get here this means we'd time out
+        }
+        finally {
+            Util.close(c2, c1);
+        }
     }
 
     /**
      * Tests issue #2 in http://jira.jboss.com/jira/browse/JGRP-335
+     * @throws Exception 
      */
-    public void testStateTransferFollowedByUnicast() throws ChannelException {
-        c1 = createChannel();
-        c1.setReceiver(new SimpleReplier(c1, true));
-        c1.connect("test");
+    @Test
+    public void testStateTransferFollowedByUnicast() throws Exception {
+        JChannel c1=null;
+        JChannel c2=null;
+        try {
 
-        Address target = c1.getLocalAddress();
-        Message unicast_msg = new Message(target);
+            c1=createChannel(true, 2);
+            c1.setReceiver(new SimpleReplier(c1, true));
+            c1.connect("testStateTransferFollowedByUnicast");
 
-        c2 = createChannel();
-        c2.setReceiver(new SimpleReplier(c2, false));
-        c2.connect("test");
+            Address target=c1.getLocalAddress();
+            Message unicast_msg=new Message(target);
 
-        log.info("\n** Getting the state **");
-        c2.getState(null, 10000);
-        // now send unicast, this might block as described in the case
-        c2.send(unicast_msg);
+            c2=createChannel(c1);
+            c2.setReceiver(new SimpleReplier(c2, false));
+            c2.connect("testStateTransferFollowedByUnicast");
+
+            log.info("\n** Getting the state **");
+            c2.getState(null, 10000);
+            // now send unicast, this might block as described in the case
+            c2.send(unicast_msg);
+        }
+        finally {
+            Util.close(c2, c1);
+        }
     }
     
+    @Test
+    public void testFlushWithCrashedFlushCoordinator() throws Exception {
+		JChannel c1 = null;
+		JChannel c2 = null;
+		JChannel c3 = null;
+		
+		try {
+			c1 = createChannel(true, 3);
+			c1.connect("testFlushWithCrashedFlushCoordinator");
+
+			c2 = createChannel(c1);
+			c2.connect("testFlushWithCrashedFlushCoordinator");
+
+			c3 = createChannel(c1);
+			c3.connect("testFlushWithCrashedFlushCoordinator");
+
+			// start flush
+			c2.startFlush(false);
+
+			// and then kill the flush coordinator
+			((JChannel) c2).shutdown();
+
+			Util.sleep(8000);
+
+			// cluster should not hang and two remaining members should have a
+			// correct view
+			assertTrue("corret view size", c1.getView().size() == 2);
+			assertTrue("corret view size", c3.getView().size() == 2);
+		} finally {
+			Util.close(c3, c2, c1);
+		}
+	}
+    
+    @Test
+	public void testFlushWithCrashedNonCoordinator() throws Exception {
+		JChannel c1 = null;
+		JChannel c2 = null;
+		JChannel c3 = null;
+
+		try {
+			c1 = createChannel(true, 3);
+			c1.connect("testFlushWithCrashedFlushCoordinator");
+
+			c2 = createChannel(c1);
+			c2.connect("testFlushWithCrashedFlushCoordinator");
+
+			c3 = createChannel(c1);
+			c3.connect("testFlushWithCrashedFlushCoordinator");
+
+			// start flush
+			c2.startFlush(false);
+
+			// and then kill the flush coordinator
+			((JChannel) c3).shutdown();
+
+			c2.stopFlush();
+			Util.sleep(8000);
+
+			// cluster should not hang and two remaining members should have a
+			// correct view
+			assertTrue("corret view size", c1.getView().size() == 2);
+			assertTrue("corret view size", c2.getView().size() == 2);
+		} finally {
+			Util.close(c3, c2, c1);
+		}
+	}
+
+	@Test
+	public void testFlushWithCrashedNonCoordinators() throws Exception {
+		JChannel c1 = null;
+		JChannel c2 = null;
+		JChannel c3 = null;
+
+		try {
+			c1 = createChannel(true, 3);
+			c1.connect("testFlushWithCrashedFlushCoordinator");
+
+			c2 = createChannel(c1);
+			c2.connect("testFlushWithCrashedFlushCoordinator");
+
+			c3 = createChannel(c1);
+			c3.connect("testFlushWithCrashedFlushCoordinator");
+
+			// start flush
+			c2.startFlush(false);
+
+			// and then kill members other than flush coordinator
+			((JChannel) c3).shutdown();
+			((JChannel) c1).shutdown();
+
+			c2.stopFlush();
+			Util.sleep(8000);
+
+			// cluster should not hang and one remaining member should have a
+			// correct view
+			assertTrue("corret view size", c2.getView().size() == 1);
+		} finally {
+			Util.close(c3, c2, c1);
+		}
+	}
+    
     /**
      * Tests http://jira.jboss.com/jira/browse/JGRP-661
      * @throws Exception 
      */
+    @Test
     public void testPartialFlush() throws Exception {
-        c1 = createChannel();
-        c1.setReceiver(new SimpleReplier(c1, true));
-        c1.connect("test");
-    
-        c2 = createChannel();
-        c2.setReceiver(new SimpleReplier(c2, false));
-        c2.connect("test");
-       
-        List<Address> members = new ArrayList<Address>();
-        members.add(c2.getLocalAddress());       
-        boolean flushedOk = c2.startFlush(members, false);
-        
-        assertTrue("Partial flush worked", flushedOk);
-        
-        c2.stopFlush(members);
-               
+        JChannel c1=null;
+        JChannel c2=null;
+        try {
+            c1=createChannel(true, 2);
+            c1.setReceiver(new SimpleReplier(c1, true));
+            c1.connect("testPartialFlush");
 
+            c2=createChannel(c1);
+            c2.setReceiver(new SimpleReplier(c2, false));
+            c2.connect("testPartialFlush");
+
+            List<Address> members=new ArrayList<Address>();
+            members.add(c2.getLocalAddress());
+            boolean flushedOk=c2.startFlush(members, false);
+
+            assertTrue("Partial flush worked", flushedOk);
+
+            c2.stopFlush(members);
+        }
+        finally {
+            Util.close(c2, c1);
+        }
     }
 
     /**
@@ -156,13 +248,16 @@ public class FlushTest extends ChannelTestBase {
      * bare channel mode 4 real channels are created.
      * 
      */
+    @Test
     public void testBlockingNoStateTransfer() {
-        String[] names = createApplicationNames(4);
-        _testChannels(names, FlushTestReceiver.CONNECT_ONLY, 4);
+        String[] names = {"A", "B", "C", "D"};
+        _testChannels(names, FlushTestReceiver.CONNECT_ONLY);
     }
 
 
 
+
+
     /**
      * Tests emition of block/unblock/set|get state events for both mux and bare
      * channel depending on mux.on parameter. In mux mode there will be only one
@@ -170,9 +265,10 @@ public class FlushTest extends ChannelTestBase {
      * channels created is getMuxFactoryCount().
      * 
      */
+    @Test
     public void testBlockingWithStateTransfer() {
-        String[] names = createApplicationNames(4);
-        _testChannels(names, FlushTestReceiver.CONNECT_AND_SEPARATE_GET_STATE, 4);
+        String[] names = {"A", "B", "C", "D"};
+        _testChannels(names, FlushTestReceiver.CONNECT_AND_SEPARATE_GET_STATE);
     }
     
     /**
@@ -182,16 +278,15 @@ public class FlushTest extends ChannelTestBase {
      * channels created is getMuxFactoryCount().
      * 
      */
+    @Test
     public void testBlockingWithConnectAndStateTransfer() {
-        String[] names = createApplicationNames(4);
-        _testChannels(names, FlushTestReceiver.CONNECT_AND_GET_STATE, 4);
+        String[] names = {"A", "B", "C", "D"};
+        _testChannels(names, FlushTestReceiver.CONNECT_AND_GET_STATE);
     }
 
 
 
-    private void _testChannels(String names[],
-                               int connectType,
-                               Assertable a) {
+    private void _testChannels(String names[], int connectType) {
         int count = names.length;
 
         ArrayList<FlushTestReceiver> channels = new ArrayList<FlushTestReceiver>(count);
@@ -202,9 +297,16 @@ public class FlushTest extends ChannelTestBase {
 
             // Create channels and their threads that will block on the
             // semaphore
-            for(int i = 0;i < count;i++){
-                FlushTestReceiver channel = new FlushTestReceiver(names[i], semaphore, 0, connectType);                
+            boolean first = true;
+            for(String channelName:names){
+                FlushTestReceiver channel = null;
+                if(first)
+                    channel=new FlushTestReceiver(channelName, semaphore, 0, connectType);
+                else{
+                    channel=new FlushTestReceiver((JChannel)channels.get(0).getChannel(),channelName, semaphore, 0, connectType);
+                }
                 channels.add(channel);
+                first = false;
 
                 // Release one ticket at a time to allow the thread to start
                 // working
@@ -215,39 +317,16 @@ public class FlushTest extends ChannelTestBase {
 
             blockUntilViewsReceived(channels, 10000);
 
-            // if state transfer is used release all at once
-            // clear all channels of view events
-            if(connectType == FlushTestReceiver.CONNECT_AND_SEPARATE_GET_STATE){
-                for(FlushTestReceiver app:channels){
-                    app.clear();
-                }
-                semaphore.release(count);
-            }
-
             // Sleep to ensure the threads get all the semaphore tickets
             Util.sleep(1000);
 
             // Reacquire the semaphore tickets; when we have them all
             // we know the threads are done
-            semaphore.tryAcquire(count, 60, TimeUnit.SECONDS);
-
-            // do general asserts about channels
-            a.verify(channels);
-
-            // kill random member
-            FlushTestReceiver randomRecv = channels.remove(RANDOM.nextInt(count));
-            log.info("Closing random member " + randomRecv.getName()
-                     + " at "
-                     + randomRecv.getLocalAddress());
-            ChannelCloseAssertable closeAssert = new ChannelCloseAssertable(randomRecv);
-            randomRecv.cleanup();
-
-            // let the view propagate and verify related asserts
-            Util.sleep(5000);
-            closeAssert.verify(channels);                     
+            semaphore.tryAcquire(count, 40, TimeUnit.SECONDS);
+                      
         }catch(Exception ex){
             log.warn("Exception encountered during test", ex);
-            fail("Exception encountered during test execution: " + ex);
+            assert false : "Exception encountered during test execution: " + ex;
         }finally{
             
             //close all channels and ....
@@ -257,87 +336,13 @@ public class FlushTest extends ChannelTestBase {
             }
             
             // verify block/unblock/view/get|set state sequences for all members
-            for(FlushTestReceiver receiver:channels) {
-                checkEventStateTransferSequence(receiver);
-            }
+			for (FlushTestReceiver receiver : channels) {
+				checkEventStateTransferSequence(receiver);
+			}
         }
     }
 
-    private  void _testChannels(String names[], int connectMethod, int viewSize) {
-        _testChannels(names, connectMethod, new ChannelAssertable(viewSize));
-    }
-
-    private class ChannelCloseAssertable implements Assertable {
-        ChannelApplication app;
-
-        View viewBeforeClose;
-
-        Address appAddress;
-
-
-        public ChannelCloseAssertable(ChannelApplication app){
-            this.app = app;
-            this.viewBeforeClose = app.getChannel().getView();
-            appAddress = app.getChannel().getLocalAddress();
-        }
-
-        public void verify(Object verifiable) {
-            Channel ch = app.getChannel();
-            assertFalse("Channel open", ch.isOpen());
-            assertFalse("Chnanel connected", ch.isConnected());
-        }
-    }
-
-    private class ChannelAssertable implements Assertable {
-        int expectedViewSize = 0;
-
-        public ChannelAssertable(int expectedViewSize){
-            this.expectedViewSize = expectedViewSize;
-        }
-
-        public void verify(Object verifiable) {
-            Collection channels = (Collection) verifiable;
-            for(Iterator iter = channels.iterator();iter.hasNext();){
-                FlushTestReceiver receiver = (FlushTestReceiver) iter.next();
-                Channel ch = receiver.getChannel();
-                assertEquals("Correct view", ch.getView().getMembers().size(), expectedViewSize);
-                assertTrue("Channel open", ch.isOpen());
-                assertTrue("Chnanel connected", ch.isConnected());
-                assertNotNull("Valid address ", ch.getLocalAddress());
-                assertTrue("Address included in view ", ch.getView()
-                                                          .getMembers()
-                                                          .contains(ch.getLocalAddress()));
-                assertNotNull("Valid cluster name ", ch.getClusterName());
-            }
-        }
-    }
-    
-    protected JChannel createChannel() throws ChannelException {
-        JChannel ret = new JChannel(CHANNEL_CONFIG);
-        ret.setOpt(Channel.BLOCK, Boolean.TRUE);
-        Protocol flush = ret.getProtocolStack().findProtocol("FLUSH");
-        if(flush != null){
-            Properties p = new Properties();
-            p.setProperty("timeout", "0");
-            flush.setProperties(p);
-
-            // send timeout up and down the stack, so other protocols can use
-            // the same value too
-            Map<String, Object> map = new HashMap<String, Object>();
-            map.put("flush_timeout", new Long(0));
-            flush.getUpProtocol().up(new Event(Event.CONFIG, map));
-            flush.getDownProtocol().down(new Event(Event.CONFIG, map));
-        }
-        return ret;
-    }
-
-    private interface Assertable {
-        public void verify(Object verifiable);
-    }
-
     private class FlushTestReceiver extends PushChannelApplicationWithSemaphore {
-        List<Object> events;
-        
         private int connectMethod;
         
         public static final int CONNECT_ONLY = 1;
@@ -357,10 +362,26 @@ public class FlushTest extends ChannelTestBase {
             this.msgCount = msgCount;
             events = Collections.synchronizedList(new LinkedList<Object>());
             if(connectMethod == CONNECT_ONLY || connectMethod == CONNECT_AND_SEPARATE_GET_STATE)
-            	channel.connect("test");
+            	channel.connect("FlushTestReceiver");
             
             if(connectMethod == CONNECT_AND_GET_STATE){
-                channel.connect("test",null,null, 25000);
+                channel.connect("FlushTestReceiver",null,null, 25000);
+            }
+        }        
+        
+        protected FlushTestReceiver(JChannel ch, String name,
+                                    Semaphore semaphore,
+                                    int msgCount,
+                                    int connectMethod) throws Exception{
+            super(ch,name, semaphore,false);
+            this.connectMethod = connectMethod;
+            this.msgCount = msgCount;
+            events = Collections.synchronizedList(new LinkedList<Object>());
+            if(connectMethod == CONNECT_ONLY || connectMethod == CONNECT_AND_SEPARATE_GET_STATE)
+                channel.connect("FlushTestReceiver");
+            
+            if(connectMethod == CONNECT_AND_GET_STATE){
+                channel.connect("FlushTestReceiver",null,null, 25000);
             }
         }        
 
@@ -472,11 +493,4 @@ public class FlushTest extends ChannelTestBase {
         }
     }
 
-    public static Test suite() {
-        return new TestSuite(FlushTest.class);
-    }
-
-    public static void main(String[] args) {
-        junit.textui.TestRunner.run(FlushTest.suite());
-    }
 }
