@@ -1,47 +1,42 @@
 package org.jgroups.tests;
 
-import junit.framework.TestCase;
 import org.jgroups.stack.GossipRouter;
 import org.jgroups.JChannel;
 import org.jgroups.View;
 import org.jgroups.ReceiverAdapter;
+import org.jgroups.Global;
 import org.jgroups.protocols.MERGE2;
 import org.jgroups.protocols.TUNNEL;
 import org.jgroups.protocols.PING;
 import org.jgroups.util.Util;
+import org.testng.Assert;
+import org.testng.annotations.AfterMethod;
+import org.testng.annotations.Test;
 
+import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.concurrent.locks.Condition;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
-import java.util.concurrent.locks.Condition;
-import java.util.concurrent.atomic.AtomicBoolean;
-import java.util.concurrent.TimeUnit;
 
 /**
  * @author Bela Ban
- * @version $Id: GossipRouterTest.java,v 1.1.2.5 2008/10/30 11:18:56 belaban Exp $
+ * @version $Id: GossipRouterTest.java,v 1.8 2008/10/31 08:38:42 belaban Exp $
  */
-public class GossipRouterTest extends TestCase {
+@Test(groups=Global.STACK_INDEPENDENT, sequential=true)
+public class GossipRouterTest {
     final static String PROPS="tunnel.xml";
     GossipRouter router;
     JChannel c1, c2;
 
-    protected void setUp() throws Exception {
-        super.setUp();
-
-    }
 
+    @AfterMethod
     protected void tearDown() throws Exception {
         if(router != null) {
             router.stop();
             router=null;
         }
-        if(c2 != null) {
-            c2.close();
-        }
-        if(c1 != null) {
-            c1.close();
-        }
-        super.tearDown();
+        Util.close(c2, c1);
     }
 
     /**
@@ -51,6 +46,7 @@ public class GossipRouterTest extends TestCase {
      * - Now first node should be able to connect and first and second node should be able to merge into a group
      * - SUCCESS: a view of 2
      */
+    @Test
     public void testLateStart() throws Exception {
         final Lock lock=new ReentrantLock();
         final Condition cond=lock.newCondition();
@@ -91,7 +87,7 @@ public class GossipRouterTest extends TestCase {
         Util.sleep(500);
         View view=c1.getView();
         System.out.println("view=" + view);
-        assertEquals(2, view.size());
+        Assert.assertEquals(2, view.size());
 
         c2.close();
         c1.close();
