@@ -5,8 +5,20 @@ import org.jgroups.conf.ConfiguratorFactory;
 import org.jgroups.conf.ProtocolData;
 import org.jgroups.conf.ProtocolParameter;
 import org.jgroups.conf.ProtocolStackConfigurator;
+import org.jgroups.protocols.BasicTCP;
+import org.jgroups.protocols.TCPPING;
+import org.jgroups.protocols.TP;
+import org.jgroups.protocols.UDP;
+import org.jgroups.stack.Protocol;
+import org.jgroups.stack.ProtocolStack;
+import org.jgroups.util.ResourceManager;
+import org.jgroups.util.TimeScheduler;
 import org.jgroups.util.Util;
+import org.testng.AssertJUnit;
+import org.testng.annotations.AfterMethod;
+import org.testng.annotations.Test;
 
+import java.net.InetAddress;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.concurrent.CountDownLatch;
@@ -16,47 +28,46 @@ import java.util.concurrent.TimeUnit;
 /**
  * Tests which test the shared transport
  * @author Bela Ban
- * @version $Id: SharedTransportTest.java,v 1.5.2.9 2008/10/28 14:16:09 vlada Exp $
+ * @version $Id: SharedTransportTest.java,v 1.25 2008/10/28 14:28:55 vlada Exp $
  */
+@Test(groups=Global.STACK_DEPENDENT,sequential=true)
 public class SharedTransportTest extends ChannelTestBase {
     private JChannel a, b, c;
     private MyReceiver r1, r2, r3;
     static final String SINGLETON_1="singleton-1", SINGLETON_2="singleton-2";
 
 
+    @AfterMethod
     protected void tearDown() throws Exception {
-        if(c != null)
-            c.close();
-        if(b != null)
-            b.close();
-        if(a != null)
-            a.close();
+        Util.close(c,b,a);
         r1=r2=r3=null;
-        super.tearDown();
     }
 
 
     public void testCreationNonSharedTransport() throws Exception {
-        a=createChannel();
-        a.connect("x");
+        a=createChannel(true);
+        a.connect("SharedTransportTest.testCreationNonSharedTransport");
         View view=a.getView();
         System.out.println("view = " + view);
-        assertEquals(1, view.size());
+        assert view.size() == 1;
     }
 
+
     public void testCreationOfDuplicateCluster() throws Exception {
         a=createSharedChannel(SINGLETON_1);
+        // makeUnique(a, 2);
         b=createSharedChannel(SINGLETON_1);
         a.connect("x");
         try {
             b.connect("x");
-            fail("b should not be able to join cluster 'x' as a has already joined it");
+            assert false : "b should not be able to join cluster 'x' as a has already joined it";
         }
         catch(Exception ex) {
             System.out.println("b was not able to join the same cluster (\"x\") as expected");
         }
     }
 
+
     public void testView() throws Exception {
         a=createSharedChannel(SINGLETON_1);
         b=createSharedChannel(SINGLETON_2);
@@ -67,11 +78,12 @@ public class SharedTransportTest extends ChannelTestBase {
         b.connect("x");
 
         View view=a.getView();
-        assertEquals(2, view.size());
+        assert view.size() == 2;
         view=b.getView();
-        assertEquals(2, view.size());
+        assert view.size() == 2;
     }
 
+
     public void testView2() throws Exception {
         a=createSharedChannel(SINGLETON_1);
         b=createSharedChannel(SINGLETON_1);
@@ -82,22 +94,25 @@ public class SharedTransportTest extends ChannelTestBase {
         b.connect("y");
 
         View view=a.getView();
-        assertEquals(1, view.size());
+        assert view.size() == 1;
         view=b.getView();
-        assertEquals(1, view.size());
+        assert view.size() == 1;
     }
 
-    /** Tests http://jira.jboss.com/jira/browse/JGRP-689: with TCP or UDP.ip_mcast=false, the transport iterates through
+    /**
+     * Tests http://jira.jboss.com/jira/browse/JGRP-689: with TCP or UDP.ip_mcast=false, the transport iterates through
      * the 'members' instance variable to send a group message. However, the 'members' var is the value of the last
      * view change received. If we receive multiple view changes, this leads to incorrect membership.
-     *
      * @throws Exception
      */
+
     public void testView3() throws Exception {
         a=createSharedChannel(SINGLETON_1);
         b=createSharedChannel(SINGLETON_1);
         c=createSharedChannel(SINGLETON_2);
-        r1=new MyReceiver("A::" + SINGLETON_1); r2=new MyReceiver("B::" + SINGLETON_1); r3=new MyReceiver("C::" + SINGLETON_2);
+        r1=new MyReceiver("A::" + SINGLETON_1);
+        r2=new MyReceiver("B::" + SINGLETON_1);
+        r3=new MyReceiver("C::" + SINGLETON_2);
         a.setReceiver(r1);
         b.setReceiver(r2);
         c.setReceiver(r3);
@@ -106,20 +121,22 @@ public class SharedTransportTest extends ChannelTestBase {
         c.connect("a");
 
         View view=a.getView();
-        assertEquals(2, view.size());
+        assert view.size() == 2;
         view=c.getView();
-        assertEquals(2, view.size());
+        assert view.size() == 2;
 
         a.send(new Message(null, null, "msg-1"));
         c.send(new Message(null, null, "msg-2"));
 
         Util.sleep(1000); // async sending - wait a little
         List<Message> list=r1.getList();
-        assertEquals(2, list.size());
+        assert list.size() == 2;
         list=r3.getList();
-        assertEquals(2, list.size());
+        assert list.size() == 2;
 
-        r1.clear(); r2.clear(); r3.clear();
+        r1.clear();
+        r2.clear();
+        r3.clear();
         b.connect("b");
 
         a.send(new Message(null, null, "msg-3"));
@@ -129,25 +146,13 @@ public class SharedTransportTest extends ChannelTestBase {
 
         // printLists(r1, r2, r3);
         list=r1.getList();
-        assertEquals(2, list.size());
+        assert list.size() == 2;
         list=r2.getList();
-        assertEquals(1, list.size());
+        assert list.size() == 1;
         list=r3.getList();
-        assertEquals(2, list.size());
+        assert list.size() == 2;
     }
 
-    private static void  printLists(MyReceiver... receivers) {
-        StringBuilder sb=new StringBuilder();
-        int cnt=1;
-        for(MyReceiver rec: receivers) {
-            List<Message> list=rec.getList();
-            sb.append("receiver #" + cnt++).append(":\n");
-            for(Message msg: list) {
-                sb.append(msg.getObject()).append("\n");
-            }
-        }
-        System.out.println(sb);
-    }
 
 
     public void testSharedTransportAndNonsharedTransport() throws Exception {
@@ -160,11 +165,12 @@ public class SharedTransportTest extends ChannelTestBase {
         b.connect("x");
 
         View view=a.getView();
-        assertEquals(2, view.size());
+        assert view.size() == 2;
         view=b.getView();
-        assertEquals(2, view.size());
+        assert view.size() == 2;
     }
 
+
     public void testCreationOfDifferentCluster() throws Exception {
         a=createSharedChannel(SINGLETON_1);
         b=createSharedChannel(SINGLETON_2);
@@ -172,10 +178,11 @@ public class SharedTransportTest extends ChannelTestBase {
         b.connect("x");
         View view=b.getView();
         System.out.println("b's view is " + view);
-        assertEquals(2, view.size());
+        assert view.size() == 2;
     }
 
 
+
     public void testReferenceCounting() throws ChannelException {
         a=createSharedChannel(SINGLETON_1);
         r1=new MyReceiver("a");
@@ -197,33 +204,36 @@ public class SharedTransportTest extends ChannelTestBase {
         b.send(null, null, "message from b");
         c.send(null, null, "message from c");
         Util.sleep(500);
-        assertEquals(1, r1.size());
-        assertEquals(1, r2.size());
-        assertEquals(1, r3.size());
-        r1.clear(); r2.clear(); r3.clear();
+        assert r1.size() == 1;
+        assert r2.size() == 1;
+        assert r3.size() == 1;
+        r1.clear();
+        r2.clear();
+        r3.clear();
 
         b.disconnect();
         System.out.println("\n");
         a.send(null, null, "message from a");
         c.send(null, null, "message from c");
         Util.sleep(500);
-        assertEquals(1, r1.size());
-        assertEquals(1, r3.size());
-        r1.clear(); r3.clear();
+        assert r1.size() == 1 : "size should be 1 but is " + r1.size();
+        assert r3.size() == 1 : "size should be 1 but is " + r3.size();
+        r1.clear();
+        r3.clear();
 
         c.disconnect();
         System.out.println("\n");
         a.send(null, null, "message from a");
         Util.sleep(500);
-        assertEquals(1, r1.size());
+        assert r1.size() == 1;
     }
-    
+
     /**
      * Tests that a second channel with the same group name can be
      * created and connected once the first channel is disconnected.
-     *
      * @throws Exception
      */
+
     public void testSimpleReCreation() throws Exception {
         a=createSharedChannel(SINGLETON_1);
         a.setReceiver(new MyReceiver("A"));
@@ -239,9 +249,9 @@ public class SharedTransportTest extends ChannelTestBase {
      * created and connected once the first channel is disconnected even
      * if 3rd channel with a different group name is still using the shared
      * transport.
-     *
      * @throws Exception
      */
+
     public void testCreationFollowedByDeletion() throws Exception {
         a=createSharedChannel(SINGLETON_1);
         a.setReceiver(new MyReceiver("A"));
@@ -256,24 +266,25 @@ public class SharedTransportTest extends ChannelTestBase {
     }
 
 
-     public void test2ChannelsCreationFollowedByDeletion() throws Exception {
-         a=createSharedChannel(SINGLETON_1);
-         a.setReceiver(new MyReceiver("A"));
-         a.connect("A");
 
-         b=createSharedChannel(SINGLETON_2);
-         b.setReceiver(new MyReceiver("B"));
-         b.connect("A");
+    public void test2ChannelsCreationFollowedByDeletion() throws Exception {
+        a=createSharedChannel(SINGLETON_1);
+        a.setReceiver(new MyReceiver("A"));
+        a.connect("A");
 
-         c=createSharedChannel(SINGLETON_2);
-         c.setReceiver(new MyReceiver("C"));
-         c.connect("B");
+        b=createSharedChannel(SINGLETON_2);
+        b.setReceiver(new MyReceiver("B"));
+        b.connect("A");
 
-         c.send(null, null, "hello world from C");
+        c=createSharedChannel(SINGLETON_2);
+        c.setReceiver(new MyReceiver("C"));
+        c.connect("B");
+
+        c.send(null, null, "hello world from C");
     }
 
- 
 
+    
     public void testReCreationWithSurvivingChannel() throws Exception {
 
         // Create 2 channels sharing a transport
@@ -297,9 +308,37 @@ public class SharedTransportTest extends ChannelTestBase {
         c.connect("A");
     }
 
+    /**
+     * Tests http://jira.jboss.com/jira/browse/JGRP-737
+     * @throws Exception
+     */
+    public void testShutdownOfTimer() throws Exception {
+        a=createSharedChannel(SINGLETON_1);
+        b=createSharedChannel(SINGLETON_1);
+        a.connect("x");
+        b.connect("y");
+        TimeScheduler timer1=a.getProtocolStack().getTransport().getTimer();
+        TimeScheduler timer2=b.getProtocolStack().getTransport().getTimer();
+
+        assert timer1 == timer2;
+
+        assert !timer1.isShutdown();
+        assert !timer2.isShutdown();
+
+        Util.sleep(500);
+        b.close();
+
+        assert !timer2.isShutdown();
+        assert !timer1.isShutdown();
+
+        a.close(); // now, reference counting reaches 0, so the timer thread pool is stopped
+        assert timer2.isShutdown();
+        assert timer1.isShutdown();
+    }
+
 
     /** Create channels A, B and C. Close A. This will close the timer and transports threads (!), so B will
-     * not be able to send messages anymore, so C will not receive any messages 
+     * not be able to send messages anymore, so C will not receive any messages
      * Tests http://jira.jboss.com/jira/browse/JGRP-737 */
     public void testSendingOfMessagesAfterChannelClose() throws ChannelException {
         MyReceiver rec_a=new MyReceiver("A"), rec_b=new MyReceiver("B"), rec_c=new MyReceiver("C");
@@ -330,6 +369,13 @@ public class SharedTransportTest extends ChannelTestBase {
         assertSize(2, rec_b, rec_c);
     }
     
+    /**
+     * Use a CountDownLatch to concurrently connect 3 channels; confirms
+     * the channels connect
+     * 
+     * @throws ChannelException
+     * @throws InterruptedException
+     */
     public void testConcurrentCreation() throws ChannelException, InterruptedException
     {
        a=createSharedChannel(SINGLETON_1);
@@ -370,24 +416,24 @@ public class SharedTransportTest extends ChannelTestBase {
           
           if (connectA.exception != null)
           {
-             fail("connectA threw exception " + connectA.exception);
+             AssertJUnit.fail("connectA threw exception " + connectA.exception);
           }
           if (connectB.exception != null)
           {
-             fail("connectB threw exception " + connectB.exception);
+             AssertJUnit.fail("connectB threw exception " + connectB.exception);
           }
           if (connectC.exception != null)
           {
-             fail("connectC threw exception " + connectC.exception);
+             AssertJUnit.fail("connectC threw exception " + connectC.exception);
           }
           
           if (!finished) {
              if (threadA.isAlive())
-                fail("threadA did not finish");
+                AssertJUnit.fail("threadA did not finish");
              if (threadB.isAlive())
-                fail("threadB did not finish");
+                AssertJUnit.fail("threadB did not finish");
              if (threadC.isAlive())
-                fail("threadC did not finish");
+                AssertJUnit.fail("threadC did not finish");
           }
        }
        finally
@@ -407,10 +453,8 @@ public class SharedTransportTest extends ChannelTestBase {
         }
     }
 
-
-
-    private static JChannel createSharedChannel(String singleton_name) throws ChannelException {
-        ProtocolStackConfigurator config=ConfiguratorFactory.getStackConfigurator(CHANNEL_CONFIG);
+    private JChannel createSharedChannel(String singleton_name) throws ChannelException {
+        ProtocolStackConfigurator config=ConfiguratorFactory.getStackConfigurator(channel_conf);
         ProtocolData[] protocols=config.getProtocolStack();
         ProtocolData transport=protocols[0];
         transport.getParameters().put(Global.SINGLETON_NAME, new ProtocolParameter(Global.SINGLETON_NAME, singleton_name));
@@ -418,6 +462,40 @@ public class SharedTransportTest extends ChannelTestBase {
     }
 
 
+    protected static void makeUnique(Channel channel, int num) throws Exception {
+        ProtocolStack stack=channel.getProtocolStack();
+        TP transport=stack.getTransport();
+        InetAddress bind_addr=transport.getBindAddressAsInetAddress();
+
+        if(transport instanceof UDP) {
+            String mcast_addr=ResourceManager.getNextMulticastAddress();
+            short mcast_port=ResourceManager.getNextMulticastPort(bind_addr);
+            ((UDP)transport).setMulticastAddress(mcast_addr);
+            ((UDP)transport).setMulticastPort(mcast_port);
+        }
+        else if(transport instanceof BasicTCP) {
+            List<Short> ports=ResourceManager.getNextTcpPorts(bind_addr, num);
+            transport.setBindPort(ports.get(0));
+            transport.setPortRange(num);
+
+            Protocol ping=stack.findProtocol(TCPPING.class);
+            if(ping == null)
+                throw new IllegalStateException("TCP stack must consist of TCP:TCPPING - other config are not supported");
+
+            List<String> initial_hosts=new LinkedList<String>();
+            for(short port: ports) {
+                initial_hosts.add(bind_addr + "[" + port + "]");
+            }
+            String tmp=Util.printListWithDelimiter(initial_hosts, ",");
+            ((TCPPING)ping).setInitialHosts(tmp);
+        }
+        else {
+            throw new IllegalStateException("Only UDP and TCP are supported as transport protocols");
+        }
+    }
+
+
+
     private static class MyReceiver extends ReceiverAdapter {
         final List<Message> list=new LinkedList<Message>();
         final String name;
@@ -450,7 +528,7 @@ public class SharedTransportTest extends ChannelTestBase {
         }
 
         public String toString() {
-            return size() + " message(s)";
+            return super.toString() + " (size=" + list.size() + ")";
         }
     }
     
